/*
 * Copyright (c) 2015-2020, Broadcom. All rights reserved.  The term
 * Broadcom refers to Broadcom Limited and/or its subsidiaries.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * BSD license below:
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Author: Eddie Wai <eddie.wai@broadcom.com>
 *
 * Description: DebugFS specifics
 */

#include "bnxt_re.h"
#include "bnxt.h"
#include "debugfs.h"
#include "ib_verbs.h"

#ifdef ENABLE_DEBUGFS

static struct dentry *bnxt_re_debugfs_root;
extern unsigned int restrict_stats;

static void bnxt_re_print_ext_stat(struct bnxt_re_dev *rdev,
				   struct seq_file *s);

static const char *qp_type_str[] = {
	"IB_QPT_SMI",
	"IB_QPT_GSI",
	"IB_QPT_RC",
	"IB_QPT_UC",
	"IB_QPT_UD",
	"IB_QPT_RAW_IPV6",
	"IB_QPT_RAW_ETHERTYPE",
	"IB_QPT_UNKNOWN",
	"IB_QPT_RAW_PACKET",
	"IB_QPT_XRC_INI",
	"IB_QPT_XRC_TGT",
	"IB_QPT_MAX"
};

static const char *qp_state_str[] = {
	"IB_QPS_RESET",
	"IB_QPS_INIT",
	"IB_QPS_RTR",
	"IB_QPS_RTS",
	"IB_QPS_SQD",
	"IB_QPS_SQE",
	"IB_QPS_ERR"
};

static void bnxt_re_fill_qp_info(struct bnxt_re_qp *qp)
{
	struct bnxt_re_dev *rdev = qp->rdev;
	struct bnxt_qplib_qp *qplib_qp;
	u16 type, state;
	u8 *cur_ptr;
	int rc;

	cur_ptr = qp->qp_data;

	qplib_qp = kcalloc(1, sizeof(*qplib_qp), GFP_KERNEL);
	if (!qplib_qp)
		return;

	qplib_qp->id = qp->qplib_qp.id;
	rc = bnxt_qplib_query_qp(&rdev->qplib_res, qplib_qp);
	type = __from_hw_to_ib_qp_type(qp->qplib_qp.type);
	cur_ptr += sprintf(cur_ptr, "type \t = %s(%d)\n",
			   (type > IB_QPT_MAX) ?
			   "IB_QPT_UNKNOWN" : qp_type_str[type],
			   type);
	state =  __to_ib_qp_state(qplib_qp->state);
	cur_ptr += sprintf(cur_ptr, "state \t = %s(%d)\n",
			   (state > IB_QPS_ERR) ?
			   "IB_QPS_UNKNOWN" : qp_state_str[state],
			   state);
	cur_ptr += sprintf(cur_ptr, "source qpn \t = %d\n", qplib_qp->id);
	cur_ptr += sprintf(cur_ptr, "dest qpn \t = %d\n", qplib_qp->dest_qpn);
	cur_ptr += sprintf(cur_ptr, "source port \t = %d\n", qp->qp_info_entry.s_port);
	cur_ptr += sprintf(cur_ptr, "dest port \t = %d\n", qp->qp_info_entry.d_port);
	cur_ptr += sprintf(cur_ptr, "port \t = %d\n", qplib_qp->port_id);
	kfree(qplib_qp);
}

#define BNXT_RE_DEBUGFS_NAME_BUF_SIZE	128

static int bnxt_re_add_stat(char *start, char *pcur,
			    char *name, u64 count, bool hex)
{
        char buff[BNXT_RE_DEBUGFS_NAME_BUF_SIZE] = {0};
        int cpy_len = 0;
	if (hex)
	        snprintf(buff, BNXT_RE_DEBUGFS_NAME_BUF_SIZE, "%s: 0x%llx\n",
			 name, count);
	else
		snprintf(buff, BNXT_RE_DEBUGFS_NAME_BUF_SIZE, "%s: %llu\n",
			 name, count);
        cpy_len = strlen(buff);

        if (pcur + cpy_len > start + BNXT_RE_DBGFS_FILE_MEM) {
                pr_err("%s: No space in stats buff\n", __func__);
                return 0;
        }

        memcpy(pcur, buff, cpy_len);
        return cpy_len;
}

static int bnxt_re_copy_roce_service_fn_id (struct bnxt_re_dev *rdev,
					    struct bnxt_qplib_query_fn_info *fninfo,
					    u32 first_pf, u32 pf_mask, char *start)
{
	struct pci_dev *pdev = rdev->en_dev->pdev;
	char *pcur = start;
	u32 devid = 0;

	devid = PCI_DEVID(pdev->bus->number, pdev->devfn);
	pcur += bnxt_re_add_stat(start, pcur, "pci_devid", devid , true);

	pcur += bnxt_re_add_stat(start, pcur, "is_vf_info", fninfo->vf_valid, true);

	pcur += bnxt_re_add_stat(start, pcur, "first_pf id", first_pf, true);
	pcur += bnxt_re_add_stat(start, pcur, "pf_mask", pf_mask, true);

	return pcur - start;
}

static int bnxt_re_copy_roce_qplist_query(struct bnxt_qplib_query_qp_info *qpinfo,
					  struct bnxt_qplib_query_qp_ext *qp_ext,
					  u32 num_qps, char *start)
{
	int i;
	char *pcur = start;

	pcur += bnxt_re_add_stat(start, pcur, "function_id",
				 qpinfo->function_id, true);
	pcur += bnxt_re_add_stat(start, pcur, "is_vf", qpinfo->vf_valid, true);
	pcur += bnxt_re_add_stat(start, pcur, "qps_requested",
				 qpinfo->num_qps, true);
	pcur += bnxt_re_add_stat(start, pcur, "qps_returned",
				 num_qps, true);
	pcur += bnxt_re_add_stat(start, pcur, "start_offset",
				 qpinfo->start_index, true);

	for (i=0; i < num_qps; i++) {
		pcur += bnxt_re_add_stat(start, pcur, "QP ID ",
					 qp_ext[i].qp_id, false);
		pcur += bnxt_re_add_stat(start, pcur, "QP state",
					 qp_ext[i].state, false);
		pcur += bnxt_re_add_stat(start, pcur, "port id",
					 qp_ext[i].port_id, false);
		pcur += bnxt_re_add_stat(start, pcur, "qkey",
					 qp_ext[i].qkey, false);
		pcur += bnxt_re_add_stat(start, pcur, "dgid[0]",
					 qp_ext[i].dgid[0], false);
		pcur += bnxt_re_add_stat(start, pcur, "dgid[1]",
					 qp_ext[i].dgid[1], false);
		pcur += bnxt_re_add_stat(start, pcur, "dgid[2]",
					 qp_ext[i].dgid[2], false);
		pcur += bnxt_re_add_stat(start, pcur, "dgid[3]",
					 qp_ext[i].dgid[3], false);
		pcur += bnxt_re_add_stat(start, pcur, "dest_qp_id",
					 qp_ext[i].dest_qp_id, false);
		pcur += bnxt_re_add_stat(start, pcur, "sgid_index",
					 qp_ext[i].sgid_index, false);
		pcur += bnxt_re_add_stat(start, pcur, "network_type",
					 qp_ext[i].network_type, false);
		pcur += bnxt_re_add_stat(start, pcur, "stat_collection_id",
					 qp_ext[i].stat_collection_id, false);
	}

	return pcur - start;
}

static int bnxt_re_copy_roce_stat(struct bnxt_qplib_query_stats_info *sinfo,
				      struct bnxt_qplib_roce_stats *stats,
				      struct bnxt_re_flow_counters *flow_stats,
				      char *start)
{
	char *pcur = start;

	pcur += bnxt_re_add_stat(start, pcur, "function_id",
				 sinfo->function_id, true);
	pcur += bnxt_re_add_stat(start, pcur, "is_vf", sinfo->vf_valid, true);
	pcur += bnxt_re_add_stat(start, pcur, "collection_id",
				 sinfo->collection_id, true);
	pcur += bnxt_re_add_stat(start, pcur, "roce_rx_pkts",
				 flow_stats->ro_stats.rx_pkts, false);
	pcur += bnxt_re_add_stat(start, pcur, "roce_rx_bytes",
				 flow_stats->ro_stats.rx_bytes, false);
	pcur += bnxt_re_add_stat(start, pcur, "roce_tx_pkts",
				 flow_stats->ro_stats.tx_pkts, false);
	pcur += bnxt_re_add_stat(start, pcur, "roce_tx_bytes",
				 flow_stats->ro_stats.tx_bytes, false);
	pcur += bnxt_re_add_stat(start, pcur, "cnp_rx_pkts",
				 flow_stats->cnp_stats.cnp_rx_pkts, false);
	pcur += bnxt_re_add_stat(start, pcur, "cnp_rx_bytes",
				 flow_stats->cnp_stats.cnp_rx_bytes, false);
	pcur += bnxt_re_add_stat(start, pcur, "cnp_tx_pkts",
				 flow_stats->cnp_stats.cnp_tx_pkts, false);
	pcur += bnxt_re_add_stat(start, pcur, "cnp_tx_bytes",
				 flow_stats->cnp_stats.cnp_tx_bytes, false);
	pcur += bnxt_re_add_stat(start, pcur, "to_retransmits",
				 stats->to_retransmits, false);
	pcur += bnxt_re_add_stat(start, pcur, "seq_err_naks_rcvd",
				 stats->seq_err_naks_rcvd, false);
	pcur += bnxt_re_add_stat(start, pcur, "max_retry_exceeded",
				 stats->max_retry_exceeded, false);
	pcur += bnxt_re_add_stat(start, pcur, "rnr_naks_rcvd",
				 stats->rnr_naks_rcvd, false);
	pcur += bnxt_re_add_stat(start, pcur, "missing_resp",
				 stats->missing_resp, false);
	pcur += bnxt_re_add_stat(start, pcur, "unrecoverable_err",
				 stats->unrecoverable_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "bad_resp_err",
				 stats->bad_resp_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "local_qp_op_err",
				 stats->local_qp_op_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "local_protection_err",
				 stats->local_protection_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "mem_mgmt_op_err",
				 stats->mem_mgmt_op_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "remote_invalid_req_err",
				 stats->remote_invalid_req_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "remote_access_err",
				 stats->remote_access_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "remote_op_err",
				 stats->remote_op_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "dup_req",
				 stats->dup_req, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_exceed_max",
				 stats->res_exceed_max, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_length_mismatch",
				 stats->res_length_mismatch, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_exceeds_wqe",
				 stats->res_exceeds_wqe, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_opcode_err",
				 stats->res_opcode_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_rx_invalid_rkey",
				 stats->res_rx_invalid_rkey, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_rx_domain_err",
				 stats->res_rx_domain_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_rx_no_perm",
				 stats->res_rx_no_perm, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_rx_range_err",
				 stats->res_rx_range_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_tx_invalid_rkey",
				 stats->res_tx_invalid_rkey, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_tx_domain_err",
				 stats->res_tx_domain_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_tx_no_perm",
				 stats->res_tx_no_perm, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_tx_range_err",
				 stats->res_tx_range_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_irrq_oflow",
				 stats->res_irrq_oflow, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_unsup_opcode",
				 stats->res_unsup_opcode, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_unaligned_atomic",
				 stats->res_unaligned_atomic, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_rem_inv_err",
				 stats->res_rem_inv_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_mem_error64",
				 stats->res_mem_error, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_srq_err",
				 stats->res_srq_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_cmp_err",
				 stats->res_cmp_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_invalid_dup_rkey",
				 stats->res_invalid_dup_rkey, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_wqe_format_err",
				 stats->res_wqe_format_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_cq_load_err",
				 stats->res_cq_load_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_srq_load_err",
				 stats->res_srq_load_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_tx_pci_err",
				 stats->res_tx_pci_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_rx_pci_err",
				 stats->res_rx_pci_err, false);
	pcur += bnxt_re_add_stat(start, pcur, "res_oos_drop_count",
				 stats->res_oos_drop_count, false);
	pcur += bnxt_re_add_stat(start, pcur, "active_qp_count_p0",
				 stats->active_qp_count_p0, false);
	pcur += bnxt_re_add_stat(start, pcur, "active_qp_count_p1",
				 stats->active_qp_count_p1, false);
	pcur += bnxt_re_add_stat(start, pcur, "active_qp_count_p2",
				 stats->active_qp_count_p2, false);
	pcur += bnxt_re_add_stat(start, pcur, "active_qp_count_p3",
				 stats->active_qp_count_p3, false);

	return pcur - start;
}


#define BNXT_RE_QP_QUERY_MAX_QPS_QUERIED	300
#define BNXT_RE_QP_INFO_SINGLE_QUERY_MAX_COUNT	20

static ssize_t bnxt_re_dbgfs_ops_read(struct file *filp, char __user *buffer,
                                      size_t usr_buf_len, loff_t *ppos)
{
	struct bnxt_qplib_query_qp_ext *qp_ext = NULL, *qp_ext_ptr = NULL;
	u32 rem_qps = 0, qp_to_query =0, qp_queried = 0, ret_count = 0;
	struct bnxt_re_dbg_file *dbg_file = filp->private_data;
        struct bnxt_re_dev *rdev = dbg_file->rdev;
	struct bnxt_qplib_roce_stats stats =  {0};
	struct bnxt_re_flow_counters flow_stats;
        char *data = dbg_file->dbg_buf;
	u32 first_pf = 0, pf_mask = 0;
	u32 current_index = 0;
        ssize_t status = 0;

	memset(data, 0, BNXT_RE_DBGFS_FILE_MEM);
        switch (dbg_file->type) {
		case BNXT_RE_STATS_QUERY:
			dev_dbg(rdev_to_dev(rdev), "Stats query");
			memset(&flow_stats, 0, sizeof(flow_stats));
			if (bnxt_qplib_get_roce_error_stats
				(&rdev->rcfw, &stats, &dbg_file->params.sinfo))
				goto exit;
			if (bnxt_re_get_flow_stats_from_service_pf
				(rdev, &flow_stats,  &dbg_file->params.sinfo))
				goto exit;
			bnxt_re_copy_roce_stat(&dbg_file->params.sinfo,
					       &stats, &flow_stats, data);
			break;
		case BNXT_RE_QP_QUERY:
			dev_dbg(rdev_to_dev(rdev), "QP information query");
			qp_ext = kzalloc(sizeof(*qp_ext) *
					 dbg_file->params.qpinfo.num_qps,
					 GFP_KERNEL);
			qp_ext_ptr = qp_ext;
			rem_qps = dbg_file->params.qpinfo.num_qps;
			current_index = dbg_file->params.qpinfo.start_index;
			while (rem_qps > 0) {
				if (rem_qps >
				    BNXT_RE_QP_INFO_SINGLE_QUERY_MAX_COUNT) {
					qp_to_query =
					BNXT_RE_QP_INFO_SINGLE_QUERY_MAX_COUNT;
					rem_qps -=
					BNXT_RE_QP_INFO_SINGLE_QUERY_MAX_COUNT;
				} else {
					qp_to_query = rem_qps;
					rem_qps = 0;
				}
				if (bnxt_qplib_query_qp_ext_fn
					(&rdev->qplib_res,
					 &dbg_file->params.qpinfo,
					 qp_to_query,
					 qp_ext_ptr,
					 &current_index,
					 &ret_count))
					break;
				qp_ext_ptr += ret_count;
				qp_queried += ret_count;
				if (ret_count != qp_to_query)
					break;
			}

			bnxt_re_copy_roce_qplist_query(&dbg_file->params.qpinfo,
						       qp_ext, qp_queried,
						       data);
			kfree(qp_ext);
			break;
		case BNXT_RE_SERVICE_FN_QUERY:
			dev_dbg(rdev_to_dev(rdev), "Service Function query");
			/* Issue command to query the function */
			if (!dbg_file->params.fninfo.vf_valid)
				if (bnxt_re_host_pf_id_query
					(rdev, &dbg_file->params.fninfo,
					 &pf_mask, &first_pf))
					break;
			bnxt_re_copy_roce_service_fn_id
				(rdev, &dbg_file->params.fninfo,
				 first_pf, pf_mask, data);
			break;
		default:
			break;
        }

        if (usr_buf_len < strlen(data)) {
                status = -ENOSPC;
                goto exit;
        }

        status = simple_read_from_buffer(buffer, usr_buf_len, ppos, data,
                                         strlen(data));
exit:
        return status;
}

#define BNXT_RE_MAX_COLLECTION_ID	16
#define MAX_SPLIT_TLEN			32
#define MAX_TOKENS			8
int __split_string(char out[][MAX_SPLIT_TLEN], char *in_buf)
{
	u32 nstr = 0;
	char *token;
	size_t tlen;

	while (((token = strsep(&in_buf, " "))) != NULL && nstr < MAX_TOKENS) {
		tlen = strlen(token);
		if (tlen > MAX_SPLIT_TLEN)
			return -EINVAL;
		strcpy(out[nstr], token);
		nstr++;
	}

	return nstr;
}

static ssize_t bnxt_re_dbgfs_ops_write(struct file *filp,
				       const char __user *buffer,
				       size_t count, loff_t *ppos)
{
	struct bnxt_re_dbg_file *dbg_file = filp->private_data;
	long val1 = 0, val2 = 0, val3 = 0, val4 = 0;
	char in_buf[BNXT_RE_DEBUGFS_NAME_BUF_SIZE];
	char tokens[MAX_TOKENS][MAX_SPLIT_TLEN];
	int ntok = 0;

	if (*ppos != 0 || count == 0 || count > BNXT_RE_DEBUGFS_NAME_BUF_SIZE)
		goto err;

	if (copy_from_user(in_buf, buffer, count))
		goto err;
	in_buf[count - 1] = '\0';
	ntok = __split_string(tokens, in_buf);
	switch (ntok) {
		case 4:
		if (kstrtol(tokens[3], 16, &val4))
			goto err;
		/* fall-thru */
		case 3:
		if (kstrtol(tokens[2], 16, &val3))
			goto err;
		/* fall-thru */
		case 2:
		if (kstrtol(tokens[1], 16, &val2))
			goto err;
		/* fall-thru */
		case 1:
		if (kstrtol(tokens[0], 16, &val1))
			goto err;
		break;
		default:
			goto err;
	}
	switch (dbg_file->type) {
		case BNXT_RE_STATS_QUERY:
			dbg_file->params.sinfo.function_id = (u32)val1;
			if (val2 >= BNXT_RE_MAX_COLLECTION_ID && val2 != 0xFF)
				/* Set the collection id value to default
				 * if the value is out of range
				 */
				val2 = 0xFF;

			dbg_file->params.sinfo.collection_id = (u8)val2;
			dbg_file->params.sinfo.vf_valid = (bool)val3;
			break;
		case BNXT_RE_QP_QUERY:
			dbg_file->params.qpinfo.function_id = (u32)val1;
			/* Restrict the max number of qps to
			 * BNXT_RE_QP_QUERY_MAX_QPS_QUERIEDas the
			 * debugfs buf size can hold only that much info
			 */
			if (val2 > BNXT_RE_QP_QUERY_MAX_QPS_QUERIED)
				val2 = BNXT_RE_QP_QUERY_MAX_QPS_QUERIED;
			dbg_file->params.qpinfo.num_qps = (u32) val2;
			dbg_file->params.qpinfo.vf_valid = (bool)val3;
			dbg_file->params.qpinfo.start_index = (u32)val4;
			break;
		case BNXT_RE_SERVICE_FN_QUERY:
			dbg_file->params.fninfo.vf_valid = (bool)val1;
			dbg_file->params.fninfo.host = (u32)val2;
			dbg_file->params.fninfo.filter = (u32)val3;
			break;
		default:
			goto err;
	}

	return count;
err:
	return -EFAULT;
}

static const struct file_operations bnxt_re_file_ops = {
        .owner = THIS_MODULE,
        .open = simple_open,
        .read = bnxt_re_dbgfs_ops_read,
        .write = bnxt_re_dbgfs_ops_write
};

static void bnxt_re_add_stats_query(struct bnxt_re_dev *rdev)
{
	struct bnxt_re_debug_entries *dbg_ent = rdev->dbg_ent;

	dbg_ent->stat_file.rdev = rdev;
	dbg_ent->stat_file.type = BNXT_RE_STATS_QUERY;
	dbg_ent->stat_file.params.sinfo.function_id  = 0xFFFFFFFF;
	dbg_ent->stat_file.params.sinfo.collection_id  = 0xFF;
	dbg_ent->stat_file.params.sinfo.vf_valid = 0;

	dbg_ent->stat_query =
		debugfs_create_file("stat_query", S_IRUSR,
				    rdev->dbg_ent->service_fn_dir,
				    &dbg_ent->stat_file,
				    &bnxt_re_file_ops);
	dbg_ent->qp_file.rdev = rdev;
	dbg_ent->qp_file.type = BNXT_RE_QP_QUERY;
	dbg_ent->qp_file.params.qpinfo.function_id  = 0xFFFFFFFF;
	dbg_ent->qp_file.params.qpinfo.num_qps = 0xFF;
	dbg_ent->qp_file.params.qpinfo.vf_valid = 0;
	dbg_ent->qp_file.params.qpinfo.start_index = 0;
	dbg_ent->qplist_query =
		debugfs_create_file("qplist_query", S_IRUSR,
				    rdev->dbg_ent->service_fn_dir,
				    &dbg_ent->qp_file,
				    &bnxt_re_file_ops);
	dbg_ent->service_fn_file.rdev = rdev;
	dbg_ent->service_fn_file.type = BNXT_RE_SERVICE_FN_QUERY;
	dbg_ent->service_fn_file.params.fninfo.vf_valid = 0;
	dbg_ent->service_fn_file.params.fninfo.filter = 0; /* Query all devices */
	dbg_ent->service_fn_file.params.fninfo.host = 0; /* Query default host*/
	dbg_ent->service_fn_query =
		debugfs_create_file("service_fn_query", S_IRUSR,
				    rdev->dbg_ent->service_fn_dir,
				    &dbg_ent->service_fn_file,
				    &bnxt_re_file_ops);
}

static ssize_t bnxt_re_qp_info_qp_read(struct file *filp, char __user *buffer,
				       size_t usr_buf_len, loff_t *ppos)
{
	struct bnxt_re_qp *qp = filp->private_data;

	if (usr_buf_len < BNXT_RE_DEBUGFS_QP_INFO_MAX_SIZE)
		return -ENOSPC;

	if (!qp->qp_data)
		return -ENOMEM;

	bnxt_re_fill_qp_info(qp);

	return simple_read_from_buffer(buffer, usr_buf_len, ppos,
				       (u8 *)(qp->qp_data),
				       BNXT_RE_DEBUGFS_QP_INFO_MAX_SIZE);
}

static const struct file_operations bnxt_re_qp_info_ops = {
	.owner = THIS_MODULE,
	.open = simple_open,
	.read = bnxt_re_qp_info_qp_read,
};

void bnxt_re_qp_info_add_qpinfo(struct bnxt_re_dev *rdev, struct bnxt_re_qp *qp)
{
	char qp_name[32];

	sprintf(qp_name, "0x%x", qp->qplib_qp.id);
	if (rdev->dbg_ent && rdev->dbg_ent->qpinfo_dir)
		qp->qp_info_dentry = debugfs_create_file(qp_name, S_IRUSR,
						     rdev->dbg_ent->qpinfo_dir, qp,
						     &bnxt_re_qp_info_ops);

	qp->qp_data = kzalloc(BNXT_RE_DEBUGFS_QP_INFO_MAX_SIZE, GFP_KERNEL);
}

void bnxt_re_qp_info_rem_qpinfo(struct bnxt_re_dev *rdev, struct bnxt_re_qp *qp)
{
	kfree(qp->qp_data);
	qp->qp_data = NULL;

	if(qp->qp_info_dentry)
		debugfs_remove(qp->qp_info_dentry);
	qp->qp_info_dentry = NULL;
}

static ssize_t bnxt_re_debugfs_clear(struct file *fil, const char __user *u,
				     size_t size, loff_t *off)
{
	/* TODO: Add support for debugfs parameter clearing */
	return size;
}

static void bnxt_re_print_roce_only_counters(struct bnxt_re_dev *rdev,
					     struct seq_file *s)
{
	struct bnxt_re_ro_counters *roce_only = &rdev->stats.dstat.cur[0];

	seq_printf(s, "\tRoCE Only Rx Pkts: %llu\n", roce_only->rx_pkts);
	seq_printf(s, "\tRoCE Only Rx Bytes: %llu\n", roce_only->rx_bytes);
	seq_printf(s, "\tRoCE Only Tx Pkts: %llu\n", roce_only->tx_pkts);
	seq_printf(s, "\tRoCE Only Tx Bytes: %llu\n", roce_only->tx_bytes);
}
static void bnxt_re_print_l2_only_counters(struct bnxt_re_dev *rdev,
					     struct seq_file *s)
{
	seq_printf(s, "\tL2 Only Rx Pkts: %llu\n",
		   rdev->stats.l2stat[0].rx_packets);
	seq_printf(s, "\tL2 Only Rx Bytes: %llu\n",
		   rdev->stats.l2stat[0].rx_bytes);
	seq_printf(s, "\tL2 Only Tx Pkts: %llu\n",
		   rdev->stats.l2stat[0].tx_packets);
	seq_printf(s, "\tL2 Only Tx Bytes: %llu\n",
		   rdev->stats.l2stat[0].tx_bytes);
}

static void bnxt_re_print_bond_l2_only_counters(struct bnxt_re_dev *rdev,
						struct seq_file *s)
{
	struct bnxt_re_l2_counters *l2stat;

	l2stat = rdev->stats.l2stat;
	seq_printf(s, "\tL2 Only Rx Pkts: %llu\n",
		   l2stat[0].rx_packets + l2stat[1].rx_packets);
	seq_printf(s, "\tL2 Only Rx Bytes: %llu\n",
		   l2stat[0].rx_bytes + l2stat[1].rx_bytes);
	seq_printf(s, "\tL2 Only Tx Pkts: %llu\n",
		   l2stat[0].tx_packets + l2stat[1].tx_packets);
	seq_printf(s, "\tL2 Only Tx Bytes: %llu\n",
		   l2stat[0].tx_bytes + l2stat[1].tx_bytes);

	seq_printf(s, "\tL2 Only Rx Pkts P0: %llu\n", l2stat[0].rx_packets);
	seq_printf(s, "\tL2 Only Rx Bytes P0: %llu\n", l2stat[0].rx_bytes);
	seq_printf(s, "\tL2 Only Tx Pkts P0: %llu\n", l2stat[0].tx_packets);
	seq_printf(s, "\tL2 Only Tx Bytes P0: %llu\n", l2stat[0].tx_bytes);

	seq_printf(s, "\tL2 Only Rx Pkts P1: %llu\n", l2stat[1].rx_packets);
	seq_printf(s, "\tL2 Only Rx Bytes P1: %llu\n", l2stat[1].rx_bytes);
	seq_printf(s, "\tL2 Only Tx Pkts P1: %llu\n", l2stat[1].tx_packets);
	seq_printf(s, "\tL2 Only Tx Bytes P1: %llu\n", l2stat[1].tx_bytes);
}

static void bnxt_re_print_normal_total_counters(struct bnxt_re_dev *rdev,
					      struct seq_file *s)
{
	struct bnxt_re_ro_counters *roce_only;
	struct bnxt_re_l2_counters *l2stat;
	struct bnxt_re_cc_stat *cnps;

	cnps = &rdev->stats.cnps;
	roce_only = &rdev->stats.dstat.cur[0];
	l2stat = rdev->stats.l2stat;

	seq_printf(s, "\tRx Pkts: %llu\n", cnps->cur[0].cnp_rx_pkts +
					   roce_only->rx_pkts);

	seq_printf(s, "\tRx Bytes: %llu\n",
		   cnps->cur[0].cnp_rx_bytes + roce_only->rx_bytes);

	seq_printf(s, "\tTx Pkts: %llu\n",
		   cnps->cur[0].cnp_tx_pkts + roce_only->tx_pkts);

	seq_printf(s, "\tTx Bytes: %llu\n",
		   cnps->cur[0].cnp_tx_bytes + roce_only->tx_bytes);

	seq_printf(s, "\tTotal Rx Good Bytes: %llu\n",
		   cnps->cur[0].cnp_rx_bytes + roce_only->rx_bytes +
		   l2stat[0].rx_bytes);

	seq_printf(s, "\tTotal Tx Good Bytes: %llu\n",
		   cnps->cur[0].cnp_tx_bytes + roce_only->tx_bytes +
		   l2stat[0].tx_bytes);
}

static void bnxt_re_print_bond_total_counters(struct bnxt_re_dev *rdev,
					      struct seq_file *s)
{
	struct bnxt_re_ro_counters *roce_only;
	struct bnxt_re_cc_stat *cnps;

	cnps = &rdev->stats.cnps;
	roce_only = &rdev->stats.dstat.cur[0];

	seq_printf(s, "\tRx Pkts: %llu\n",
		   cnps->cur[0].cnp_rx_pkts +
		   cnps->cur[1].cnp_rx_pkts +
		   roce_only[0].rx_pkts +
		   roce_only[1].rx_pkts);

	seq_printf(s, "\tRx Bytes: %llu\n",
		   cnps->cur[0].cnp_rx_bytes +
		   cnps->cur[1].cnp_rx_bytes +
		   roce_only[0].rx_bytes +
		   roce_only[1].rx_bytes);

	seq_printf(s, "\tTx Pkts: %llu\n",
		   cnps->cur[0].cnp_tx_pkts +
		   cnps->cur[1].cnp_tx_pkts +
		   roce_only[0].tx_pkts +
		   roce_only[1].tx_pkts);

	seq_printf(s, "\tTx Bytes: %llu\n",
		   cnps->cur[0].cnp_tx_bytes +
		   cnps->cur[1].cnp_tx_bytes +
		   roce_only[0].tx_bytes +
		   roce_only[1].tx_bytes);

	/* Disable per port stat display for gen-p5 B0 onwards. */
	if (_is_chip_gen_p5(rdev->chip_ctx) && !_is_chip_a0(rdev->chip_ctx))
		return;
	seq_printf(s, "\tRx Pkts P0: %llu\n",
		   cnps->cur[0].cnp_rx_pkts + roce_only[0].rx_pkts);
	seq_printf(s, "\tRx Bytes P0: %llu\n",
		   cnps->cur[0].cnp_rx_bytes + roce_only[0].rx_bytes);
	seq_printf(s, "\tTx Pkts P0: %llu\n",
		   cnps->cur[0].cnp_tx_pkts + roce_only[0].tx_pkts);
	seq_printf(s, "\tTx Bytes P0: %llu\n",
		   cnps->cur[0].cnp_tx_bytes + roce_only[0].tx_bytes);

	seq_printf(s, "\tRx Pkts P1: %llu\n",
		   cnps->cur[1].cnp_rx_pkts + roce_only[1].rx_pkts);
	seq_printf(s, "\tRx Bytes P1: %llu\n",
		   cnps->cur[1].cnp_rx_bytes + roce_only[1].rx_bytes);
	seq_printf(s, "\tTx Pkts P1: %llu\n",
		   cnps->cur[1].cnp_tx_pkts + roce_only[1].tx_pkts);
	seq_printf(s, "\tTx Bytes P1: %llu\n",
		   cnps->cur[1].cnp_tx_bytes + roce_only[1].tx_bytes);
}

static void bnxt_re_print_bond_roce_only_counters(struct bnxt_re_dev *rdev,
					     struct seq_file *s)
{
	struct bnxt_re_ro_counters *roce_only;

	roce_only = rdev->stats.dstat.cur;
	seq_printf(s, "\tRoCE Only Rx Pkts: %llu\n" ,roce_only[0].rx_pkts +
			roce_only[1].rx_pkts);
	seq_printf(s, "\tRoCE Only Rx Bytes: %llu\n", roce_only[0].rx_bytes +
			roce_only[1].rx_bytes);
	seq_printf(s, "\tRoCE Only Tx Pkts: %llu\n", roce_only[0].tx_pkts +
			roce_only[1].tx_pkts);
	seq_printf(s, "\tRoCE Only Tx Bytes: %llu\n", roce_only[0].tx_bytes +
			roce_only[1].tx_bytes);

	/* Disable per port stat display for gen-p5 B0 onwards. */
	if (_is_chip_gen_p5(rdev->chip_ctx) && !_is_chip_a0(rdev->chip_ctx))
		return;
	seq_printf(s, "\tRoCE Only Rx Pkts P0: %llu\n", roce_only[0].rx_pkts);
	seq_printf(s, "\tRoCE Only Rx Bytes P0: %llu\n", roce_only[0].rx_bytes);
	seq_printf(s, "\tRoCE Only Tx Pkts P0: %llu\n", roce_only[0].tx_pkts);
	seq_printf(s, "\tRoCE Only Tx Bytes P0: %llu\n", roce_only[0].tx_bytes);

	seq_printf(s, "\tRoCE Only Rx Pkts P1: %llu\n", roce_only[1].rx_pkts);
	seq_printf(s, "\tRoCE Only Rx Bytes P1: %llu\n", roce_only[1].rx_bytes);
	seq_printf(s, "\tRoCE Only Tx Pkts P1: %llu\n", roce_only[1].tx_pkts);
	seq_printf(s, "\tRoCE Only Tx Bytes P1: %llu\n", roce_only[1].tx_bytes);
}


static void bnxt_re_print_bond_total_good_bytes_counters
		(struct bnxt_re_dev *rdev, struct seq_file *s)
{
	struct bnxt_re_ro_counters *roce_only;
	struct bnxt_re_l2_counters *l2stat;
	struct bnxt_re_cc_stat *cnps;

	l2stat = rdev->stats.l2stat;
	roce_only = rdev->stats.dstat.cur;
	cnps = &rdev->stats.cnps;

	seq_printf(s, "\tTotal Rx Good Bytes: %llu\n",
		   cnps->cur[0].cnp_rx_bytes + cnps->cur[1].cnp_rx_bytes +
		   roce_only[0].rx_bytes + roce_only[1].rx_bytes +
		   l2stat[0].rx_bytes + l2stat[1].rx_bytes);

	seq_printf(s, "\tTotal Tx Good Bytes: %llu\n",
		   cnps->cur[0].cnp_tx_bytes + cnps->cur[1].cnp_tx_bytes +
		   roce_only[0].tx_bytes + roce_only[1].tx_bytes +
		   l2stat[0].tx_bytes + l2stat[1].tx_bytes);

	/* Disable per port stat display for gen-p5 B0 onwards. */
	if (_is_chip_gen_p5(rdev->chip_ctx) && !_is_chip_a0(rdev->chip_ctx))
		return;

	seq_printf(s, "\tTotal Rx Good Bytes P0: %llu\n",
		   cnps->cur[0].cnp_rx_bytes + roce_only[0].rx_bytes +
		   l2stat[0].rx_bytes);

	seq_printf(s, "\tTotal Tx Good Bytes P0: %llu\n",
		   cnps->cur[0].cnp_tx_bytes + roce_only[0].tx_bytes +
		   l2stat[0].tx_bytes);

	seq_printf(s, "\tTotal Rx Good Bytes P1: %llu\n",
		   cnps->cur[1].cnp_rx_bytes + roce_only[1].rx_bytes +
		   l2stat[1].rx_bytes);

	seq_printf(s, "\tTotal Tx Good Bytes P1: %llu\n",
		   cnps->cur[1].cnp_tx_bytes + roce_only[1].tx_bytes +
		   l2stat[1].tx_bytes);
}

static void bnxt_re_print_bond_counters(struct bnxt_re_dev *rdev,
					struct seq_file *s)
{
	struct bnxt_qplib_roce_stats *roce_stats;
	struct bnxt_re_rdata_counters *stats1;
	struct bnxt_re_rdata_counters *stats2;
	struct bnxt_re_cc_stat *cnps;
	long long oob_cnt = 0;
	bool is_g5a0, en_disp;

	roce_stats = &rdev->stats.dstat.errs;
	stats1 = &rdev->stats.dstat.rstat[0];
	stats2 = &rdev->stats.dstat.rstat[1];
	cnps = &rdev->stats.cnps;
	is_g5a0 = _is_chip_gen_p5(rdev->chip_ctx) && _is_chip_a0(rdev->chip_ctx);
	en_disp = !_is_chip_gen_p5(rdev->chip_ctx) || is_g5a0;

	seq_printf(s, "\tActive QPs P0: %lld\n", roce_stats->active_qp_count_p0);
	seq_printf(s, "\tActive QPs P1: %lld\n", roce_stats->active_qp_count_p1);

	bnxt_re_print_bond_total_counters(rdev, s);

	seq_printf(s, "\tCNP Tx Pkts: %llu\n",
		   cnps->cur[0].cnp_tx_pkts + cnps->cur[1].cnp_tx_pkts);
	if (en_disp)
		seq_printf(s, "\tCNP Tx Bytes: %llu\n",
			   cnps->cur[0].cnp_tx_bytes +
			   cnps->cur[1].cnp_tx_bytes);
	seq_printf(s, "\tCNP Rx Pkts: %llu\n",
		   cnps->cur[0].cnp_rx_pkts + cnps->cur[1].cnp_rx_pkts);
	if (en_disp)
		seq_printf(s, "\tCNP Rx Bytes: %llu\n",
			   cnps->cur[0].cnp_rx_bytes +
			   cnps->cur[1].cnp_rx_bytes);

	seq_printf(s, "\tCNP Tx Pkts P0: %llu\n", cnps->cur[0].cnp_tx_pkts);
	if (en_disp)
		seq_printf(s, "\tCNP Tx Bytes P0: %llu\n",
			   cnps->cur[0].cnp_tx_bytes);
	seq_printf(s, "\tCNP Rx Pkts P0: %llu\n", cnps->cur[0].cnp_rx_pkts);
	if (en_disp)
		seq_printf(s, "\tCNP Rx Bytes P0: %llu\n",
			   cnps->cur[0].cnp_rx_bytes);
	seq_printf(s, "\tCNP Tx Pkts P1: %llu\n", cnps->cur[1].cnp_tx_pkts);
	if (en_disp)
		seq_printf(s, "\tCNP Tx Bytes P1: %llu\n",
			   cnps->cur[1].cnp_tx_bytes);
	seq_printf(s, "\tCNP Rx Pkts P1: %llu\n", cnps->cur[1].cnp_rx_pkts);
	if (en_disp)
		seq_printf(s, "\tCNP Rx Bytes P1: %llu\n",
			   cnps->cur[1].cnp_rx_bytes);
	bnxt_re_print_bond_l2_only_counters(rdev, s);
	/* Print RoCE only bytes.. CNP counters include RoCE packets also */
	bnxt_re_print_bond_roce_only_counters(rdev, s);
	bnxt_re_print_bond_total_good_bytes_counters(rdev, s);


	seq_printf(s, "\trx_roce_error_pkts: %lld\n",
		   (stats1 ? stats1->rx_error_pkts : 0) +
		   (stats2 ? stats2->rx_error_pkts : 0));
	seq_printf(s, "\trx_roce_discard_pkts: %lld\n",
		   (stats1 ? stats1->rx_discard_pkts : 0) +
		   (stats2 ? stats2->rx_discard_pkts : 0));
	if (!en_disp) {
		/* show only for Gen P5 or higher */
		seq_printf(s, "\ttx_roce_error_pkts: %lld\n",
			   (stats1 ? stats1->tx_error_pkts : 0) +
			   (stats2 ? stats2->tx_error_pkts : 0));
		seq_printf(s, "\ttx_roce_discard_pkts: %lld\n",
			   (stats1 ? stats1->tx_discard_pkts : 0) +
			   (stats2 ? stats2->tx_discard_pkts : 0));
	}
	/* No need to sum-up both port stat counts in bond mode */
	if (_is_ext_stats_supported(rdev->dev_attr->dev_cap_flags) &&
	    !rdev->is_virtfn) {
		seq_printf(s, "\tres_oob_drop_count: %lld\n",
			   rdev->stats.dstat.e_errs.oob);
		bnxt_re_print_ext_stat(rdev, s);
	} else {
		oob_cnt = (stats1 ? stats1->rx_discard_pkts : 0) +
			(stats2 ? stats2->rx_discard_pkts : 0) -
			rdev->stats.dstat.errs.res_oos_drop_count;

		/*
		 * oob count is calculated from the output of two seperate
		 * HWRM commands. To avoid reporting inconsistent values
		 * due to the time delta between two different queries,
		 * report newly calculated value only if it is more than the
		 * previously reported OOB value.
		 */
		if (oob_cnt < rdev->stats.dstat.prev_oob)
			oob_cnt = rdev->stats.dstat.prev_oob;
		seq_printf(s, "\tres_oob_drop_count: %lld\n", oob_cnt);
		rdev->stats.dstat.prev_oob = oob_cnt;
	}
}

static void bnxt_re_print_ext_stat(struct bnxt_re_dev *rdev,
				   struct seq_file *s)
{
	struct bnxt_re_cnp_counters *cnp;
	struct bnxt_re_ext_rstat *ext_s;

	ext_s = &rdev->stats.dstat.ext_rstat[0];
	cnp = &rdev->stats.cnps.cur[0];

	seq_printf(s, "\ttx_atomic_req: %llu\n", ext_s->tx.atomic_req);
	seq_printf(s, "\trx_atomic_req: %llu\n", ext_s->rx.atomic_req);
	seq_printf(s, "\ttx_read_req: %llu\n", ext_s->tx.read_req);
	seq_printf(s, "\ttx_read_resp: %llu\n", ext_s->tx.read_resp);
	seq_printf(s, "\trx_read_req: %llu\n", ext_s->rx.read_req);
	seq_printf(s, "\trx_read_resp: %llu\n", ext_s->rx.read_resp);
	seq_printf(s, "\ttx_write_req: %llu\n", ext_s->tx.write_req);
	seq_printf(s, "\trx_write_req: %llu\n", ext_s->rx.write_req);
	seq_printf(s, "\ttx_send_req: %llu\n", ext_s->tx.send_req);
	seq_printf(s, "\trx_send_req: %llu\n", ext_s->rx.send_req);
	seq_printf(s, "\trx_good_pkts: %llu\n", ext_s->grx.rx_pkts);
	seq_printf(s, "\trx_good_bytes: %llu\n", ext_s->grx.rx_bytes);
	seq_printf(s, "\trx_ecn_marked_pkts: %llu\n", cnp->ecn_marked);
}

static void bnxt_re_print_normal_counters(struct bnxt_re_dev *rdev,
					  struct seq_file *s)
{
	struct bnxt_re_rdata_counters *stats;
	struct bnxt_re_cc_stat *cnps;
	bool is_g5a0, en_disp;

	stats = &rdev->stats.dstat.rstat[0];
	cnps = &rdev->stats.cnps;
	is_g5a0 = _is_chip_gen_p5(rdev->chip_ctx) &&
		  _is_chip_a0(rdev->chip_ctx);
	en_disp = !_is_chip_gen_p5(rdev->chip_ctx) || is_g5a0;

	bnxt_re_print_normal_total_counters(rdev, s);
	if (!rdev->is_virtfn) {
		seq_printf(s, "\tCNP Tx Pkts: %llu\n",
			   cnps->cur[0].cnp_tx_pkts);
		if (en_disp)
			seq_printf(s, "\tCNP Tx Bytes: %llu\n",
				   cnps->cur[0].cnp_tx_bytes);
		seq_printf(s, "\tCNP Rx Pkts: %llu\n",
			   cnps->cur[0].cnp_rx_pkts);
		if (en_disp)
			seq_printf(s, "\tCNP Rx Bytes: %llu\n",
				   cnps->cur[0].cnp_rx_bytes);
	}
	/* Print L2 only counters */
	bnxt_re_print_l2_only_counters(rdev, s);
	/* Print RoCE only bytes.. CNP counters include RoCE packets also */
	bnxt_re_print_roce_only_counters(rdev, s);

	seq_printf(s, "\trx_roce_error_pkts: %lld\n",
		   stats ? stats->rx_error_pkts : 0);
	seq_printf(s, "\trx_roce_discard_pkts: %lld\n",
		   stats ? stats->rx_discard_pkts : 0);
	if (!en_disp) {
		seq_printf(s, "\ttx_roce_error_pkts: %lld\n",
			   stats ? stats->tx_error_pkts : 0);
		seq_printf(s, "\ttx_roce_discards_pkts: %lld\n",
			   stats ? stats->tx_discard_pkts : 0);
	}

	if (_is_ext_stats_supported(rdev->dev_attr->dev_cap_flags) &&
	    !rdev->is_virtfn) {
		seq_printf(s, "\tres_oob_drop_count: %lld\n",
			   rdev->stats.dstat.e_errs.oob);
		bnxt_re_print_ext_stat(rdev, s);
	}
}

static int bnxt_re_debugfs_show(struct seq_file *s, void *unused)
{
	struct bnxt_re_dev *rdev = s->private;
	struct bnxt_re_ext_roce_stats *e_errs;
	struct bnxt_re_rdata_counters *rstat;
	struct bnxt_qplib_roce_stats *errs;
	unsigned long tstamp_diff;
	struct pci_dev *pdev;
	struct bnxt *bp;
	int sched_msec;
	int rc = 0;

	seq_printf(s, "bnxt_re debug info:\n");
	if (!rdev)
		return -ENODEV;

	bnxt_re_hold(rdev);
	bp = netdev_priv(rdev->en_dev->net);
	pdev = rdev->en_dev->pdev;

	errs = &rdev->stats.dstat.errs;
	rstat = &rdev->stats.dstat.rstat[0];
	e_errs = &rdev->stats.dstat.e_errs;
	sched_msec = (bp->stats_coal_ticks / 1000);
	if (sched_msec > BNXT_RE_30SEC_MSEC || sched_msec == 0)
		sched_msec = BNXT_RE_STATS_CTX_UPDATE_TIMER;
	tstamp_diff = jiffies - rdev->stats.read_tstamp;
	if (test_bit(BNXT_RE_FLAG_IBDEV_REGISTERED, &rdev->flags)) {
		if (restrict_stats && tstamp_diff <
		    msecs_to_jiffies(sched_msec))
			goto skip_query;
		rc = bnxt_re_get_device_stats(rdev);
		if (rc)
			dev_err(rdev_to_dev(rdev),
				"Failed to query device stats\n");
		rdev->stats.read_tstamp = jiffies;
	}
skip_query:
	seq_printf(s, "=====[ IBDEV %s ]=============================\n",
		   rdev->ibdev.name);
	if (rdev->netdev)
		seq_printf(s, "\tlink state: %s\n",
			   bnxt_re_link_state_str(rdev));
	seq_printf(s, "\tMax QP:\t\t%d\n", rdev->dev_attr->max_qp);
	seq_printf(s, "\tMax SRQ:\t%d\n", rdev->dev_attr->max_srq);
	seq_printf(s, "\tMax CQ:\t\t%d\n", rdev->dev_attr->max_cq);
	seq_printf(s, "\tMax MR:\t\t%d\n", rdev->dev_attr->max_mr);
	seq_printf(s, "\tMax MW:\t\t%d\n", rdev->dev_attr->max_mw);
	seq_printf(s, "\tMax AH:\t\t%d\n", rdev->dev_attr->max_ah);
	seq_printf(s, "\tMax PD:\t\t%d\n", rdev->dev_attr->max_pd);
	seq_printf(s, "\tActive DMA Mem Usage:\t%llu\n",
		   rdev->qplib_res.mem_usage);
	seq_printf(s, "\tActive QP:\t%d\n",
		   atomic_read(&rdev->stats.rsors.qp_count));
	seq_printf(s, "\tActive SRQ:\t%d\n",
		   atomic_read(&rdev->stats.rsors.srq_count));
	seq_printf(s, "\tActive CQ:\t%d\n",
		   atomic_read(&rdev->stats.rsors.cq_count));
	seq_printf(s, "\tActive MR:\t%d\n",
		   atomic_read(&rdev->stats.rsors.mr_count));
	seq_printf(s, "\tActive MW:\t%d\n",
		   atomic_read(&rdev->stats.rsors.mw_count));
	seq_printf(s, "\tActive AH:\t%d\n",
		   atomic_read(&rdev->stats.rsors.ah_count));
	seq_printf(s, "\tActive PD:\t%d\n",
		   atomic_read(&rdev->stats.rsors.pd_count));
	seq_printf(s, "\tRecoverable Errors: %lld\n",
		   rstat ? rstat->tx_bcast_pkts : 0);
	if (rdev->binfo)
		bnxt_re_print_bond_counters(rdev, s);
	else
		bnxt_re_print_normal_counters(rdev, s);

	seq_printf(s,"\tto_retransmits: %llu\n", errs->to_retransmits);
	seq_printf(s,"\tseq_err_naks_rcvd: %llu\n", errs->seq_err_naks_rcvd);
	seq_printf(s, "\tmax_retry_exceeded: %llu\n", errs->max_retry_exceeded);
	seq_printf(s, "\trnr_naks_rcvd: %llu\n", errs->rnr_naks_rcvd);
	seq_printf(s, "\tmissing_resp: %llu\n", errs->missing_resp);
	seq_printf(s, "\tunrecoverable_err: %llu\n", errs->unrecoverable_err);
	seq_printf(s, "\tbad_resp_err: %llu\n", errs->bad_resp_err);
	seq_printf(s, "\tlocal_qp_op_err: %llu\n", errs->local_qp_op_err);
	seq_printf(s, "\tlocal_protection_err: %llu\n", errs->local_protection_err);
	seq_printf(s, "\tmem_mgmt_op_err: %llu\n", errs->mem_mgmt_op_err);
	seq_printf(s, "\tremote_invalid_req_err: %llu\n", errs->remote_invalid_req_err);
	seq_printf(s, "\tremote_access_err: %llu\n", errs->remote_access_err);
	seq_printf(s, "\tremote_op_err: %llu\n", errs->remote_op_err);
	seq_printf(s, "\tdup_req: %llu\n", errs->dup_req);
	seq_printf(s, "\tres_exceed_max: %llu\n", errs->res_exceed_max);
	seq_printf(s, "\tres_length_mismatch: %llu\n", errs->res_length_mismatch);
	seq_printf(s, "\tres_exceeds_wqe: %llu\n", errs->res_exceeds_wqe);
	seq_printf(s, "\tres_opcode_err: %llu\n", errs->res_opcode_err);
	seq_printf(s, "\tres_rx_invalid_rkey: %llu\n", errs->res_rx_invalid_rkey);
	seq_printf(s, "\tres_rx_domain_err: %llu\n", errs->res_rx_domain_err);
	seq_printf(s, "\tres_rx_no_perm: %llu\n", errs->res_rx_no_perm);
	seq_printf(s, "\tres_rx_range_err: %llu\n", errs->res_rx_range_err);
	seq_printf(s, "\tres_tx_invalid_rkey: %llu\n", errs->res_tx_invalid_rkey);
	seq_printf(s, "\tres_tx_domain_err: %llu\n", errs->res_tx_domain_err);
	seq_printf(s, "\tres_tx_no_perm: %llu\n", errs->res_tx_no_perm);
	seq_printf(s, "\tres_tx_range_err: %llu\n", errs->res_tx_range_err);
	seq_printf(s, "\tres_irrq_oflow: %llu\n", errs->res_irrq_oflow);
	seq_printf(s, "\tres_unsup_opcode: %llu\n", errs->res_unsup_opcode);
	seq_printf(s, "\tres_unaligned_atomic: %llu\n", errs->res_unaligned_atomic);
	seq_printf(s, "\tres_rem_inv_err: %llu\n", errs->res_rem_inv_err);
	seq_printf(s, "\tres_mem_error64: %llu\n", errs->res_mem_error);
	seq_printf(s, "\tres_srq_err: %llu\n", errs->res_srq_err);
	seq_printf(s, "\tres_cmp_err: %llu\n", errs->res_cmp_err);
	seq_printf(s, "\tres_invalid_dup_rkey: %llu\n", errs->res_invalid_dup_rkey);
	seq_printf(s, "\tres_wqe_format_err: %llu\n", errs->res_wqe_format_err);
	seq_printf(s, "\tres_cq_load_err: %llu\n", errs->res_cq_load_err);
	seq_printf(s, "\tres_srq_load_err: %llu\n", errs->res_srq_load_err);
	seq_printf(s, "\tres_tx_pci_err: %llu\n", errs->res_tx_pci_err);
	seq_printf(s, "\tres_rx_pci_err: %llu\n", errs->res_rx_pci_err);
	if (_is_ext_stats_supported(rdev->dev_attr->dev_cap_flags) &&
	    !rdev->is_virtfn) {
		seq_printf(s, "\tres_oos_drop_count: %llu\n",
			   e_errs->oos);
	} else {
		/* Display on function 0 as OOS counters are chip-wide */
		if (PCI_FUNC(pdev->devfn) == 0)
			seq_printf(s, "\tres_oos_drop_count: %llu\n",
					errs->res_oos_drop_count);
	}

	seq_printf(s, "\tnum_irq_started : %u\n", rdev->rcfw.num_irq_started);
	seq_printf(s, "\tnum_irq_stopped : %u\n", rdev->rcfw.num_irq_stopped);
	seq_printf(s, "\tpoll_in_intr_en : %u\n", rdev->rcfw.poll_in_intr_en);
	seq_printf(s, "\tpoll_in_intr_dis : %u\n", rdev->rcfw.poll_in_intr_dis);
	seq_printf(s, "\tcmdq_full_dbg_cnt : %u\n", rdev->rcfw.cmdq_full_dbg);
	if (!rdev->is_virtfn)
		seq_printf(s, "\tfw_service_prof_type_sup : %u\n",
			   is_qport_service_type_supported(rdev));
	seq_printf(s, "\n");
	bnxt_re_put(rdev);
	return 0;
}

static int bnxt_re_debugfs_open(struct inode *inode, struct file *file)
{
	struct bnxt_re_dev *rdev = inode->i_private;

	return single_open(file, bnxt_re_debugfs_show, rdev);
}

static int bnxt_re_debugfs_release(struct inode *inode, struct file *file)
{
	return single_release(inode, file);
}

static const struct file_operations bnxt_re_dbg_ops = {
	.owner		= THIS_MODULE,
	.open		= bnxt_re_debugfs_open,
	.read		= seq_read,
	.write		= bnxt_re_debugfs_clear,
	.llseek		= seq_lseek,
	.release	= bnxt_re_debugfs_release,
};

void bnxt_re_add_dbg_files(struct bnxt_re_dev *rdev)
{
	if (rdev->dbg_ent) {
		rdev->dbg_ent->qpinfo_dir =
		debugfs_create_dir("qp_info", rdev->port_debug_dir);
		rdev->dbg_ent->service_fn_dir =
			debugfs_create_dir("service_fn_query",
					   rdev->port_debug_dir);
	}

	if (rdev->dbg_ent && rdev->dbg_ent->service_fn_dir)
		bnxt_re_add_stats_query(rdev);
}

void bnxt_re_debugfs_add_port(struct bnxt_re_dev *rdev)
{
	if (rdev->en_dev)
		rdev->port_debug_dir =
		debugfs_create_dir(dev_name(&rdev->ibdev.dev),
				   bnxt_re_debugfs_root);
	if (rdev->port_debug_dir) {
		rdev->info = debugfs_create_file("info", S_IRUSR,
						 rdev->port_debug_dir, rdev,
						 &bnxt_re_dbg_ops);
		if (IS_ERR_OR_NULL(rdev->info)) {
			dev_dbg(rdev_to_dev(rdev),
				"Unable to create debugfs info node rdev =%p",
				rdev);
			debugfs_remove_recursive(rdev->port_debug_dir);
			rdev->port_debug_dir = NULL;
			return;
		}
	}
	rdev->dbg_ent = kzalloc(sizeof(*rdev->dbg_ent), GFP_KERNEL);
}

void bnxt_re_rem_dbg_files(struct bnxt_re_dev *rdev)
{
	if (rdev->dbg_ent) {
		debugfs_remove_recursive(rdev->dbg_ent->qpinfo_dir);
		debugfs_remove_recursive(rdev->dbg_ent->service_fn_dir);
		rdev->dbg_ent->qpinfo_dir = NULL;
		rdev->dbg_ent->service_fn_dir = NULL;
	}
}

void bnxt_re_debugfs_rem_port(struct bnxt_re_dev *rdev)
{
	debugfs_remove_recursive(rdev->port_debug_dir);
	kfree(rdev->dbg_ent);
	rdev->info = NULL;
	rdev->dbg_ent = NULL;
	rdev->port_debug_dir = NULL;
}

void bnxt_re_debugfs_remove(void)
{
	debugfs_remove_recursive(bnxt_re_debugfs_root);
	bnxt_re_debugfs_root = NULL;
}

void bnxt_re_debugfs_init(void)
{
	bnxt_re_debugfs_root = debugfs_create_dir(ROCE_DRV_MODULE_NAME, NULL);
	if (IS_ERR_OR_NULL(bnxt_re_debugfs_root)) {
		dev_dbg(NULL, "%s: Unable to create debugfs root directory ",
			ROCE_DRV_MODULE_NAME);
		dev_dbg(NULL, "with err 0x%lx", PTR_ERR(bnxt_re_debugfs_root));
		return;
	}
}
#endif
