/*
 * Copyright (c) 2015-2020, Broadcom. All rights reserved.  The term
 * Broadcom refers to Broadcom Limited and/or its subsidiaries.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * BSD license below:
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Author: Eddie Wai <eddie.wai@broadcom.com>
 *
 * Description: RDMA Controller HW interface
 */

#include <linux/interrupt.h>
#include <linux/spinlock.h>
#include <linux/dma-mapping.h>
#include <linux/sched.h>
#include <linux/pci.h>
#include <linux/delay.h>

#include "roce_hsi.h"

#include "qplib_tlv.h"
#include "qplib_res.h"
#include "qplib_sp.h"
#include "qplib_rcfw.h"
#include "compat.h"

static void bnxt_qplib_service_creq(
#ifdef HAS_TASKLET_SETUP
		struct tasklet_struct *t
#else
		unsigned long data
#endif
		);

static int __check_cmdq_stall(struct bnxt_qplib_rcfw *rcfw,
			      u32 *cur_prod, u32 *cur_cons)
{
	struct bnxt_qplib_cmdq_ctx *cmdq;

	cmdq = &rcfw->cmdq;

	if (*cur_prod == cmdq->hwq.prod &&
	    *cur_cons == cmdq->hwq.cons)
		/* No activity on CMDQ or CREQ. FW down */
		return -ETIMEDOUT;

	*cur_prod = cmdq->hwq.prod;
	*cur_cons = cmdq->hwq.cons;
	return 0;
}

/* Hardware communication channel */
static int __wait_for_resp(struct bnxt_qplib_rcfw *rcfw, u16 cookie)
{
	u32 poll_cnt = RCFW_CMD_WAIT_RETRY_CNT * RCFW_CMD_WAIT_TIME_MS;
	u32 wait_cnt = RCFW_CMD_WAIT_RETRY_CNT;
	struct bnxt_qplib_cmdq_ctx *cmdq;
	u32 cur_prod = 0, cur_cons = 0;
	bool reschedule = false;
	u16 cbit;
	int rc;

	cmdq = &rcfw->cmdq;
	cur_prod = cmdq->hwq.prod;
	cur_cons = cmdq->hwq.cons;

resched:
	reschedule = false;

	/* If the driver is already in timed out state, dont proceed*/
	if (test_bit(FIRMWARE_TIMED_OUT, &cmdq->flags))
		return -ETIMEDOUT;
	if (test_bit(ERR_DEVICE_DETACHED, &cmdq->flags))
		return 0;

	cbit = cookie % RCFW_MAX_OUTSTANDING_CMD;
	/* Go to wait mode if interrupts are enabled, else go to poll mode */
	if (atomic_read(&rcfw->rcfw_intr_enabled)) {
		rc = wait_event_timeout(cmdq->waitq,
					!test_bit(cbit, cmdq->cmdq_bitmap),
					msecs_to_jiffies(RCFW_CMD_WAIT_TIME_MS));
		if (!rc) {
			dev_dbg(&rcfw->pdev->dev,
				"%s: command wait timeout of %d ms. retry rem = %d\n",
				__func__, RCFW_CMD_WAIT_TIME_MS, wait_cnt);
			rcfw->poll_in_intr_en++;

			/* Check polling mode */
			bnxt_qplib_service_creq(
#ifdef HAS_TASKLET_SETUP
					&rcfw->creq.creq_tasklet
#else
					(unsigned long)rcfw
#endif
					);

			rc = __check_cmdq_stall(rcfw, &cur_prod, &cur_cons);
			if (rc)
				return rc;

			if ((test_bit(cbit, cmdq->cmdq_bitmap) && --wait_cnt)) {
				msleep(1);
				goto resched;
			}
		}
	} else {
		/* Check polling mode */
		rcfw->poll_in_intr_dis++;
		dev_dbg(&rcfw->pdev->dev,
			"%s: poll mode - retry rem = %d\n",
			__func__, poll_cnt);
		bnxt_qplib_service_creq(
#ifdef HAS_TASKLET_SETUP
				&rcfw->creq.creq_tasklet
#else
				(unsigned long)rcfw
#endif
				);

		poll_cnt--;

		if (poll_cnt % RCFW_CMD_WAIT_TIME_MS == 0) {
			rc = __check_cmdq_stall(rcfw, &cur_prod, &cur_cons);
			if (rc)
				return rc;
		}
		if ((test_bit(cbit, cmdq->cmdq_bitmap) && poll_cnt)) {
			msleep(1);
			goto resched;
		}
	}
	if (!poll_cnt || !wait_cnt)
		return -ETIMEDOUT;
	else
		return 0;
};

static int __block_for_resp(struct bnxt_qplib_rcfw *rcfw, u16 cookie)
{
	u32 count = RCFW_BLOCKED_CMD_WAIT_COUNT;
	struct bnxt_qplib_cmdq_ctx *cmdq;
	u32 cur_prod = 0, cur_cons = 0;
	u16 cbit;
	int rc;

	cmdq = &rcfw->cmdq;
	cbit = cookie % RCFW_MAX_OUTSTANDING_CMD;
	cur_prod = cmdq->hwq.prod;
	cur_cons = cmdq->hwq.cons;
	count--;
	if (!test_bit(cbit, cmdq->cmdq_bitmap))
		goto done;
	do {
		udelay(1); /* 1 usec */
		bnxt_qplib_service_creq(
#ifdef HAS_TASKLET_SETUP
				&rcfw->creq.creq_tasklet
#else
				(unsigned long)rcfw
#endif
				);
		if (count % RCFW_BLOCKED_STALL_CHECK == 0) {
			rc = __check_cmdq_stall(rcfw, &cur_prod, &cur_cons);
			if (rc)
				return rc;
		}

		if (test_bit(ERR_DEVICE_DETACHED, &cmdq->flags))
			return 0;
	} while (test_bit(cbit, cmdq->cmdq_bitmap) && --count);
done:
	return count ? 0 : -ETIMEDOUT;
};

static int __send_message(struct bnxt_qplib_rcfw *rcfw,
			  struct bnxt_qplib_cmdqmsg *msg)
{
	struct bnxt_qplib_cmdq_ctx *cmdq = &rcfw->cmdq;
	struct bnxt_qplib_hwq *hwq = &cmdq->hwq;
	struct bnxt_qplib_crsqe *crsqe;
	struct bnxt_qplib_cmdqe *cmdqe;
	u32 sw_prod, cmdq_prod;
	struct pci_dev *pdev;
	unsigned long flags;
	u32 bsize, opcode;
	u16 cookie, cbit;
	int pg, idx;
	u8 *preq;

	pdev = rcfw->pdev;

	opcode = __get_cmdq_base_opcode(msg->req, msg->req_sz);
	if (!test_bit(FIRMWARE_INITIALIZED_FLAG, &cmdq->flags) &&
	    (opcode != CMDQ_BASE_OPCODE_QUERY_FUNC &&
	     opcode != CMDQ_BASE_OPCODE_INITIALIZE_FW &&
	     opcode != CMDQ_BASE_OPCODE_QUERY_VERSION)) {
		dev_err(&pdev->dev,
			"QPLIB: RCFW not initialized, reject opcode 0x%x",
			opcode);
		return -EINVAL;
	}

	if (test_bit(FIRMWARE_INITIALIZED_FLAG, &cmdq->flags) &&
	    opcode == CMDQ_BASE_OPCODE_INITIALIZE_FW) {
		dev_err(&pdev->dev, "QPLIB: RCFW already initialized!");
		return -EINVAL;
	}

	if (test_bit(FIRMWARE_TIMED_OUT, &cmdq->flags))
		return -ETIMEDOUT;

	/* Cmdq are in 16-byte units, each request can consume 1 or more
	   cmdqe */
	spin_lock_irqsave(&hwq->lock, flags);
	if (bnxt_qplib_get_cmd_slots(msg->req) >= HWQ_FREE_SLOTS(hwq)) {
		dev_dbg(&pdev->dev, "QPLIB: RCFW: CMDQ is full!");
		rcfw->cmdq_full_dbg++;
		spin_unlock_irqrestore(&hwq->lock, flags);
		return -EAGAIN;
	}

	cookie = cmdq->seq_num & RCFW_MAX_COOKIE_VALUE;
	cbit = cookie % RCFW_MAX_OUTSTANDING_CMD;
	if (msg->block)
		cookie |= RCFW_CMD_IS_BLOCKING;
	if (test_bit(cbit, cmdq->cmdq_bitmap)) {
		dev_dbg(&pdev->dev,
			"%s: QPLIB: cmdq bitmap set at %d", __func__, cbit);
		spin_unlock_irqrestore(&hwq->lock, flags);
		return -EAGAIN;
	}

	set_bit(cbit, cmdq->cmdq_bitmap);
	__set_cmdq_base_cookie(msg->req, msg->req_sz, cpu_to_le16(cookie));
	crsqe = &rcfw->crsqe_tbl[cbit];
	if (unlikely(crsqe->resp)) {
		dev_dbg(&rcfw->pdev->dev,
			"%s: QPLIB: crsqe not cleared at %d", __func__, cbit);
		clear_bit(cbit, cmdq->cmdq_bitmap);
		spin_unlock_irqrestore(&hwq->lock, flags);
		return -EBUSY;
	}
	/* Set cmd_size in terms of 16B slots in req. */
	bsize = bnxt_qplib_set_cmd_slots(msg->req);
	/* GET_CMD_SIZE would return number of slots in either case of tlv
	 * and non-tlv commands after call to bnxt_qplib_set_cmd_slots()
	 */
	crsqe->resp = (struct creq_qp_event *)msg->resp;
	crsqe->resp->cookie = cpu_to_le16(cookie);
	crsqe->req_size = __get_cmdq_base_cmd_size(msg->req, msg->req_sz);
	if (__get_cmdq_base_resp_size(msg->req, msg->req_sz) && msg->sb) {
		struct bnxt_qplib_rcfw_sbuf *sbuf = msg->sb;

		__set_cmdq_base_resp_addr(msg->req, msg->req_sz,
					  cpu_to_le64(sbuf->dma_addr));
		__set_cmdq_base_resp_size(msg->req, msg->req_sz,
					  ALIGN(sbuf->size,
						BNXT_QPLIB_CMDQE_UNITS));
	}

	preq = (u8 *)msg->req;
	do {
		pg = 0;
		idx = 0;
		/* Locate the next cmdq slot */
		sw_prod = HWQ_CMP(hwq->prod, hwq);
		cmdqe = bnxt_qplib_get_qe(hwq, sw_prod, NULL);
		/* Copy a segment of the req cmd to the cmdq */
		memset(cmdqe, 0, sizeof(*cmdqe));
		memcpy(cmdqe, preq, min_t(u32, bsize, sizeof(*cmdqe)));
		preq += min_t(u32, bsize, sizeof(*cmdqe));
		bsize -= min_t(u32, bsize, sizeof(*cmdqe));
		hwq->prod++;
	} while (bsize > 0);
	cmdq->seq_num++;

	cmdq_prod = hwq->prod;
	if (test_bit(FIRMWARE_FIRST_FLAG, &cmdq->flags)) {
		/* The very first doorbell write
		 * is required to set this flag
		 * which prompts the FW to reset
		 * its internal pointers
		 */
		cmdq_prod |= BIT(FIRMWARE_FIRST_FLAG);
		clear_bit(FIRMWARE_FIRST_FLAG, &cmdq->flags);
	}
	/* ring CMDQ DB */
	wmb();
	writel(cmdq_prod, cmdq->cmdq_mbox.prod);
	writel(RCFW_CMDQ_TRIG_VAL, cmdq->cmdq_mbox.db);

	dev_dbg(&pdev->dev, "QPLIB: RCFW sent request with ");
	dev_dbg(&pdev->dev,
		"QPLIB: opcode 0x%x with cookie 0x%x at cmdq/crsq 0x%p/0x%p",
		__get_cmdq_base_opcode(msg->req, msg->req_sz),
		__get_cmdq_base_cookie(msg->req, msg->req_sz),
		cmdqe, crsqe);
	spin_unlock_irqrestore(&hwq->lock, flags);
	/* Return the CREQ response pointer */
	return 0;
}

int bnxt_qplib_rcfw_send_message(struct bnxt_qplib_rcfw *rcfw,
				 struct bnxt_qplib_cmdqmsg *msg)
{
	u64 issue_time = 0, compl_time = 0;
	u8 opcode, retry_cnt = BNXT_QPLIB_RCFW_SEND_RETRY_COUNT;
	struct creq_qp_event *evnt;
	struct pci_dev *pdev;
	u16 cookie;
	int rc = 0;

	pdev = rcfw->pdev;
	evnt = (struct creq_qp_event *)msg->resp;
	/* Get current tick*/
	issue_time = get_jiffies_64();
	do {
		/* Prevent posting if f/w is not in a state to process */
		if (test_bit(ERR_DEVICE_DETACHED, &rcfw->cmdq.flags))
			return 0;
		opcode = __get_cmdq_base_opcode(msg->req, msg->req_sz);
		rc = __send_message(rcfw, msg);
		cookie = le16_to_cpu(__get_cmdq_base_cookie(msg->req,
							    msg->req_sz)) &
				     RCFW_MAX_COOKIE_VALUE;
		if (!rc)
			break;

		if (!retry_cnt || (rc != -EAGAIN && rc != -EBUSY)) {
			/* send failed */
			dev_err(&pdev->dev,
				"QPLIB: cmdq[%#x]=%#x send failed - retries:%d",
				cookie, opcode,
				BNXT_QPLIB_RCFW_SEND_RETRY_COUNT - retry_cnt);
			return rc;
		}
		/* If the thread is busy looping for posting,
		 * explicitly poll for command completions as the interrupts
		 * might be disabled on that processor
		 */
		if (msg->block)
			bnxt_qplib_service_creq(
#ifdef HAS_TASKLET_SETUP
					&rcfw->creq.creq_tasklet
#else
					(unsigned long)rcfw
#endif
				);

		msg->block ? udelay(1000) : usleep_range(1000, 1001);
	} while (retry_cnt--);

	if (msg->block)
		rc = __block_for_resp(rcfw, cookie);
	else
		rc = __wait_for_resp(rcfw, cookie);
	compl_time = get_jiffies_64();
	if (rc) {
		/* timed out */
		dev_err(&pdev->dev, "QPLIB: cmdq[%#x]=%#x timedout (%d)msec",
			cookie, opcode, jiffies_to_msecs(compl_time - issue_time));
		set_bit(FIRMWARE_TIMED_OUT, &rcfw->cmdq.flags);
		return rc;
	}

	/* Add a warning if the command takes more than 20 sec */
	if ((jiffies_to_msecs(compl_time - issue_time)) > 20000) {
		dev_dbg(&pdev->dev, "QPLIB: cmdq[%#x]=%#x taken more than (%u) msec",
			cookie, opcode, jiffies_to_msecs(compl_time - issue_time));
	}

	if (evnt->status) {
		/* failed with status */
		dev_err(&pdev->dev, "QPLIB: cmdq[%#x]=%#x status %#x",
			cookie, opcode, evnt->status);
		rc = -EFAULT;
		/*
		 * Workaround to avoid errors in the stack during bond
		 * creation and deletion.
		 * Disable error returned for  ADD_GID/DEL_GID
		 */
		if (opcode == CMDQ_BASE_OPCODE_ADD_GID ||
		    opcode == CMDQ_BASE_OPCODE_DELETE_GID)
			rc = 0;
	}

	return rc;
}

/* Completions */
static int bnxt_qplib_process_qp_event(struct bnxt_qplib_rcfw *rcfw,
				       struct creq_qp_event *event)
{
	struct bnxt_qplib_hwq *hwq = &rcfw->cmdq.hwq;
	struct creq_cq_error_notification *cqerr;
	struct creq_qp_error_notification *qperr;
	struct bnxt_qplib_crsqe *crsqe;
	struct bnxt_qplib_reftbl *tbl;
	struct bnxt_qplib_qp *qp;
	struct bnxt_qplib_cq *cq;
	u16 cbit, blocked = 0;
	struct pci_dev *pdev;
	unsigned long flags;
	__le16 mcookie;
	u16 cookie;
	int rc = 0;
	u32 qp_idx;
	u32 xid;

	pdev = rcfw->pdev;
	switch (event->event) {
	case CREQ_QP_EVENT_EVENT_QP_ERROR_NOTIFICATION:
		tbl = &rcfw->res->reftbl.qpref;
		qperr = (struct creq_qp_error_notification *)event;
		xid = le32_to_cpu(qperr->xid);
		qp_idx = map_qp_id_to_tbl_indx(xid, tbl);
		spin_lock(&tbl->lock);
		qp = tbl->rec[qp_idx].handle;
		if (!qp) {
			spin_unlock(&tbl->lock);
			break;
		}
		bnxt_qplib_mark_qp_error(qp);
		rc = rcfw->creq.aeq_handler(rcfw, event, qp);
		spin_unlock(&tbl->lock);
		/*
		 * Keeping these prints as debug to avoid flooding of log
		 * messages during modify QP to error state by applications
		 */
		dev_dbg(&pdev->dev, "QPLIB: QP Error encountered!");
		dev_dbg(&pdev->dev,
			"QPLIB: qpid 0x%x, req_err=0x%x, resp_err=0x%x\n",
			xid, qperr->req_err_state_reason,
			qperr->res_err_state_reason);
		break;
	case CREQ_QP_EVENT_EVENT_CQ_ERROR_NOTIFICATION:
		tbl = &rcfw->res->reftbl.cqref;
		cqerr = (struct creq_cq_error_notification *)event;
		xid = le32_to_cpu(cqerr->xid);
		spin_lock(&tbl->lock);
		cq = tbl->rec[GET_TBL_INDEX(xid, tbl)].handle;
		if (!cq) {
			spin_unlock(&tbl->lock);
			break;
		}
		rc = rcfw->creq.aeq_handler(rcfw, event, cq);
		spin_unlock(&tbl->lock);
		dev_dbg(&pdev->dev, "QPLIB: CQ error encountered!");
		break;
	default:
		/*
		 * Command Response
		 * cmdq hwq lock needs to be acquired to synchronize
		 * the command send and completion reaping. This function
		 * is always called with creq hwq lock held. So there is no
		 * chance of deadlock here as the locking is in correct sequence.
		 * Using  the nested variant of spin_lock to annotate
		 */
		spin_lock_irqsave_nested(&hwq->lock, flags,
					 SINGLE_DEPTH_NESTING);
		cookie = le16_to_cpu(event->cookie);
		mcookie = event->cookie;
		blocked = cookie & RCFW_CMD_IS_BLOCKING;
		cookie &= RCFW_MAX_COOKIE_VALUE;
		cbit = cookie % RCFW_MAX_OUTSTANDING_CMD;
		crsqe = &rcfw->crsqe_tbl[cbit];
		if (crsqe->resp) {
			if (crsqe->resp->cookie == mcookie) {
				memcpy(crsqe->resp, event, sizeof(*event));
				crsqe->resp = NULL;
			} else
				dev_err(&pdev->dev,
					"QPLIB:CMD mismatch resp->cookie = %#x,"
					" evnt->cookie = %#x",
					crsqe->resp->cookie, mcookie);
		} else
			dev_err(&pdev->dev,
				"QPLIB: CMD collision? event->status = %#x, cbit = %#x",
				event->status, cbit);

		if (!test_and_clear_bit(cbit, rcfw->cmdq.cmdq_bitmap))
			dev_warn(&pdev->dev,
				 "QPLIB: CMD bit %d was not requested", cbit);
		hwq->cons += crsqe->req_size;
		crsqe->req_size = 0;
		if (!blocked)
			wake_up(&rcfw->cmdq.waitq);
		spin_unlock_irqrestore(&hwq->lock, flags);
	}
	return rc;
}

/* SP - CREQ Completion handlers */
static void bnxt_qplib_service_creq(
#ifdef HAS_TASKLET_SETUP
		struct tasklet_struct *t
#else
		unsigned long data
#endif
		)
{
#ifdef HAS_TASKLET_SETUP
	struct bnxt_qplib_rcfw *rcfw = from_tasklet(rcfw, t, creq.creq_tasklet);
#else

	struct bnxt_qplib_rcfw *rcfw = (struct bnxt_qplib_rcfw *)data;
#endif
	struct bnxt_qplib_creq_ctx *creq = &rcfw->creq;
	struct bnxt_qplib_res *res;
	u32 type, budget = CREQ_ENTRY_POLL_BUDGET;
	struct bnxt_qplib_hwq *hwq = &creq->hwq;
	struct creq_base *creqe;
	struct pci_dev *pdev;
	unsigned long flags;
	int rc;

	pdev = rcfw->pdev;
	res = rcfw->res;
	/* Service the CREQ until empty */
	spin_lock_irqsave(&hwq->lock, flags);
	while (budget > 0) {
		if (test_bit(ERR_DEVICE_DETACHED, &rcfw->cmdq.flags)) {
			spin_unlock_irqrestore(&hwq->lock, flags);
			return;
		}
		creqe = bnxt_qplib_get_qe(hwq, hwq->cons, NULL);
		if (!CREQ_CMP_VALID(creqe, creq->creq_db.dbinfo.flags))
			break;
		/* The valid test of the entry must be done first before
		 * reading any further.
		 */
		dma_rmb();
		type = creqe->type & CREQ_BASE_TYPE_MASK;
		switch (type) {
		case CREQ_BASE_TYPE_QP_EVENT:
			bnxt_qplib_process_qp_event(rcfw,
					(struct creq_qp_event *)creqe);
			creq->stats.creq_qp_event_processed++;
			break;
		case CREQ_BASE_TYPE_FUNC_EVENT:
			rc = rcfw->creq.aeq_handler(rcfw, creqe, NULL);
			if (rc)
				dev_warn(&pdev->dev,
					 "QPLIB: async event type = 0x%x not handled",
					 type);
			creq->stats.creq_func_event_processed++;
			break;
		default:
			if (type != ASYNC_EVENT_CMPL_TYPE_HWRM_ASYNC_EVENT) {
				dev_warn(&pdev->dev,
				 	 "QPLIB: op_event = 0x%x not handled",
					 type);
			}
			break;
		}
		budget--;
		bnxt_qplib_hwq_incr_cons(hwq->max_elements, &hwq->cons,
					 1, &creq->creq_db.dbinfo.flags);
	}
	if (budget == CREQ_ENTRY_POLL_BUDGET) {
		/* No completions received during this poll. Enable interrupt now */
		bnxt_qplib_ring_nq_db(&creq->creq_db.dbinfo, res->cctx, true);
		creq->stats.creq_arm_count++;
		dev_dbg(&pdev->dev, "QPLIB: Num of Func (0x%llx) ",
			creq->stats.creq_func_event_processed);
		dev_dbg(&pdev->dev, "QPLIB: QP (0x%llx) events processed",
			creq->stats.creq_qp_event_processed);
		dev_dbg(&pdev->dev, "QPLIB: Armed:%#llx resched:%#llx ",
			creq->stats.creq_arm_count,
			creq->stats.creq_tasklet_schedule_count);
	} else if (creq->requested) {
		/*
		 * To reduce the number of interrupts from HW,
		 * reschedule the tasklet instead of
		 * enabling interrupts. Ring doorbell to update
		 * consumer index.
		 */
		bnxt_qplib_ring_nq_db(&creq->creq_db.dbinfo, res->cctx, false);
		tasklet_schedule(&creq->creq_tasklet);
		creq->stats.creq_tasklet_schedule_count++;
	}
	spin_unlock_irqrestore(&hwq->lock, flags);
}

static irqreturn_t bnxt_qplib_creq_irq(int irq, void *dev_instance)
{
	struct bnxt_qplib_rcfw *rcfw = dev_instance;

	bnxt_qplib_service_creq(
#ifdef HAS_TASKLET_SETUP
			&rcfw->creq.creq_tasklet
#else
			(unsigned long)rcfw
#endif
			);
	return IRQ_HANDLED;
}

/* RCFW */
int bnxt_qplib_deinit_rcfw(struct bnxt_qplib_rcfw *rcfw)
{
	struct creq_deinitialize_fw_resp resp = {};
	struct cmdq_deinitialize_fw req = {};
	struct bnxt_qplib_cmdqmsg msg = {};
	int rc;

	bnxt_qplib_rcfw_cmd_prep(&req, CMDQ_BASE_OPCODE_DEINITIALIZE_FW,
				 sizeof(req));
	bnxt_qplib_fill_cmdqmsg(&msg, &req, &resp, NULL,
				sizeof(req), sizeof(resp), 0);
	rc = bnxt_qplib_rcfw_send_message(rcfw, &msg);
	if (rc)
		return rc;
	clear_bit(FIRMWARE_INITIALIZED_FLAG, &rcfw->cmdq.flags);
	return 0;
}

int bnxt_qplib_init_rcfw(struct bnxt_qplib_rcfw *rcfw, int is_virtfn)
{
	struct creq_initialize_fw_resp resp = {};
	struct cmdq_initialize_fw req = {};
	struct bnxt_qplib_cmdqmsg msg = {};
	struct bnxt_qplib_creq_ctx *creq;
	struct bnxt_qplib_chip_ctx *cctx;
	struct bnxt_qplib_ctx *hctx;
	struct bnxt_qplib_res *res;
	struct bnxt_qplib_hwq *hwq;
	int rc;

	res = rcfw->res;
	creq = &rcfw->creq;
	cctx = res->cctx;
	hctx = res->hctx;

	bnxt_qplib_rcfw_cmd_prep(&req, CMDQ_BASE_OPCODE_INITIALIZE_FW,
				 sizeof(req));
	/* Supply (log-base-2-of-host-page-size - base-page-shift)
	 * to bono to adjust the doorbell page sizes.
	 */
	req.log2_dbr_pg_size = cpu_to_le16(PAGE_SHIFT -
					   RCFW_DBR_BASE_PAGE_SHIFT);
	/*
	 * VFs need not setup the HW context area, PF
	 * shall setup this area for VF. Skipping the
	 * HW programming
	 */
	if (is_virtfn || _is_chip_gen_p5(cctx))
		goto skip_ctx_setup;

	hwq = &hctx->qp_ctx.hwq;
	req.qpc_page_dir = cpu_to_le64(_get_base_addr(hwq));
	req.number_of_qp = cpu_to_le32(hwq->max_elements);
	req.qpc_pg_size_qpc_lvl = (_get_pte_pg_size(hwq) <<
				   CMDQ_INITIALIZE_FW_QPC_PG_SIZE_SFT) |
				   (u8)hwq->level;

	hwq = &hctx->mrw_ctx.hwq;
	req.mrw_page_dir = cpu_to_le64(_get_base_addr(hwq));
	req.number_of_mrw = cpu_to_le32(hwq->max_elements);
	req.mrw_pg_size_mrw_lvl = (_get_pte_pg_size(hwq) <<
				   CMDQ_INITIALIZE_FW_MRW_PG_SIZE_SFT) |
				   (u8)hwq->level;

	hwq = &hctx->srq_ctx.hwq;
	req.srq_page_dir = cpu_to_le64(_get_base_addr(hwq));
	req.number_of_srq = cpu_to_le32(hwq->max_elements);
	req.srq_pg_size_srq_lvl = (_get_pte_pg_size(hwq) <<
				   CMDQ_INITIALIZE_FW_SRQ_PG_SIZE_SFT) |
				   (u8)hwq->level;

	hwq = &hctx->cq_ctx.hwq;
	req.cq_page_dir = cpu_to_le64(_get_base_addr(hwq));
	req.number_of_cq = cpu_to_le32(hwq->max_elements);
	req.cq_pg_size_cq_lvl = (_get_pte_pg_size(hwq) <<
				 CMDQ_INITIALIZE_FW_CQ_PG_SIZE_SFT) |
				 (u8)hwq->level;

	hwq = &hctx->tim_ctx.hwq;
	req.tim_page_dir = cpu_to_le64(_get_base_addr(hwq));
	req.tim_pg_size_tim_lvl = (_get_pte_pg_size(hwq) <<
				   CMDQ_INITIALIZE_FW_TIM_PG_SIZE_SFT) |
				   (u8)hwq->level;
	hwq = &hctx->tqm_ctx.pde;
	req.tqm_page_dir = cpu_to_le64(_get_base_addr(hwq));
	req.tqm_pg_size_tqm_lvl = (_get_pte_pg_size(hwq) <<
				   CMDQ_INITIALIZE_FW_TQM_PG_SIZE_SFT) |
				   (u8)hwq->level;
skip_ctx_setup:
	req.stat_ctx_id = cpu_to_le32(hctx->stats.fw_id);
	bnxt_qplib_fill_cmdqmsg(&msg, &req, &resp, NULL,
				sizeof(req), sizeof(resp), 0);
	rc = bnxt_qplib_rcfw_send_message(rcfw, &msg);
	if (rc)
		return rc;
	set_bit(FIRMWARE_INITIALIZED_FLAG, &rcfw->cmdq.flags);

	return 0;
}

void bnxt_qplib_free_rcfw_channel(struct bnxt_qplib_res *res)
{
	struct bnxt_qplib_rcfw *rcfw = res->rcfw;

	kfree(rcfw->cmdq.cmdq_bitmap);
	kfree(rcfw->crsqe_tbl);
	bnxt_qplib_free_hwq(res, &rcfw->cmdq.hwq);
	bnxt_qplib_free_hwq(res, &rcfw->creq.hwq);
	rcfw->pdev = NULL;
}

int bnxt_qplib_alloc_rcfw_channel(struct bnxt_qplib_res *res)
{
	struct bnxt_qplib_hwq_attr hwq_attr = {};
	struct bnxt_qplib_rcfw *rcfw = res->rcfw;
	struct bnxt_qplib_sg_info sginfo = {};
	struct bnxt_qplib_cmdq_ctx *cmdq;
	struct bnxt_qplib_creq_ctx *creq;
	u32 bmap_size, bits;

	rcfw->pdev = res->pdev;
	rcfw->res = res;
	cmdq = &rcfw->cmdq;
	creq = &rcfw->creq;

	sginfo.pgsize = PAGE_SIZE;
	sginfo.pgshft = PAGE_SHIFT;

	hwq_attr.sginfo = &sginfo;
	hwq_attr.res = rcfw->res;
	hwq_attr.depth = BNXT_QPLIB_CREQE_MAX_CNT;
	hwq_attr.stride = BNXT_QPLIB_CREQE_UNITS;
	hwq_attr.type = _get_hwq_type(res);

	if (bnxt_qplib_alloc_init_hwq(&creq->hwq, &hwq_attr)) {
		dev_err(&rcfw->pdev->dev,
			"QPLIB: HW channel CREQ allocation failed");
		goto fail;
	}

	sginfo.pgsize = BNXT_QPLIB_CMDQE_PAGE_SIZE;
	hwq_attr.depth = BNXT_QPLIB_CMDQE_MAX_CNT;
	hwq_attr.stride = BNXT_QPLIB_CMDQE_UNITS;
	hwq_attr.type = HWQ_TYPE_CTX;
	if (bnxt_qplib_alloc_init_hwq(&cmdq->hwq, &hwq_attr)) {
		dev_err(&rcfw->pdev->dev,
			"QPLIB: HW channel CMDQ allocation failed");
		goto fail;
	}

	rcfw->crsqe_tbl = kcalloc(cmdq->hwq.max_elements,
			sizeof(*rcfw->crsqe_tbl), GFP_KERNEL);
	if (!rcfw->crsqe_tbl) {
		dev_err(&rcfw->pdev->dev,
			"QPLIB: HW channel CRSQ allocation failed");
		goto fail;
	}

	bits = RCFW_MAX_OUTSTANDING_CMD * sizeof(unsigned long);
	bmap_size = BITS_TO_LONGS(bits);
	cmdq->cmdq_bitmap = kzalloc(bmap_size, GFP_KERNEL);
	if (!cmdq->cmdq_bitmap)
		goto fail;
	cmdq->bmap_size = bmap_size;

	return 0;

fail:
	bnxt_qplib_free_rcfw_channel(res);
	return -ENOMEM;
}

void bnxt_qplib_rcfw_stop_irq(struct bnxt_qplib_rcfw *rcfw, bool kill)
{
	struct bnxt_qplib_creq_ctx *creq;
	struct bnxt_qplib_res *res;

	creq = &rcfw->creq;
	res = rcfw->res;

	if (!creq->requested)
		return;

	creq->requested = false;
	/* Mask h/w interrupts */
	bnxt_qplib_ring_nq_db(&creq->creq_db.dbinfo, res->cctx, false);
	/* Sync with last running IRQ-handler */
	synchronize_irq(creq->msix_vec);
	free_irq(creq->msix_vec, rcfw);
	/* rcfw_intr_enabled should not be greater than 1. Debug
	 * print to check if that is the case
	 */
	if (atomic_read(&rcfw->rcfw_intr_enabled) > 1) {
		dev_err(&rcfw->pdev->dev,
			"%s: rcfw->rcfw_intr_enabled = 0x%x", __func__,
			atomic_read(&rcfw->rcfw_intr_enabled));
	}
	atomic_set(&rcfw->rcfw_intr_enabled, 0);
	rcfw->num_irq_stopped++;
	/* Cleanup Tasklet */
	if (kill)
		tasklet_kill(&creq->creq_tasklet);
	tasklet_disable(&creq->creq_tasklet);
}

void bnxt_qplib_disable_rcfw_channel(struct bnxt_qplib_rcfw *rcfw)
{
	struct bnxt_qplib_creq_ctx *creq;
	struct bnxt_qplib_cmdq_ctx *cmdq;
	unsigned long indx;

	creq = &rcfw->creq;
	cmdq = &rcfw->cmdq;
	/* Make sure the HW channel is stopped! */
	bnxt_qplib_rcfw_stop_irq(rcfw, true);

	creq->creq_db.reg.bar_reg = NULL;
	creq->creq_db.db = NULL;

	if (cmdq->cmdq_mbox.reg.bar_reg) {
		iounmap(cmdq->cmdq_mbox.reg.bar_reg);
		cmdq->cmdq_mbox.reg.bar_reg = NULL;
		cmdq->cmdq_mbox.prod = NULL;
		cmdq->cmdq_mbox.db = NULL;
	}

	indx = find_first_bit(cmdq->cmdq_bitmap, cmdq->bmap_size);
	if (indx != cmdq->bmap_size)
		dev_err(&rcfw->pdev->dev,
			"QPLIB: disabling RCFW with pending cmd-bit %lx", indx);

	creq->aeq_handler = NULL;
	creq->msix_vec = 0;
}

int bnxt_qplib_rcfw_start_irq(struct bnxt_qplib_rcfw *rcfw, int msix_vector,
			      bool need_init)
{
	struct bnxt_qplib_creq_ctx *creq;
	struct bnxt_qplib_res *res;
	int rc;

	creq = &rcfw->creq;
	res = rcfw->res;

	if (creq->requested)
		return -EFAULT;

	creq->msix_vec = msix_vector;
	if (need_init)
		compat_tasklet_init(&creq->creq_tasklet,
				    bnxt_qplib_service_creq,
				    (unsigned long)rcfw);
	else
		tasklet_enable(&creq->creq_tasklet);
	rc = request_irq(creq->msix_vec, bnxt_qplib_creq_irq, 0,
			 "bnxt_qplib_creq", rcfw);
	if (rc) {
		tasklet_disable(&creq->creq_tasklet);
		return rc;
	}
        creq->requested = true;

	bnxt_qplib_ring_nq_db(&creq->creq_db.dbinfo, res->cctx, true);

	rcfw->num_irq_started++;
	/* Debug print to check rcfw interrupt enable/disable is invoked
	 * out of sequence
	 */
	if (atomic_read(&rcfw->rcfw_intr_enabled) > 0) {
		dev_err(&rcfw->pdev->dev,
			"%s: rcfw->rcfw_intr_enabled = 0x%x", __func__,
			atomic_read(&rcfw->rcfw_intr_enabled));
	}
	atomic_inc(&rcfw->rcfw_intr_enabled);
	return 0;
}

static int bnxt_qplib_map_cmdq_mbox(struct bnxt_qplib_rcfw *rcfw, bool is_vf)
{
	struct bnxt_qplib_cmdq_mbox *mbox;
	resource_size_t bar_reg;
	struct pci_dev *pdev;
	u16 prod_offt;
	int rc = 0;

	pdev = rcfw->pdev;
	mbox = &rcfw->cmdq.cmdq_mbox;

	mbox->reg.bar_id = RCFW_COMM_PCI_BAR_REGION;
	mbox->reg.len = RCFW_COMM_SIZE;
	mbox->reg.bar_base = pci_resource_start(pdev, mbox->reg.bar_id);
	if (!mbox->reg.bar_base) {
		dev_err(&pdev->dev,
			"QPLIB: CMDQ BAR region %d resc start is 0!\n",
			mbox->reg.bar_id);
		return -ENOMEM;
	}

	bar_reg = mbox->reg.bar_base + RCFW_COMM_BASE_OFFSET;
	mbox->reg.len = RCFW_COMM_SIZE;
	mbox->reg.bar_reg = ioremap(bar_reg, mbox->reg.len);
	if (!mbox->reg.bar_reg) {
		dev_err(&pdev->dev,
			"QPLIB: CMDQ BAR region %d mapping failed\n",
			mbox->reg.bar_id);
		return -ENOMEM;
	}

	prod_offt = is_vf ? RCFW_VF_COMM_PROD_OFFSET :
			    RCFW_PF_COMM_PROD_OFFSET;
	mbox->prod = (void  __iomem *)((char *)mbox->reg.bar_reg + prod_offt);
	mbox->db = (void __iomem *)((char *)mbox->reg.bar_reg +
				     RCFW_COMM_TRIG_OFFSET);
	return rc;
}

static int bnxt_qplib_map_creq_db(struct bnxt_qplib_rcfw *rcfw, u32 reg_offt)
{
	struct bnxt_qplib_creq_db *creq_db;
	struct bnxt_qplib_reg_desc *dbreg;
	struct bnxt_qplib_res *res;

	res = rcfw->res;
	creq_db = &rcfw->creq.creq_db;
	dbreg = &res->dpi_tbl.ucreg;

	creq_db->reg.bar_id = dbreg->bar_id;
	creq_db->reg.bar_base = dbreg->bar_base;
	creq_db->reg.bar_reg = dbreg->bar_reg + reg_offt;
	creq_db->reg.len = _is_chip_gen_p5(res->cctx) ? sizeof(u64) :
							sizeof(u32);

	creq_db->dbinfo.db = creq_db->reg.bar_reg;
	creq_db->dbinfo.hwq = &rcfw->creq.hwq;
	creq_db->dbinfo.xid = rcfw->creq.ring_id;
	creq_db->dbinfo.flags = 0;

	return 0;
}

static void bnxt_qplib_start_rcfw(struct bnxt_qplib_rcfw *rcfw)
{
	struct bnxt_qplib_cmdq_ctx *cmdq;
	struct bnxt_qplib_creq_ctx *creq;
	struct bnxt_qplib_cmdq_mbox *mbox;
	struct cmdq_init init = {0};

	cmdq = &rcfw->cmdq;
	creq = &rcfw->creq;
	mbox = &cmdq->cmdq_mbox;

	init.cmdq_pbl = cpu_to_le64(cmdq->hwq.pbl[PBL_LVL_0].pg_map_arr[0]);
	init.cmdq_size_cmdq_lvl = cpu_to_le16(
			((BNXT_QPLIB_CMDQE_MAX_CNT << CMDQ_INIT_CMDQ_SIZE_SFT) &
			 CMDQ_INIT_CMDQ_SIZE_MASK) |
			((cmdq->hwq.level << CMDQ_INIT_CMDQ_LVL_SFT) &
			 CMDQ_INIT_CMDQ_LVL_MASK));
	init.creq_ring_id = cpu_to_le16(creq->ring_id);
	/* Write to the Bono mailbox register */
	__iowrite32_copy(mbox->reg.bar_reg, &init, sizeof(init) / 4);
}

int bnxt_qplib_enable_rcfw_channel(struct bnxt_qplib_rcfw *rcfw,
				   int msix_vector,
				   int cp_bar_reg_off, int virt_fn,
				   aeq_handler_t aeq_handler)
{
	struct bnxt_qplib_cmdq_ctx *cmdq;
	struct bnxt_qplib_creq_ctx *creq;
	int rc;

	cmdq = &rcfw->cmdq;
	creq = &rcfw->creq;

	/* Clear to defaults */
	cmdq->seq_num = 0;
	set_bit(FIRMWARE_FIRST_FLAG, &cmdq->flags);
	init_waitqueue_head(&cmdq->waitq);

	creq->stats.creq_qp_event_processed = 0;
	creq->stats.creq_func_event_processed = 0;
	creq->aeq_handler = aeq_handler;

	rc = bnxt_qplib_map_cmdq_mbox(rcfw, virt_fn);
	if (rc)
		return rc;

	rc = bnxt_qplib_map_creq_db(rcfw, cp_bar_reg_off);
	if (rc)
		return rc;

	rc = bnxt_qplib_rcfw_start_irq(rcfw, msix_vector, true);
	if (rc) {
		dev_err(&rcfw->pdev->dev,
			"QPLIB: Failed to request IRQ for CREQ rc = 0x%x", rc);
		bnxt_qplib_disable_rcfw_channel(rcfw);
		return rc;
	}

	bnxt_qplib_start_rcfw(rcfw);

	return 0;
}

struct bnxt_qplib_rcfw_sbuf *bnxt_qplib_rcfw_alloc_sbuf(
		struct bnxt_qplib_rcfw *rcfw,
		u32 size)
{
	struct bnxt_qplib_rcfw_sbuf *sbuf;

	sbuf = kzalloc(sizeof(*sbuf), GFP_KERNEL);
	if (!sbuf)
		return NULL;

	sbuf->size = size;
	sbuf->sb = bnxt_re_dma_alloc_coherent(&rcfw->pdev->dev, sbuf->size,
					      &sbuf->dma_addr, GFP_KERNEL);
	if (!sbuf->sb)
		goto bail;

	return sbuf;
bail:
	kfree(sbuf);
	return NULL;
}

void bnxt_qplib_rcfw_free_sbuf(struct bnxt_qplib_rcfw *rcfw,
			       struct bnxt_qplib_rcfw_sbuf *sbuf)
{
	if (sbuf->sb)
		dma_free_coherent(&rcfw->pdev->dev, sbuf->size,
				  sbuf->sb, sbuf->dma_addr);
	kfree(sbuf);
}
