/*
 * Copyright (c) 2015-2020, Broadcom. All rights reserved.  The term
 * Broadcom refers to Broadcom Limited and/or its subsidiaries.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * BSD license below:
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Author: Devesh Sharma <devesh.sharma@broadcom.com>
 *
 * Description: statistics related functions
 */

#include "bnxt_re.h"
#include "stats.h"

int bnxt_re_get_flow_stats_from_service_pf(struct bnxt_re_dev *rdev,
					   struct bnxt_re_flow_counters *stats,
					   struct bnxt_qplib_query_stats_info *sinfo)
{
	struct hwrm_cfa_flow_stats_output resp = {};
	struct hwrm_cfa_flow_stats_input req = {};
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	struct bnxt_fw_msg fw_msg = {};
	u16 target_id;
	int rc = 0;

	/* FIXME Workaround to avoid system hang when this
	 * thread is competing with device create/destroy
	 * sequence
	 */
	if (!bnxt_re_rtnl_trylock())
		/* Not querying stats. Return older values */
		return 0;

	if (sinfo->function_id == 0xFFFFFFFF)
		target_id = -1;
	else
		target_id = sinfo->function_id + 1;

	/* Issue HWRM cmd to read flow counters for CNP tx and rx */
	bnxt_re_init_hwrm_hdr(rdev, (void *)&req, HWRM_CFA_FLOW_STATS, -1, target_id);
	req.num_flows = cpu_to_le16(6);
	req.flow_handle_0 = cpu_to_le16(CFA_FLOW_INFO_REQ_FLOW_HANDLE_CNP_CNT);
	req.flow_handle_1 = cpu_to_le16(CFA_FLOW_INFO_REQ_FLOW_HANDLE_CNP_CNT |
					CFA_FLOW_INFO_REQ_FLOW_HANDLE_DIR_RX);
	req.flow_handle_2 = cpu_to_le16(CFA_FLOW_INFO_REQ_FLOW_HANDLE_ROCEV1_CNT);
	req.flow_handle_3 = cpu_to_le16(CFA_FLOW_INFO_REQ_FLOW_HANDLE_ROCEV1_CNT |
					CFA_FLOW_INFO_REQ_FLOW_HANDLE_DIR_RX);
	req.flow_handle_4 = cpu_to_le16(CFA_FLOW_INFO_REQ_FLOW_HANDLE_ROCEV2_CNT);
	req.flow_handle_5 = cpu_to_le16(CFA_FLOW_INFO_REQ_FLOW_HANDLE_ROCEV2_CNT |
					CFA_FLOW_INFO_REQ_FLOW_HANDLE_DIR_RX);
	bnxt_re_fill_fw_msg(&fw_msg, &req, sizeof(req), &resp,
			    sizeof(resp), DFLT_HWRM_CMD_TIMEOUT);
	rc = en_dev->en_ops->bnxt_send_fw_msg(en_dev, BNXT_ROCE_ULP, &fw_msg);
	if (rc) {
		rtnl_unlock();
		dev_err(rdev_to_dev(rdev),
			"Failed to get CFA Flow stats : rc = 0x%x", rc);
		return rc;
	}

	stats->cnp_stats.cnp_tx_pkts = le64_to_cpu(resp.packet_0);
	stats->cnp_stats.cnp_tx_bytes = le64_to_cpu(resp.byte_0);
	stats->cnp_stats.cnp_rx_pkts = le64_to_cpu(resp.packet_1);
	stats->cnp_stats.cnp_rx_bytes = le64_to_cpu(resp.byte_1);

	stats->ro_stats.tx_pkts = le64_to_cpu(resp.packet_2) +
		le64_to_cpu(resp.packet_4);
	stats->ro_stats.tx_bytes = le64_to_cpu(resp.byte_2) +
		le64_to_cpu(resp.byte_4);
	stats->ro_stats.rx_pkts = le64_to_cpu(resp.packet_3) +
		le64_to_cpu(resp.packet_5);
	stats->ro_stats.rx_bytes = le64_to_cpu(resp.byte_3) +
		le64_to_cpu(resp.byte_5);

	rtnl_unlock();
	return 0;
}

int bnxt_re_get_qos_stats(struct bnxt_re_dev *rdev)
{
	struct bnxt_re_ro_counters roce_only_tmp[2] = {{}, {}};
	struct bnxt_re_cnp_counters tmp_counters[2] = {{}, {}};
	struct hwrm_cfa_flow_stats_output resp = {};
	struct hwrm_cfa_flow_stats_input req = {};
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	struct bnxt_fw_msg fw_msg = {};
	struct bnxt_re_cc_stat *cnps;
	struct bnxt_re_rstat *dstat;
	int rc = 0;
	u64 bytes;
	u64 pkts;

	/* FIXME Workaround to avoid system hang when this
	 * thread is competing with device create/destroy
	 * sequence
	 */
	if (!bnxt_re_rtnl_trylock())
		/* Not querying stats. Return older values */
		return 0;

	/* Issue HWRM cmd to read flow counters for CNP tx and rx */
	bnxt_re_init_hwrm_hdr(rdev, (void *)&req, HWRM_CFA_FLOW_STATS, -1, -1);
	req.num_flows = cpu_to_le16(6);
	req.flow_handle_0 = cpu_to_le16(CFA_FLOW_INFO_REQ_FLOW_HANDLE_CNP_CNT);
	req.flow_handle_1 = cpu_to_le16(CFA_FLOW_INFO_REQ_FLOW_HANDLE_CNP_CNT |
					CFA_FLOW_INFO_REQ_FLOW_HANDLE_DIR_RX);
	req.flow_handle_2 = cpu_to_le16(CFA_FLOW_INFO_REQ_FLOW_HANDLE_ROCEV1_CNT);
	req.flow_handle_3 = cpu_to_le16(CFA_FLOW_INFO_REQ_FLOW_HANDLE_ROCEV1_CNT |
					CFA_FLOW_INFO_REQ_FLOW_HANDLE_DIR_RX);
	req.flow_handle_4 = cpu_to_le16(CFA_FLOW_INFO_REQ_FLOW_HANDLE_ROCEV2_CNT);
	req.flow_handle_5 = cpu_to_le16(CFA_FLOW_INFO_REQ_FLOW_HANDLE_ROCEV2_CNT |
				       CFA_FLOW_INFO_REQ_FLOW_HANDLE_DIR_RX);
	bnxt_re_fill_fw_msg(&fw_msg, (void *)&req, sizeof(req), (void *)&resp,
			    sizeof(resp), DFLT_HWRM_CMD_TIMEOUT);
	rc = en_dev->en_ops->bnxt_send_fw_msg(en_dev, BNXT_ROCE_ULP, &fw_msg);
	if (rc) {
		dev_err(rdev_to_dev(rdev),
			"Failed to get CFA Flow stats : rc = 0x%x", rc);
		goto done;
	}

	tmp_counters[0].cnp_tx_pkts = le64_to_cpu(resp.packet_0);
	tmp_counters[0].cnp_tx_bytes = le64_to_cpu(resp.byte_0);
	tmp_counters[0].cnp_rx_pkts = le64_to_cpu(resp.packet_1);
	tmp_counters[0].cnp_rx_bytes = le64_to_cpu(resp.byte_1);

	roce_only_tmp[0].tx_pkts = le64_to_cpu(resp.packet_2) +
				   le64_to_cpu(resp.packet_4);
	roce_only_tmp[0].tx_bytes = le64_to_cpu(resp.byte_2) +
				    le64_to_cpu(resp.byte_4);
	roce_only_tmp[0].rx_pkts = le64_to_cpu(resp.packet_3) +
				   le64_to_cpu(resp.packet_5);
	roce_only_tmp[0].rx_bytes = le64_to_cpu(resp.byte_3) +
				    le64_to_cpu(resp.byte_5);

	if (rdev->binfo) {
		struct bnxt *second_bp = netdev_priv(rdev->binfo->slave2);

		memset(&req, 0, sizeof(req));
		memset(&fw_msg, 0, sizeof(fw_msg));
		bnxt_re_init_hwrm_hdr(rdev, (void *)&req, HWRM_CFA_FLOW_STATS,
				      -1, PCI_FUNC(second_bp->pdev->devfn) + 1);
		req.num_flows = cpu_to_le16(6);
		req.flow_handle_0 = cpu_to_le16(CFA_FLOW_INFO_REQ_FLOW_HANDLE_CNP_CNT);
		req.flow_handle_1 = cpu_to_le16(CFA_FLOW_INFO_REQ_FLOW_HANDLE_CNP_CNT |
				CFA_FLOW_INFO_REQ_FLOW_HANDLE_DIR_RX);
		req.flow_handle_2 = cpu_to_le16(CFA_FLOW_INFO_REQ_FLOW_HANDLE_ROCEV1_CNT);
		req.flow_handle_3 = cpu_to_le16(CFA_FLOW_INFO_REQ_FLOW_HANDLE_ROCEV1_CNT |
				CFA_FLOW_INFO_REQ_FLOW_HANDLE_DIR_RX);
		req.flow_handle_4 = cpu_to_le16(CFA_FLOW_INFO_REQ_FLOW_HANDLE_ROCEV2_CNT);
		req.flow_handle_5 = cpu_to_le16(CFA_FLOW_INFO_REQ_FLOW_HANDLE_ROCEV2_CNT |
				CFA_FLOW_INFO_REQ_FLOW_HANDLE_DIR_RX);
		bnxt_re_fill_fw_msg(&fw_msg, &req, sizeof(req), &resp,
				    sizeof(resp), DFLT_HWRM_CMD_TIMEOUT);
		rc = en_dev->en_ops->bnxt_send_fw_msg(en_dev, BNXT_ROCE_ULP, &fw_msg);
		if (rc) {
			dev_dbg(rdev_to_dev(rdev),
				"Failed to get CFA 2nd port : rc = 0x%x",
				rc);
			/* Workaround for avoiding CFA query problem for bond */
			rc = 0;
			goto done;
		}
		tmp_counters[1].cnp_tx_pkts = le64_to_cpu(resp.packet_0);
		tmp_counters[1].cnp_tx_bytes = le64_to_cpu(resp.byte_0);
		tmp_counters[1].cnp_rx_pkts = le64_to_cpu(resp.packet_1);
		tmp_counters[1].cnp_rx_bytes = le64_to_cpu(resp.byte_1);

		roce_only_tmp[1].tx_pkts = le64_to_cpu(resp.packet_2) +
					   le64_to_cpu(resp.packet_4);
		roce_only_tmp[1].tx_bytes = le64_to_cpu(resp.byte_2) +
					    le64_to_cpu(resp.byte_4);
		roce_only_tmp[1].rx_pkts = le64_to_cpu(resp.packet_3) +
					   le64_to_cpu(resp.packet_5);
		roce_only_tmp[1].rx_bytes = le64_to_cpu(resp.byte_3) +
					    le64_to_cpu(resp.byte_5);
	}

	cnps = &rdev->stats.cnps;
	dstat = &rdev->stats.dstat;
	if (!cnps->is_first) {
		/* First query done.. */
		cnps->is_first = true;
		cnps->prev[0].cnp_tx_pkts = tmp_counters[0].cnp_tx_pkts;
		cnps->prev[0].cnp_tx_bytes = tmp_counters[0].cnp_tx_bytes;
		cnps->prev[0].cnp_rx_pkts = tmp_counters[0].cnp_rx_pkts;
		cnps->prev[0].cnp_rx_bytes = tmp_counters[0].cnp_rx_bytes;

		cnps->prev[1].cnp_tx_pkts = tmp_counters[1].cnp_tx_pkts;
		cnps->prev[1].cnp_tx_bytes = tmp_counters[1].cnp_tx_bytes;
		cnps->prev[1].cnp_rx_pkts = tmp_counters[1].cnp_rx_pkts;
		cnps->prev[1].cnp_rx_bytes = tmp_counters[1].cnp_rx_bytes;

		dstat->prev[0].tx_pkts = roce_only_tmp[0].tx_pkts;
		dstat->prev[0].tx_bytes = roce_only_tmp[0].tx_bytes;
		dstat->prev[0].rx_pkts = roce_only_tmp[0].rx_pkts;
		dstat->prev[0].rx_bytes = roce_only_tmp[0].rx_bytes;

		dstat->prev[1].tx_pkts = roce_only_tmp[1].tx_pkts;
		dstat->prev[1].tx_bytes = roce_only_tmp[1].tx_bytes;
		dstat->prev[1].rx_pkts = roce_only_tmp[1].rx_pkts;
		dstat->prev[1].rx_bytes = roce_only_tmp[1].rx_bytes;
	} else {
		u64 byte_mask, pkts_mask;
		u64 diff;

		byte_mask = bnxt_re_get_cfa_stat_mask(rdev->chip_ctx,
						      BYTE_MASK);
		pkts_mask = bnxt_re_get_cfa_stat_mask(rdev->chip_ctx,
						      PKTS_MASK);
		/*
		 * Calculate the number of cnp packets and use
		 * the value to calculate the CRC bytes.
		 * Multply pkts with 4 and add it to total bytes
		 */
		pkts = bnxt_re_stat_diff(tmp_counters[0].cnp_tx_pkts,
					 &cnps->prev[0].cnp_tx_pkts,
					 pkts_mask);
		cnps->cur[0].cnp_tx_pkts += pkts;
		diff = bnxt_re_stat_diff(tmp_counters[0].cnp_tx_bytes,
					 &cnps->prev[0].cnp_tx_bytes,
					 byte_mask);
		bytes = diff + pkts * 4;
		cnps->cur[0].cnp_tx_bytes += bytes;
		pkts = bnxt_re_stat_diff(tmp_counters[0].cnp_rx_pkts,
					 &cnps->prev[0].cnp_rx_pkts,
					 pkts_mask);
		cnps->cur[0].cnp_rx_pkts += pkts;
		bytes = bnxt_re_stat_diff(tmp_counters[0].cnp_rx_bytes,
					  &cnps->prev[0].cnp_rx_bytes,
					  byte_mask);
		cnps->cur[0].cnp_rx_bytes += bytes;

		/*
		 * Calculate the number of cnp packets and use
		 * the value to calculate the CRC bytes.
		 * Multply pkts with 4 and add it to total bytes
		 */
		pkts = bnxt_re_stat_diff(tmp_counters[1].cnp_tx_pkts,
					 &cnps->prev[1].cnp_tx_pkts,
					 pkts_mask);
		cnps->cur[1].cnp_tx_pkts += pkts;
		diff = bnxt_re_stat_diff(tmp_counters[1].cnp_tx_bytes,
					 &cnps->prev[1].cnp_tx_bytes,
					 byte_mask);
		cnps->cur[1].cnp_tx_bytes += diff + pkts * 4;
		pkts = bnxt_re_stat_diff(tmp_counters[1].cnp_rx_pkts,
					 &cnps->prev[1].cnp_rx_pkts,
					 pkts_mask);
		cnps->cur[1].cnp_rx_pkts += pkts;
		bytes = bnxt_re_stat_diff(tmp_counters[1].cnp_rx_bytes,
					  &cnps->prev[1].cnp_rx_bytes,
					  byte_mask);
		cnps->cur[1].cnp_rx_bytes += bytes;

		pkts = bnxt_re_stat_diff(roce_only_tmp[0].tx_pkts,
					 &dstat->prev[0].tx_pkts,
					 pkts_mask);
		dstat->cur[0].tx_pkts += pkts;
		diff = bnxt_re_stat_diff(roce_only_tmp[0].tx_bytes,
					 &dstat->prev[0].tx_bytes,
					 byte_mask);
		dstat->cur[0].tx_bytes += diff + pkts * 4;
		pkts = bnxt_re_stat_diff(roce_only_tmp[0].rx_pkts,
					 &dstat->prev[0].rx_pkts,
					 pkts_mask);
		dstat->cur[0].rx_pkts += pkts;

		bytes = bnxt_re_stat_diff(roce_only_tmp[0].rx_bytes,
					  &dstat->prev[0].rx_bytes,
					  byte_mask);
		dstat->cur[0].rx_bytes += bytes;
		pkts = bnxt_re_stat_diff(roce_only_tmp[1].tx_pkts,
					 &dstat->prev[1].tx_pkts,
					 pkts_mask);
		dstat->cur[1].tx_pkts += pkts;
		diff = bnxt_re_stat_diff(roce_only_tmp[1].tx_bytes,
					 &dstat->prev[1].tx_bytes,
					 byte_mask);
		dstat->cur[1].tx_bytes += diff + pkts * 4;
		pkts = bnxt_re_stat_diff(roce_only_tmp[1].rx_pkts,
					 &dstat->prev[1].rx_pkts,
					 pkts_mask);
		dstat->cur[1].rx_pkts += pkts;
		bytes = bnxt_re_stat_diff(roce_only_tmp[1].rx_bytes,
					  &dstat->prev[1].rx_bytes,
					  byte_mask);
		dstat->cur[1].rx_bytes += bytes;
	}
done:
	rtnl_unlock();
	return rc;
}

static void bnxt_re_copy_ext_stats(struct bnxt_re_dev *rdev,
				   u8 indx, struct bnxt_qplib_ext_stat *s)
{
	struct bnxt_re_ext_roce_stats *e_errs;
	struct bnxt_re_cnp_counters *cnp;
	struct bnxt_re_ext_rstat *ext_d;
	struct bnxt_re_ro_counters *ro;

	cnp = &rdev->stats.cnps.cur[indx];
	ro = &rdev->stats.dstat.cur[indx];
	ext_d = &rdev->stats.dstat.ext_rstat[indx];
	e_errs = &rdev->stats.dstat.e_errs;

	cnp->cnp_tx_pkts = s->tx_cnp;
	cnp->cnp_rx_pkts = s->rx_cnp;
	/* In bonding mode do not duplicate other stats */
	if (indx)
		return;
	cnp->ecn_marked = s->rx_ecn_marked;

	ro->tx_pkts = s->tx_roce_pkts;
	ro->tx_bytes = s->tx_roce_bytes;
	ro->rx_pkts = s->rx_roce_pkts;
	ro->rx_bytes = s->rx_roce_bytes;

	ext_d->tx.atomic_req = s->tx_atomic_req;
	ext_d->tx.read_req = s->tx_read_req;
	ext_d->tx.read_resp = s->tx_read_res;
	ext_d->tx.write_req = s->tx_write_req;
	ext_d->tx.send_req = s->tx_send_req;
	ext_d->rx.atomic_req = s->rx_atomic_req;
	ext_d->rx.read_req = s->rx_read_req;
	ext_d->rx.read_resp = s->rx_read_res;
	ext_d->rx.write_req = s->rx_write_req;
	ext_d->rx.send_req = s->rx_send_req;
	ext_d->grx.rx_pkts = s->rx_roce_good_pkts;
	ext_d->grx.rx_bytes = s->rx_roce_good_bytes;
	e_errs->oob = s->rx_out_of_buffer;
	e_errs->oos = s->rx_out_of_sequence;
}

static int bnxt_re_get_ext_stat(struct bnxt_re_dev *rdev)
{
	struct bnxt_qplib_ext_stat estat[2] = {{}, {}};
	u32 fid;
	int rc;

	fid = PCI_FUNC(rdev->en_dev->pdev->devfn);
	rc = bnxt_qplib_qext_stat(&rdev->rcfw, fid, &estat[0]);
	if (rc)
		goto done;
	bnxt_re_copy_ext_stats(rdev, 0, &estat[0]);

	if (rdev->binfo) {
		struct bnxt *slave2_bp = netdev_priv(rdev->binfo->slave2);

		fid = PCI_FUNC(slave2_bp->pdev->devfn);
		rc = bnxt_qplib_qext_stat(&rdev->rcfw, fid, &estat[1]);
		if (rc)
			goto done;
		bnxt_re_copy_ext_stats(rdev, 1, &estat[1]);
	}
done:
	return rc;
}

static void bnxt_re_copy_rstat(struct bnxt_re_rdata_counters *d,
			       struct hwrm_stat_ctx_query_output *s)
{
	d->tx_ucast_pkts = le64_to_cpu(s->tx_ucast_pkts);
	d->tx_mcast_pkts = le64_to_cpu(s->tx_mcast_pkts);
	d->tx_bcast_pkts = le64_to_cpu(s->tx_bcast_pkts);
	d->tx_discard_pkts = le64_to_cpu(s->tx_discard_pkts);
	d->tx_error_pkts = le64_to_cpu(s->tx_error_pkts);
	d->tx_ucast_bytes = le64_to_cpu(s->tx_ucast_bytes);
	/* Add four bytes of CRC bytes per packet */
	d->tx_ucast_bytes +=  d->tx_ucast_pkts * 4;
	d->tx_mcast_bytes = le64_to_cpu(s->tx_mcast_bytes);
	d->tx_bcast_bytes = le64_to_cpu(s->tx_bcast_bytes);
	d->rx_ucast_pkts = le64_to_cpu(s->rx_ucast_pkts);
	d->rx_mcast_pkts = le64_to_cpu(s->rx_mcast_pkts);
	d->rx_bcast_pkts = le64_to_cpu(s->rx_bcast_pkts);
	d->rx_discard_pkts = le64_to_cpu(s->rx_discard_pkts);
	d->rx_error_pkts = le64_to_cpu(s->rx_error_pkts);
	d->rx_ucast_bytes = le64_to_cpu(s->rx_ucast_bytes);
	d->rx_mcast_bytes = le64_to_cpu(s->rx_mcast_bytes);
	d->rx_bcast_bytes = le64_to_cpu(s->rx_bcast_bytes);
	d->rx_agg_pkts = le64_to_cpu(s->rx_agg_pkts);
	d->rx_agg_bytes = le64_to_cpu(s->rx_agg_bytes);
	d->rx_agg_events = le64_to_cpu(s->rx_agg_events);
	d->rx_agg_aborts = le64_to_cpu(s->rx_agg_aborts);
}

static int bnxt_re_get_roce_data_stats(struct bnxt_re_dev *rdev)
{
	struct hwrm_stat_ctx_query_output resp = {};
	struct hwrm_stat_ctx_query_input req = {};
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	struct bnxt_re_rdata_counters *rstat;
	struct bnxt_qplib_stats *stat_ctx;
	struct bnxt_fw_msg fw_msg = {};
	int rc = 0;

	/* FIXME Workaround to avoid system hang when this
	 * thread is competing with device create/destroy
	 * sequence
	 */
	stat_ctx = &rdev->qplib_res.hctx->stats;
	if (!bnxt_re_rtnl_trylock())
		/* Not querying stats. Return older values */
		return 0;

	bnxt_re_init_hwrm_hdr(rdev, (void *)&req, HWRM_STAT_CTX_QUERY, -1, -1);
	req.stat_ctx_id = cpu_to_le32(stat_ctx->fw_id);
	bnxt_re_fill_fw_msg(&fw_msg, (void *)&req, sizeof(req), (void *)&resp,
			    sizeof(resp), DFLT_HWRM_CMD_TIMEOUT);
	rc = en_dev->en_ops->bnxt_send_fw_msg(en_dev, BNXT_ROCE_ULP, &fw_msg);
	if (rc) {
		dev_err(rdev_to_dev(rdev),
			"Failed to get roce data stats : rc = 0x%x", rc);
		rc = 0;
		goto done;
	}

	rstat = &rdev->stats.dstat.rstat[0];
	bnxt_re_copy_rstat(rstat, &resp);

	/* Query second port if LAG is enabled */
	if (rdev->binfo) {
		struct bnxt *second_bp = netdev_priv(rdev->binfo->slave2);
		struct bnxt_qplib_stats *stat_ctx;

		stat_ctx = &rdev->qplib_res.hctx->stats2;
		memset(&req, 0, sizeof(req));
		memset(&fw_msg, 0, sizeof(fw_msg));

		bnxt_re_init_hwrm_hdr(rdev, (void *)&req, HWRM_STAT_CTX_QUERY,
				      -1, PCI_FUNC(second_bp->pdev->devfn) +
				      1);
		req.stat_ctx_id = cpu_to_le32(stat_ctx->fw_id);
		bnxt_re_fill_fw_msg(&fw_msg, (void *)&req, sizeof(req),
				    (void *)&resp, sizeof(resp),
				    DFLT_HWRM_CMD_TIMEOUT);
		rc = en_dev->en_ops->bnxt_send_fw_msg(en_dev, BNXT_ROCE_ULP,
						      &fw_msg);
		if (rc) {
			dev_err(rdev_to_dev(rdev),
				"Failed to get roce data stats : rc = 0x%x",
				rc);
			rc = 0;
			goto done;
		}

		rstat = &rdev->stats.dstat.rstat[1];
		bnxt_re_copy_rstat(rstat, &resp);
	}
done:
	rtnl_unlock();

	return rc;
}

int bnxt_re_get_device_stats(struct bnxt_re_dev *rdev)
{
	struct bnxt *bp = netdev_priv(rdev->en_dev->net);
	struct bnxt_qplib_query_stats_info sinfo;
	struct rtnl_link_stats64 nstat = {0};
	struct bnxt_re_l2_counters *l2stat;
	int rc = 0;

	/* Stats are in 1s cadence */
	if (test_bit(BNXT_RE_FLAG_ISSUE_CFA_FLOW_STATS, &rdev->flags)) {
		if (_is_ext_stats_supported(rdev->dev_attr->dev_cap_flags) &&
		    !rdev->is_virtfn)
			rc = bnxt_re_get_ext_stat(rdev);
		else
			rc = bnxt_re_get_qos_stats(rdev);
		if (rc)
			clear_bit(BNXT_RE_FLAG_ISSUE_CFA_FLOW_STATS,
				  &rdev->flags);
	}

	if (test_bit(BNXT_RE_FLAG_ISSUE_ROCE_STATS, &rdev->flags)) {
		rc = bnxt_re_get_roce_data_stats(rdev);
		if (rc)
			clear_bit(BNXT_RE_FLAG_ISSUE_ROCE_STATS,
				  &rdev->flags);

		/* Set default values for sinfo */
		sinfo.function_id = 0xFFFFFFFF;
		sinfo.collection_id = 0xFF;
		sinfo.vf_valid  = false;

		rc = bnxt_qplib_get_roce_error_stats(&rdev->rcfw,
						     &rdev->stats.dstat.errs,
						     &sinfo);
		if (rc)
			clear_bit(BNXT_RE_FLAG_ISSUE_ROCE_STATS,
				  &rdev->flags);
	}

	/* FIXME Workaround to avoid system hang when this
	 * thread is competing with device create/destroy
	 * sequence
	 */
	if (!bnxt_re_rtnl_trylock())
		/* Not querying stats. Return older values */
		return 0;
	/* Retrieve L2 counters*/
	l2stat = rdev->stats.l2stat;
	bnxt_re_l2_dev_get_stats(bp->dev, &nstat);

	l2stat[0].tx_bytes = nstat.tx_bytes;
	l2stat[0].tx_packets = nstat.tx_packets;
	l2stat[0].rx_bytes = nstat.rx_bytes;
	l2stat[0].rx_packets = nstat.rx_packets;
	if (rdev->binfo) {
		bnxt_re_l2_dev_get_stats(rdev->binfo->slave2, &nstat);
		l2stat[1].tx_bytes = nstat.tx_bytes;
		l2stat[1].tx_packets = nstat.tx_packets;
		l2stat[1].rx_bytes = nstat.rx_bytes;
		l2stat[1].rx_packets = nstat.rx_packets;
	}

	rtnl_unlock();
	return rc;
}
