/*
 * Copyright (c) 2015-2020, Broadcom. All rights reserved.  The term
 * Broadcom refers to Broadcom Limited and/or its subsidiaries.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * BSD license below:
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Author: Eddie Wai <eddie.wai@broadcom.com>
 *
 * Description: Main component of the bnxt_re driver
 */

#include "bnxt_re.h"
#include <linux/inetdevice.h>

#include "configfs.h"
#ifdef ENABLE_DEBUGFS
#include "debugfs.h"
#endif

#include "ib_verbs.h"
#include "bnxt_re-abi.h"
#include "dcb.h"
/* bnxt_en.h includes */
#include "bnxt.h"

#include <linux/if_bonding.h>

static char version[] =
		"Broadcom NetXtreme-C/E RoCE Driver " ROCE_DRV_MODULE_NAME \
		" v" ROCE_DRV_MODULE_VERSION " (" ROCE_DRV_MODULE_RELDATE ")\n";

#define BNXT_RE_DESC	"Broadcom NetXtreme RoCE"

MODULE_AUTHOR("Eddie Wai <eddie.wai@broadcom.com>");
MODULE_DESCRIPTION(BNXT_RE_DESC " Driver");
MODULE_LICENSE("Dual BSD/GPL");
MODULE_VERSION(ROCE_DRV_MODULE_VERSION);

static unsigned int restrict_mrs = 0;
module_param(restrict_mrs, uint, 0);
MODULE_PARM_DESC(restrict_mrs, " Restrict the no. of MRs 0 = 256K , 1 = 64K");

unsigned int restrict_stats = 0;
module_param(restrict_stats, uint, 0);
MODULE_PARM_DESC(restrict_stats, "Restrict stats query frequency to ethtool coalesce value. Disabled by default");

unsigned int enable_fc = 1;
module_param(enable_fc, uint, 0);
MODULE_PARM_DESC(enable_fc, "Enable default PFC, CC,ETS during driver load. 1 - fc enable, 0 - fc disable - Default is 1");

unsigned int roce_mode = 2;
/* globals */
struct list_head bnxt_re_dev_list = LIST_HEAD_INIT(bnxt_re_dev_list);
u32 adapter_count;
DEFINE_MUTEX(bnxt_re_dev_lock);
static u32 gmod_exit;
static u32 gadd_dev_inprogress;

/* Global variable to handle bond creation after L2 bond is created */
bool delayed_bond_task_scheduled;

struct list_head bnxt_re_recovery_snapshot_list =
	LIST_HEAD_INIT(bnxt_re_recovery_snapshot_list);
static struct list_head bnxt_re_bond_list = LIST_HEAD_INIT(bnxt_re_bond_list);

static void bnxt_re_task(struct work_struct *work_task);
static struct workqueue_struct *bnxt_re_wq;
static void bnxt_re_ib_uninit(struct bnxt_re_dev *rdev);
static void bnxt_re_dev_uninit(struct bnxt_re_dev *rdev, u8 op_type);

static bool bnxt_re_bond_update_reqd(struct netdev_bonding_info *netdev_binfo,
				     struct bnxt_re_bond_info *binfo,
				     struct bnxt_re_dev *rdev,
				     struct net_device *sl_netdev);
static int bnxt_re_update_fw_lag_info(struct bnxt_re_bond_info *binfo,
			       struct bnxt_re_dev *rdev,
			       bool bond_mode);
static int bnxt_re_query_hwrm_intf_version(struct bnxt_re_dev *rdev);

static int bnxt_re_hwrm_qcfg(struct bnxt_re_dev *rdev, u32 *db_len,
			     u32 *offset);

static int bnxt_re_hwrm_pri2cos_qcfg(struct bnxt_re_dev *rdev,
				     struct bnxt_re_tc_rec *tc_rec,
				     u16 target_id,
				     u8 *prio_valid,
				     u8 *roce_prio, u8 *cnp_prio);

static int bnxt_re_hwrm_qcaps(struct bnxt_re_dev *rdev, u8 *enabled);
void bnxt_re_clear_dcbx_cc_param(struct bnxt_re_dev *rdev);

static const char *bnxt_re_event2str[] = {"NONE", "UP", "DOWN", "REBOOT",
				   "CHANGE", "REGISTER", "UNREGISTER",
				   "CHANGEMTU", "CHANGEADDR", "GOING_DOWN",
				   "CHANGENAME", "FEAT_CHANGE",
				   "BONDING_FAILOVER", "PRE_UP",
				   "PRE_TYPE_CHANGE", "POST_TYPE_CHANGE",
				   "POST_INIT", "UNREGISTER_FINAL", "RELEASE",
				   "NOTIFY_PEERS", "JOIN", "CHANGEUPPER",
				   "RESEND_IGMP", "PRECHANGEMTU",
				   "CHANGEINFODATA", "BONDING_INFO",
				   "PRECHANGEUPPER", "CHANGELOWERSTATE",
				   "UDP_TUNNEL_PUSH_INFO", "UDP_TUNNEL_DROP_INFO",
				   "CHANGE_TX_QUEUE_LEN", "CVLAN_FILTER_PUSH_INFO",
				   "CVLAN_FILTER_DROP_INFO", "SVLAN_FILTER_PUSH_INFO",
				   "SVLAN_FILTER_DROP_INFO",
				   "UNKNOWN"};

#define NETDEV_UNKNOWN	35

/* SR-IOV helper functions */

static void bnxt_re_get_sriov_func_type(struct bnxt_re_dev *rdev)
{
	struct bnxt *bp;

	if (rdev->binfo)
		return;

	bp = netdev_priv(rdev->en_dev->net);
	if (BNXT_VF(bp))
		rdev->is_virtfn = 1;
}

/* Set the maximum number of each resource that the driver actually wants
 * to allocate. This may be up to the maximum number the firmware has
 * reserved for the function. The driver may choose to allocate fewer
 * resources than the firmware maximum.
 */
static void bnxt_re_limit_pf_res(struct bnxt_re_dev *rdev)
{
	struct bnxt_qplib_dev_attr *attr;
	struct bnxt_qplib_ctx *hctx;
	int i;

	attr = rdev->dev_attr;
	hctx = rdev->qplib_res.hctx;

	if (!_is_chip_gen_p5(rdev->chip_ctx)) {
		hctx->qp_ctx.max = min_t(u32, BNXT_RE_MAX_QPC_COUNT,
					 attr->max_qp);
		hctx->mrw_ctx.max = restrict_mrs ? BNXT_RE_MAX_MRW_COUNT_64K :
						   BNXT_RE_MAX_MRW_COUNT_256K;
		/* Use max_mr from fw since max_mrw does not get set */
		hctx->mrw_ctx.max = min_t(u32, hctx->mrw_ctx.max, attr->max_mr);
		hctx->srq_ctx.max = min_t(u32, BNXT_RE_MAX_SRQC_COUNT,
					  attr->max_srq);
		hctx->cq_ctx.max = min_t(u32, BNXT_RE_MAX_CQ_COUNT,
					 attr->max_cq);
		for (i = 0; i < MAX_TQM_ALLOC_REQ; i++)
			hctx->tqm_ctx.qcount[i] = attr->tqm_alloc_reqs[i];
	} else {
		hctx->qp_ctx.max = attr->max_qp ? attr->max_qp :
						  BNXT_RE_MAX_QPC_COUNT;
		hctx->mrw_ctx.max = attr->max_mr ? attr->max_mr :
						   BNXT_RE_MAX_MRW_COUNT_256K;
		hctx->srq_ctx.max = attr->max_srq ? attr->max_srq :
						    BNXT_RE_MAX_SRQC_COUNT;
		hctx->cq_ctx.max = attr->max_cq ? attr->max_cq :
						  BNXT_RE_MAX_CQ_COUNT;
	}
}

static void bnxt_re_limit_vf_res(struct bnxt_qplib_ctx *hctx, u32 num_vf)
{
	struct bnxt_qplib_vf_res *vf_res;
	u32 mrws = 0;
	u32 vf_pct;
	u32 nvfs;

	vf_res = &hctx->vf_res;
	/*
	 * Reserve a set of resources for the PF. Divide the remaining
	 * resources among the VFs
	 */
	vf_pct = 100 - BNXT_RE_PCT_RSVD_FOR_PF;
	nvfs = num_vf;
	num_vf = 100 * num_vf;
	/* TODO Remove this workaround when  CTRL-53556 is fixed */
#ifndef BNXT_RE_MAX_VF_QPS_PER_PF
	vf_res->max_qp = (hctx->qp_ctx.max * vf_pct) / num_vf;
#else
	vf_res->max_qp = BNXT_RE_MAX_VF_QPS_PER_PF / nvfs;
#endif

	vf_res->max_srq = (hctx->srq_ctx.max * vf_pct) / num_vf;
	vf_res->max_cq = (hctx->cq_ctx.max * vf_pct) / num_vf;
	/*
	 * The driver allows many more MRs than other resources. If the
	 * firmware does also, then reserve a fixed amount for the PF and
	 * divide the rest among VFs. VFs may use many MRs for NFS
	 * mounts, ISER, NVME applications, etc. If the firmware severely
	 * restricts the number of MRs, then let PF have half and divide
	 * the rest among VFs, as for the other resource types.
	 */
	if (hctx->mrw_ctx.max < BNXT_RE_MAX_MRW_COUNT_64K) {
		mrws = hctx->mrw_ctx.max * vf_pct;
		nvfs = num_vf;
	} else {
		mrws = hctx->mrw_ctx.max - BNXT_RE_RESVD_MR_FOR_PF;
	}
	vf_res->max_mrw = (mrws / nvfs);
	vf_res->max_gid = BNXT_RE_MAX_GID_PER_VF;
}

static void bnxt_re_set_resource_limits(struct bnxt_re_dev *rdev)
{
	struct bnxt_qplib_ctx *hctx;

	hctx = rdev->qplib_res.hctx;
	memset(&hctx->vf_res, 0, sizeof(struct bnxt_qplib_vf_res));
	bnxt_re_limit_pf_res(rdev);

	if (rdev->num_vfs)
		bnxt_re_limit_vf_res(hctx, rdev->num_vfs);
}

static void bnxt_re_dettach_irq(struct bnxt_re_dev *rdev)
{
	struct bnxt_qplib_rcfw *rcfw = NULL;
	struct bnxt_qplib_nq *nq;
	int indx;

	rcfw = &rdev->rcfw;
	for (indx = 0; indx < rdev->nqr.max_init; indx++) {
		nq = &rdev->nqr.nq[indx];
		bnxt_qplib_nq_stop_irq(nq, false);
	}

	bnxt_qplib_rcfw_stop_irq(rcfw, false);
}

static void bnxt_re_detach_err_device(struct bnxt_re_dev *rdev,
				      struct bnxt_re_pre_recovery_snapshot *rec)
{
	/* Delete from the recovery list. Device wont be restarted */
	mutex_lock(&bnxt_re_dev_lock);
	list_del(&rec->list);
	mutex_unlock(&bnxt_re_dev_lock);
	kfree(rec);
	/* Free the MSIx vectors only so that L2 can proceed with MSIx disable */
	bnxt_re_dettach_irq(rdev);

	/* Set the state as detached to prevent sending any more commands */
	set_bit(ERR_DEVICE_DETACHED, &rdev->rcfw.cmdq.flags);
	set_bit(BNXT_RE_FLAG_ERR_DEVICE_DETACHED, &rdev->flags);
}

int bnxt_re_get_pri_dscp_settings(struct bnxt_re_dev *rdev,
				  u16 target_id,
				  struct bnxt_re_tc_rec *tc_rec)
{
	struct bnxt_re_dscp2pri d2p[8] = {};
	u16 count = 8;
	int rc = 0;
	int i;

	tc_rec->prio_valid = 0;
	tc_rec->dscp_valid = 0;
	rc = bnxt_re_hwrm_pri2cos_qcfg(rdev, tc_rec, target_id,
				       &tc_rec->prio_valid,
				       &tc_rec->roce_prio,
				       &tc_rec->cnp_prio);
	if (rc)
		return rc;

	rc = bnxt_re_query_hwrm_dscp2pri(rdev, d2p, &count, target_id);

	for (i = 0; i < count; i++) {
		if (d2p[i].pri == tc_rec->roce_prio) {
			tc_rec->roce_dscp = d2p[i].dscp;
			tc_rec->dscp_valid |= (1 << ROCE_DSCP_VALID);
		} else if (d2p[i].pri == tc_rec->cnp_prio) {
			tc_rec->cnp_dscp = d2p[i].dscp;
			tc_rec->dscp_valid |= (1 << CNP_DSCP_VALID);
		}
	}

	return rc;
}

struct bnxt_re_dcb_work {
	struct work_struct work;
	struct bnxt_re_dev *rdev;
	struct hwrm_async_event_cmpl cmpl;
};

void bnxt_re_init_dcb_wq(struct bnxt_re_dev *rdev)
{
	rdev->dcb_wq = create_singlethread_workqueue("bnxt_re_dcb_wq");
}

void bnxt_re_uninit_dcb_wq(struct bnxt_re_dev *rdev)
{
	if (!rdev->dcb_wq)
		return;
	flush_workqueue(rdev->dcb_wq);
	destroy_workqueue(rdev->dcb_wq);
	rdev->dcb_wq = NULL;
}

void bnxt_re_dcb_wq_task(struct work_struct *work)
{
	struct bnxt_qplib_cc_param *cc_param;
	struct bnxt_re_tc_rec *tc_rec;
	struct bnxt_re_dev *rdev;
	struct bnxt_re_dcb_work *dcb_work =
			container_of(work, struct bnxt_re_dcb_work, work);

	rdev = dcb_work->rdev;

	if (!rdev)
		goto exit;

	cc_param = &rdev->cc_param;
	tc_rec = &rdev->tc_rec[0];

	if (!bnxt_re_get_pri_dscp_settings(rdev, 0xFFFF, tc_rec)) {
		cc_param->alt_tos_dscp = tc_rec->cnp_dscp;
		cc_param->alt_vlan_pcp = tc_rec->cnp_prio;
		cc_param->tos_dscp = tc_rec->roce_dscp;
		cc_param->roce_pri = tc_rec->roce_prio;
	}
exit:
	kfree(dcb_work);
}

static void bnxt_re_async_notifier(void *handle, struct hwrm_async_event_cmpl *cmpl)
{
	struct net_device *netdev = (struct net_device *)handle;
	struct bnxt_re_dev *rdev = bnxt_re_from_netdev(netdev);
	struct bnxt_re_dcb_work *dcb_work;

	u16 event_id = le16_to_cpu(cmpl->event_id);
	u32 data1 = le32_to_cpu(cmpl->event_data1);
	u32 data2 = le32_to_cpu(cmpl->event_data2);

	if (!rdev)
		return;

	dev_dbg(rdev_to_dev(rdev), "Async event_id = %d data1 = %d data2 = %d",
		event_id, data1, data2);
	switch (event_id) {
	case ASYNC_EVENT_CMPL_EVENT_ID_DCB_CONFIG_CHANGE:
		/* Not handling the event in older FWs */
		if (!is_qport_service_type_supported(rdev))
			break;
		dcb_work = kzalloc(sizeof(*dcb_work), GFP_ATOMIC);
		if (!dcb_work)
			break;

		dcb_work->rdev = rdev;
		memcpy(&dcb_work->cmpl, cmpl, sizeof(*cmpl));
		INIT_WORK(&dcb_work->work, bnxt_re_dcb_wq_task);
		queue_work(rdev->dcb_wq, &dcb_work->work);
		break;
	default:
		break;
	}
}

static void bnxt_re_stop(void *handle)
{
	struct bnxt_re_pre_recovery_snapshot *recovery_item = NULL;
	struct net_device *netdev = (struct net_device *)handle;
	struct bnxt_re_dev *rdev = bnxt_re_from_netdev(netdev);
	struct bnxt *bp = netdev_priv(netdev);
	int rc = 0;

	if (!rdev)
		return;
	if (test_bit(BNXT_RE_FLAG_ERR_DEVICE_DETACHED, &rdev->flags))
		return;
	recovery_item = kzalloc(sizeof(*recovery_item), GFP_KERNEL);
	if (!recovery_item)
		return;
	recovery_item->netdev = netdev;
	recovery_item->en_dev = rdev->en_dev;
	recovery_item->wqe_mode = rdev->chip_ctx->modes.wqe_mode;
	recovery_item->gsi_mode = rdev->gsi_ctx.gsi_qp_mode;
	init_waitqueue_head(&recovery_item->waitq);

	mutex_lock(&bnxt_re_dev_lock);
	list_add_tail(&recovery_item->list, &bnxt_re_recovery_snapshot_list);
	mutex_unlock(&bnxt_re_dev_lock);

	/*
	 * Check if fw has undergone reset or is in a fatal condition.
	 * If so, set flags so that no further commands are sent down to FW
	 */
	if (test_bit(BNXT_STATE_FW_FATAL_COND, &bp->state) ||
	    test_bit(BNXT_STATE_FW_RESET_DET, &bp->state)) {
		/* Set rcfw flag to control commands send to Bono */
		set_bit(ERR_DEVICE_DETACHED, &rdev->rcfw.cmdq.flags);
		/* Set bnxt_re flag to control commands send via L2 driver */
		set_bit(BNXT_RE_FLAG_ERR_DEVICE_DETACHED, &rdev->flags);
	}

	dev_info(rdev_to_dev(rdev), "%s: L2 driver notified to stop."
		 "Attempting to stop and Dispatching event "
		 "to inform the stack\n", __func__);
	/* Schedule a work item to handle IB UNINIT for recovery */
	bnxt_re_schedule_work(rdev, NETDEV_UNREGISTER,
			      NULL, NULL, NULL, recovery_item->netdev);

	rtnl_unlock();
	rc = wait_event_timeout(recovery_item->waitq,
				recovery_item->ib_uninit_done,
				msecs_to_jiffies(BNXT_RE_RECOVERY_IB_UNINIT_WAIT_TIME_MS));
	rtnl_lock();
	/* TODO: Handle return values when bnxt_en supports */
	if (!rc) {
		dev_info(rdev_to_dev(rdev), "%s: Attempt to recover failed\n",
			 __func__);
		bnxt_re_detach_err_device(rdev, recovery_item);
		return;
	}
	bnxt_re_remove_device(rdev, BNXT_RE_PRE_RECOVERY_REMOVE);

	/* TODO: Handle return values when bnxt_en supports */
	return;
}

static void bnxt_re_start(void *handle)
{
	struct bnxt_re_pre_recovery_snapshot *recovery_item = NULL;
	struct net_device *netdev = (struct net_device *)handle;
	struct bnxt_re_dev *rdev = NULL;
	struct net_device *real_dev;
	struct bnxt_en_dev *en_dev;
	u8 found = false;
	int rc;

	/* TODO: Handle return values
	 * when bnxt_en supports it
	 */
	mutex_lock(&bnxt_re_dev_lock);
	list_for_each_entry(recovery_item,
			    &bnxt_re_recovery_snapshot_list, list) {
		if (recovery_item->netdev == netdev) {
			list_del(&recovery_item->list);
			found = true;
			break;
		}
	}
	mutex_unlock(&bnxt_re_dev_lock);
	if (found) {
		en_dev = recovery_item->en_dev;
		real_dev = rdma_vlan_dev_real_dev(netdev);
		if (!real_dev)
			real_dev = netdev;
		rc = bnxt_re_add_device(&rdev, real_dev, NULL,
					recovery_item->gsi_mode,
					BNXT_RE_POST_RECOVERY_INIT,
					recovery_item->wqe_mode);
		if (rc) {
			/* Add device failed. Unregister the device.
			 * This has to be done explicitly as
			 * bnxt_re_stop would not have unregistered
			 */
			en_dev->en_ops->bnxt_unregister_device(en_dev,
							       BNXT_ROCE_ULP);
			kfree(recovery_item);
			mutex_lock(&bnxt_re_dev_lock);
			gadd_dev_inprogress--;
			mutex_unlock(&bnxt_re_dev_lock);
			return;
		}
		bnxt_re_get_link_speed(rdev);
		bnxt_re_schedule_work(rdev, NETDEV_REGISTER,
				      NULL, NULL, NULL, NULL);
		kfree(recovery_item);
	}
}

static void bnxt_re_sriov_config(void *p, int num_vfs)
{
	struct bnxt_re_dev *rdev = bnxt_re_from_netdev(p);
	int retry_count = BNXT_RE_SRIOV_CFG_TIMEOUT;

        if (!rdev)
                return;

	if (test_bit(BNXT_RE_FLAG_ERR_DEVICE_DETACHED, &rdev->flags))
		return;

	while (--retry_count && !(bnxt_re_rdev_ready(rdev)))
		msleep(1000);

	if (!retry_count) {
		pr_err("%s: Failed to set VF resources\n", __func__);
		return;
	}

	if (rdev->binfo) {
		dev_info(rdev_to_dev(rdev),
			 "RoCE LAG enabled .. RoCE disabled on VFs");
		return;
	}
	rdev->num_vfs = num_vfs;
	bnxt_re_set_resource_limits(rdev);
	bnxt_qplib_set_func_resources(&rdev->qplib_res);
}

static void bnxt_re_shutdown(void *p)
{
	struct bnxt_re_dev *rdev = bnxt_re_from_netdev(p);

	if (!rdev)
		return;

	bnxt_re_ib_uninit(rdev);
	/* rtnl_lock held by L2 before coming here */
	bnxt_re_dev_uninit(rdev, BNXT_RE_COMPLETE_REMOVE);
}

static void bnxt_re_stop_irq(void *handle)
{
	struct bnxt_re_dev *rdev = bnxt_re_from_netdev(handle);
	struct bnxt_qplib_rcfw *rcfw = NULL;
	struct bnxt_qplib_nq *nq;
	int indx;

	if (!rdev)
		return;
	rcfw = &rdev->rcfw;
	for (indx = 0; indx < rdev->nqr.max_init; indx++) {
		nq = &rdev->nqr.nq[indx];
		bnxt_qplib_nq_stop_irq(nq, false);
	}

	bnxt_qplib_rcfw_stop_irq(rcfw, false);
}

static void bnxt_re_start_irq(void *handle, struct bnxt_msix_entry *ent)
{
	struct bnxt_re_dev *rdev = bnxt_re_from_netdev(handle);
	struct bnxt_msix_entry *msix_ent = NULL;
	struct bnxt_qplib_rcfw *rcfw = NULL;
	struct bnxt_qplib_nq *nq;
	int indx, rc, vec;

	if (!rdev)
		return;
	if (test_bit(BNXT_RE_FLAG_ERR_DEVICE_DETACHED, &rdev->flags))
		return;
	msix_ent = rdev->nqr.msix_entries;
	rcfw = &rdev->rcfw;

	if (!ent) {
		/* Not setting the f/w timeout bit in rcfw.
		 * During the driver unload the first command
		 * to f/w will timout and that will set the
		 * timeout bit.
		 */
		dev_err(rdev_to_dev(rdev), "Failed to re-start IRQs\n");
		return;
	}

	/* Vectors may change after restart, so update with new vectors
	 * in device sctructure.
	 */
	for (indx = 0; indx < rdev->nqr.num_msix; indx++)
		rdev->nqr.msix_entries[indx].vector = ent[indx].vector;

	rc = bnxt_qplib_rcfw_start_irq(rcfw, msix_ent[BNXT_RE_AEQ_IDX].vector,
				       false);
	if (rc) {
		dev_warn(rdev_to_dev(rdev),
			 "Failed to reinit CREQ\n");
		return;
	}
	for (indx = 0 ; indx < rdev->nqr.max_init; indx++) {
		nq = &rdev->nqr.nq[indx];
		vec = indx + 1;
		rc = bnxt_qplib_nq_start_irq(nq, indx, msix_ent[vec].vector,
					     false);
		if (rc) {
			dev_warn(rdev_to_dev(rdev),
				 "Failed to reinit NQ index %d\n", indx);
			return;
		}
	}
}

/*
 * Except for ulp_async_notifier and ulp_sriov_cfg, the remaining ulp_ops
 * below are called with rtnl_lock held
 */
static struct bnxt_ulp_ops bnxt_re_ulp_ops = {
	.ulp_async_notifier = bnxt_re_async_notifier,
	.ulp_stop = bnxt_re_stop,
	.ulp_start = bnxt_re_start,
	.ulp_sriov_config = bnxt_re_sriov_config,
	.ulp_shutdown = bnxt_re_shutdown,
	.ulp_irq_stop = bnxt_re_stop_irq,
	.ulp_irq_restart = bnxt_re_start_irq
};

static inline const char *bnxt_re_netevent(unsigned long event)
{
	if (event >= ARRAY_SIZE(bnxt_re_event2str))
		event = NETDEV_UNKNOWN;
	return bnxt_re_event2str[(event)];
}

/* RoCE -> Net driver */

/* Driver registration routines used to let the networking driver (bnxt_en)
 * to know that the RoCE driver is now installed */
static int bnxt_re_unregister_netdev(struct bnxt_re_dev *rdev)
{
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	int rc;

	rc = en_dev->en_ops->bnxt_unregister_device(rdev->en_dev,
						    BNXT_ROCE_ULP);
	if (rc)
		dev_err(rdev_to_dev(rdev), "netdev %p unregister failed!",
			rdev->en_dev->net);

	return 0;
}

static int bnxt_re_register_netdev(struct bnxt_re_dev *rdev)
{
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	int rc = 0;

	rc = en_dev->en_ops->bnxt_register_device(en_dev,
						  BNXT_ROCE_ULP,
						  &bnxt_re_ulp_ops,
						  rdev->netdev);
	if (rc) {
		dev_err(rdev_to_dev(rdev), "netdev %p register failed!",
			rdev->netdev);
		goto done;
	}
	dev_dbg(rdev_to_dev(rdev), "REGISTER NETDEV %p hold!", rdev->netdev);
done:
	return rc;
}

u8 bnxt_re_get_db_push_status(struct bnxt_re_dev *rdev)
{
	u8 enabled = false;

	if (bnxt_re_hwrm_qcaps(rdev, &enabled))
		dev_info(rdev_to_dev(rdev),
			 "Low latency framework is disabled\n");
	return enabled;
}

void bnxt_re_set_db_offset(struct bnxt_re_dev *rdev)
{
	struct bnxt_qplib_chip_ctx *cctx;
	struct bnxt_en_dev *en_dev;
	struct bnxt_qplib_res *res;
	u32 l2db_len = 0;
	u32 offset = 0;
	u32 barlen;
	int rc;

	res = &rdev->qplib_res;
	en_dev = rdev->en_dev;
	cctx = rdev->chip_ctx;

	/* Issue qcfg */
	rc = bnxt_re_hwrm_qcfg(rdev, &l2db_len, &offset);
	if (rc)
		dev_info(rdev_to_dev(rdev),
			 "Couldn't get DB bar size, Low latency framework is disabled\n");
	/* set register offsets for both UC and WC */
	if (_is_chip_sr2(cctx))
		res->dpi_tbl.ucreg.offset = offset;
	else
		res->dpi_tbl.ucreg.offset = res->is_vf ? BNXT_QPLIB_DBR_VF_DB_OFFSET :
							 BNXT_QPLIB_DBR_PF_DB_OFFSET;
	res->dpi_tbl.wcreg.offset = res->dpi_tbl.ucreg.offset;

	/* If WC mapping is disabled by L2 driver then en_dev->l2_db_size
	 * is equal to the DB-Bar actual size. This indicates that L2
	 * is mapping entire bar as UC-. RoCE driver can't enable WC mapping
	 * in such cases and DB-push will be disabled.
	 */
	barlen = pci_resource_len(res->pdev, RCFW_DBR_PCI_BAR_REGION);
	if (cctx->modes.db_push && l2db_len && en_dev->l2_db_size != barlen) {
		res->dpi_tbl.wcreg.offset = en_dev->l2_db_size;
		dev_info(rdev_to_dev(rdev),
			 "Low latency framework is enabled\n");
	}

	return;
}

void bnxt_re_set_drv_mode(struct bnxt_re_dev *rdev, u8 mode)
{
	struct bnxt_qplib_chip_ctx *cctx;
	struct bnxt_en_dev *en_dev;
	struct bnxt *bp;

	en_dev = rdev->en_dev;
	bp = netdev_priv(en_dev->net);
	cctx = rdev->chip_ctx;
	cctx->modes.wqe_mode = _is_chip_gen_p5(rdev->chip_ctx) ?
					mode : BNXT_QPLIB_WQE_MODE_STATIC;
	cctx->modes.te_bypass = false;
	cctx->modes.db_push = bnxt_re_get_db_push_status(rdev);
	 /*
	  * TODO: Need a better mechanism for spreading of the
	  * 512 extended PPP pages in the presence of VF and
	  * NPAR, until then not enabling push
	  */
	if (_is_chip_sr2(rdev->chip_ctx) && cctx->modes.db_push) {
		if (BNXT_VF(bp) || BNXT_NPAR(bp))
			cctx->modes.db_push = false;
	}
}

static void bnxt_re_destroy_chip_ctx(struct bnxt_re_dev *rdev)
{
	struct bnxt_qplib_chip_ctx *chip_ctx;
	struct bnxt_qplib_res *res;

	if (!rdev->chip_ctx)
		return;

	res = &rdev->qplib_res;
	bnxt_qplib_unmap_db_bar(res);

	kfree(res->hctx);
	res->rcfw = NULL;
	kfree(rdev->dev_attr);
	rdev->dev_attr = NULL;

	chip_ctx = rdev->chip_ctx;
	rdev->chip_ctx = NULL;
	res->cctx = NULL;
	res->hctx = NULL;
	res->pdev = NULL;
	res->netdev = NULL;
	kfree(chip_ctx);
}

static int bnxt_re_setup_chip_ctx(struct bnxt_re_dev *rdev, u8 wqe_mode)
{
	struct bnxt_qplib_chip_ctx *chip_ctx;
	struct bnxt_en_dev *en_dev;
	struct bnxt *bp;
	int rc;

	en_dev = rdev->en_dev;
	bp = netdev_priv(en_dev->net);
	/* Supply pci device to qplib */
	rdev->qplib_res.pdev = en_dev->pdev;
	rdev->qplib_res.netdev = rdev->netdev;

	chip_ctx = kzalloc(sizeof(*chip_ctx), GFP_KERNEL);
	if (!chip_ctx)
		return -ENOMEM;
	rdev->chip_ctx = chip_ctx;
	rdev->qplib_res.cctx = chip_ctx;
	rc = bnxt_re_query_hwrm_intf_version(rdev);
	if (rc)
		return rc;

	rdev->dev_attr = kzalloc(sizeof(*rdev->dev_attr), GFP_KERNEL);
	if (!rdev->dev_attr)
		return -ENOMEM;
	rdev->qplib_res.dattr = rdev->dev_attr;
	rdev->qplib_res.rcfw = &rdev->rcfw;
	rdev->qplib_res.is_vf = BNXT_VF(bp);

	rdev->qplib_res.hctx = kzalloc(sizeof(*rdev->qplib_res.hctx),
				       GFP_KERNEL);
	if (!rdev->qplib_res.hctx)
		return -ENOMEM;

	bnxt_re_set_drv_mode(rdev, wqe_mode);

	bnxt_re_set_db_offset(rdev);
	rc = bnxt_qplib_map_db_bar(&rdev->qplib_res);
	if (rc)
		return rc;

	if (!BNXT_VF(bp)) {
		if (bnxt_qplib_enable_atomic_ops_to_root(en_dev->pdev))
			dev_info(rdev_to_dev(rdev),
				 "pci-atomics not supported. Disabled");
	}

	return 0;
}

static int bnxt_re_free_msix(struct bnxt_re_dev *rdev)
{
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	int rc;

	rc = en_dev->en_ops->bnxt_free_msix(rdev->en_dev, BNXT_ROCE_ULP);
	if (rc)
		dev_err(rdev_to_dev(rdev), "netdev %p free_msix failed!",
			rdev->netdev);

	return 0;
}

static int bnxt_re_request_msix(struct bnxt_re_dev *rdev)
{
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	struct bnxt *bp = netdev_priv(en_dev->net);
	int rc = 0, num_msix_want, num_msix_got;
	struct bnxt_msix_entry *entry;

	/*
	 * Request MSIx based on the function type. This is
	 * a temporory solution to enable max VFs when NPAR is
	 * enabled.
	 * TODO - change the scheme with an adapter specific check
	 * as the latest adapters can support more NQs. For now
	 * this change satisfy all adapter versions.
	 */

	if (rdev->is_virtfn)
		num_msix_want = BNXT_RE_MAX_MSIX_VF;
	else if (BNXT_NPAR(bp))
		num_msix_want = BNXT_RE_MAX_MSIX_NPAR_PF;
	else
		num_msix_want = BNXT_RE_MAX_MSIX_PF;

	/*
	 * Since MSIX vectors are used for both NQs and CREQ, we should try to
	 * allocate num_online_cpus + 1 by taking into account the CREQ. This
	 * leaves the number of MSIX vectors for NQs match the number of CPUs
	 * and allows the system to be fully utilized
	 */
	num_msix_want = min_t(u32, num_msix_want, num_online_cpus() + 1);

	entry = rdev->nqr.msix_entries;
	num_msix_got = en_dev->en_ops->bnxt_request_msix(en_dev, BNXT_ROCE_ULP,
							 entry, num_msix_want);
	if (num_msix_got < BNXT_RE_MIN_MSIX) {
		rc = -EINVAL;
		goto done;
	}
	if (num_msix_got != num_msix_want)
		dev_warn(rdev_to_dev(rdev),
			 "bnxt_request_msix: wanted %d vectors, got %d\n",
			 num_msix_want, num_msix_got);

	rdev->nqr.num_msix = num_msix_got;
	return 0;
done:
	if (num_msix_got)
		en_dev->en_ops->bnxt_free_msix(rdev->en_dev, BNXT_ROCE_ULP);
	return rc;
}

static u16 bnxt_re_get_rtype(struct bnxt_re_dev *rdev) {
	return _is_chip_gen_p5(rdev->chip_ctx) ?
	       RING_ALLOC_REQ_RING_TYPE_NQ :
	       RING_ALLOC_REQ_RING_TYPE_ROCE_CMPL;
}

static int bnxt_re_net_ring_free(struct bnxt_re_dev *rdev, u16 fw_ring_id)
{
	int rc = -EINVAL;
	struct hwrm_ring_free_input req = {0};
	struct hwrm_ring_free_output resp;
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	struct bnxt_fw_msg fw_msg;

	if (!en_dev)
		return rc;

	/* To avoid unnecessary error messages duringrecovery.
         * HW is anyway in error state. So dont send down the command */
	if (test_bit(BNXT_RE_FLAG_ERR_DEVICE_DETACHED, &rdev->flags))
		return 0;

	/* allocation had failed, no need to issue hwrm */
	if (fw_ring_id == 0xffff)
		return 0;

	memset(&fw_msg, 0, sizeof(fw_msg));

	bnxt_re_init_hwrm_hdr(rdev, (void *)&req, HWRM_RING_FREE, -1, -1);
	req.ring_type = bnxt_re_get_rtype(rdev);
	req.ring_id = cpu_to_le16(fw_ring_id);
	bnxt_re_fill_fw_msg(&fw_msg, (void *)&req, sizeof(req), (void *)&resp,
			    sizeof(resp), DFLT_HWRM_CMD_TIMEOUT);
	rc = en_dev->en_ops->bnxt_send_fw_msg(en_dev, BNXT_ROCE_ULP, &fw_msg);
	if (rc) {
		dev_err(rdev_to_dev(rdev),
			"Failed to free HW ring with rc = 0x%x", rc);
		goto done;
	}
	dev_dbg(rdev_to_dev(rdev), "HW ring freed with id = 0x%x\n",
		fw_ring_id);
done:
	return rc;
}

static int bnxt_re_net_ring_alloc(struct bnxt_re_dev *rdev,
				  struct bnxt_re_ring_attr *ring_attr,
				  u16 *fw_ring_id)
{
	int rc = -EINVAL;
	struct hwrm_ring_alloc_input req = {0};
	struct hwrm_ring_alloc_output resp;
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	struct bnxt_fw_msg fw_msg;

	if (!en_dev)
		return rc;

	memset(&fw_msg, 0, sizeof(fw_msg));
	bnxt_re_init_hwrm_hdr(rdev, (void *)&req, HWRM_RING_ALLOC, -1, -1);
	req.enables = 0;
	req.page_tbl_addr =  cpu_to_le64(ring_attr->dma_arr[0]);
	if (ring_attr->pages > 1) {
		/* Page size is in log2 units */
		req.page_size = BNXT_PAGE_SHIFT;
		req.page_tbl_depth = 1;
	} else {
		req.page_size = 4; /* FIXME: hardconding */
		req.page_tbl_depth = 0;
	}

	req.fbo = 0;
	/* Association of ring index with doorbell index and MSIX number */
	req.logical_id = cpu_to_le16(ring_attr->lrid);
	req.length = cpu_to_le32(ring_attr->depth + 1);
	req.ring_type = ring_attr->type;
	req.int_mode = ring_attr->mode;
	bnxt_re_fill_fw_msg(&fw_msg, (void *)&req, sizeof(req), (void *)&resp,
			    sizeof(resp), DFLT_HWRM_CMD_TIMEOUT);
	rc = en_dev->en_ops->bnxt_send_fw_msg(en_dev, BNXT_ROCE_ULP, &fw_msg);
	if (rc) {
		dev_err(rdev_to_dev(rdev),
			"Failed to allocate HW ring with rc = 0x%x", rc);
		goto done;
	}
	*fw_ring_id = le16_to_cpu(resp.ring_id);
	dev_dbg(rdev_to_dev(rdev),
		"HW ring allocated with id = 0x%x at slot 0x%x",
		resp.ring_id, ring_attr->lrid);
done:
	return rc;
}

static int bnxt_re_net_stats_ctx_free(struct bnxt_re_dev *rdev,
				      u32 fw_stats_ctx_id, u16 tid)
{
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	struct hwrm_stat_ctx_free_input req = {0};
	struct hwrm_stat_ctx_free_output resp;
	struct bnxt_fw_msg fw_msg;
	int rc = -EINVAL;

	if (!en_dev)
		return rc;

	/* To avoid unnecessary error messages duringrecovery.
         * HW is anyway in error state. So dont send down the command */
	if (test_bit(BNXT_RE_FLAG_ERR_DEVICE_DETACHED, &rdev->flags))
		return 0;
	memset(&fw_msg, 0, sizeof(fw_msg));
	bnxt_re_init_hwrm_hdr(rdev, (void *)&req, HWRM_STAT_CTX_FREE, -1, tid);
	req.stat_ctx_id = cpu_to_le32(fw_stats_ctx_id);
	bnxt_re_fill_fw_msg(&fw_msg, (void *)&req, sizeof(req), (void *)&resp,
			    sizeof(resp), DFLT_HWRM_CMD_TIMEOUT);
	rc = en_dev->en_ops->bnxt_send_fw_msg(en_dev, BNXT_ROCE_ULP, &fw_msg);
	if (rc) {
		dev_err(rdev_to_dev(rdev),
			"Failed to free HW stats ctx with rc = 0x%x", rc);
		goto done;
	}
	dev_dbg(rdev_to_dev(rdev),
		"HW stats ctx freed with id = 0x%x", fw_stats_ctx_id);
done:
	return rc;
}

static int bnxt_re_net_stats_ctx_alloc(struct bnxt_re_dev *rdev, u16 tid)
{
	struct hwrm_stat_ctx_alloc_output resp = {};
	struct hwrm_stat_ctx_alloc_input req = {};
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	struct bnxt_qplib_stats *stat;
	struct bnxt_qplib_ctx *hctx;
	struct bnxt_fw_msg fw_msg;
	int rc = 0;

	hctx = rdev->qplib_res.hctx;
	stat = (tid == 0xffff) ? &hctx->stats : &hctx->stats2;
	stat->fw_id = INVALID_STATS_CTX_ID;

	if (!en_dev)
		goto done;

	memset(&fw_msg, 0, sizeof(fw_msg));
	bnxt_re_init_hwrm_hdr(rdev, (void *)&req,
			      HWRM_STAT_CTX_ALLOC, -1, tid);
	req.update_period_ms = 0;
	req.stats_dma_addr = cpu_to_le64(stat->dma_map);
	req.stat_ctx_flags = STAT_CTX_ALLOC_REQ_STAT_CTX_FLAGS_ROCE;
	bnxt_re_fill_fw_msg(&fw_msg, (void *)&req, sizeof(req), (void *)&resp,
			    sizeof(resp), DFLT_HWRM_CMD_TIMEOUT);
	rc = en_dev->en_ops->bnxt_send_fw_msg(en_dev, BNXT_ROCE_ULP, &fw_msg);
	if (rc) {
		dev_err(rdev_to_dev(rdev),
			"Failed to allocate HW stats ctx, rc = 0x%x", rc);
		goto done;
	}
	stat->fw_id = le32_to_cpu(resp.stat_ctx_id);
	dev_dbg(rdev_to_dev(rdev), "HW stats ctx allocated with id = 0x%x",
		stat->fw_id);
done:
	return rc;
}

static int bnxt_re_query_hwrm_intf_version(struct bnxt_re_dev *rdev)
{
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	struct hwrm_ver_get_output resp = {0};
	struct hwrm_ver_get_input req = {0};
	struct bnxt_qplib_chip_ctx *cctx;
	struct bnxt_fw_msg fw_msg;
	int rc = 0;

	memset(&fw_msg, 0, sizeof(fw_msg));
	bnxt_re_init_hwrm_hdr(rdev, (void *)&req,
			      HWRM_VER_GET, -1, -1);
	req.hwrm_intf_maj = HWRM_VERSION_MAJOR;
	req.hwrm_intf_min = HWRM_VERSION_MINOR;
	req.hwrm_intf_upd = HWRM_VERSION_UPDATE;
	bnxt_re_fill_fw_msg(&fw_msg, (void *)&req, sizeof(req), (void *)&resp,
			    sizeof(resp), DFLT_HWRM_CMD_TIMEOUT);
	rc = en_dev->en_ops->bnxt_send_fw_msg(en_dev, BNXT_ROCE_ULP, &fw_msg);
	if (rc) {
		dev_err(rdev_to_dev(rdev),
			"Failed to query HW version, rc = 0x%x", rc);
		return rc;
	}
	cctx = rdev->chip_ctx;
	cctx->hwrm_intf_ver = (u64) le16_to_cpu(resp.hwrm_intf_major) << 48 |
			      (u64) le16_to_cpu(resp.hwrm_intf_minor) << 32 |
			      (u64) le16_to_cpu(resp.hwrm_intf_build) << 16 |
				    le16_to_cpu(resp.hwrm_intf_patch);

	cctx->chip_num = le16_to_cpu(resp.chip_num);
	cctx->chip_rev = resp.chip_rev;
	cctx->chip_metal = resp.chip_metal;
	return 0;
}

/* Query device config using common hwrm */
static int bnxt_re_hwrm_qcfg(struct bnxt_re_dev *rdev, u32 *db_len,
			     u32 *offset)
{
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	struct hwrm_func_qcfg_output resp = {0};
	struct hwrm_func_qcfg_input req = {0};
	struct bnxt_fw_msg fw_msg;
	int rc;

	memset(&fw_msg, 0, sizeof(fw_msg));
	bnxt_re_init_hwrm_hdr(rdev, (void *)&req,
			      HWRM_FUNC_QCFG, -1, -1);
	req.fid = cpu_to_le16(0xffff);
	bnxt_re_fill_fw_msg(&fw_msg, (void *)&req, sizeof(req), (void *)&resp,
			    sizeof(resp), DFLT_HWRM_CMD_TIMEOUT);
	rc = en_dev->en_ops->bnxt_send_fw_msg(en_dev, BNXT_ROCE_ULP, &fw_msg);
	if (rc) {
		dev_err(rdev_to_dev(rdev),
			"Failed to query config, rc = %#x", rc);
		return rc;
	}

	*db_len = PAGE_ALIGN(le16_to_cpu(resp.l2_doorbell_bar_size_kb) * 1024);
	*offset = PAGE_ALIGN(le16_to_cpu(resp.legacy_l2_db_size_kb) * 1024);
	return 0;
}

/* Query function capabilities using common hwrm */
static int bnxt_re_hwrm_qcaps(struct bnxt_re_dev *rdev, u8 *enabled)
{
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	struct hwrm_func_qcaps_output resp = {0};
	struct hwrm_func_qcaps_input req = {0};
	struct bnxt_fw_msg fw_msg;
	int rc;

	memset(&fw_msg, 0, sizeof(fw_msg));
	bnxt_re_init_hwrm_hdr(rdev, (void *)&req,
			      HWRM_FUNC_QCAPS, -1, -1);
	req.fid = cpu_to_le16(0xffff);
	bnxt_re_fill_fw_msg(&fw_msg, (void *)&req, sizeof(req), (void *)&resp,
			    sizeof(resp), DFLT_HWRM_CMD_TIMEOUT);
	rc = en_dev->en_ops->bnxt_send_fw_msg(en_dev, BNXT_ROCE_ULP, &fw_msg);
	if (rc) {
		dev_err(rdev_to_dev(rdev),
			"Failed to query capabilities, rc = %#x", rc);
		return rc;
	}
	if (_is_chip_sr2(rdev->chip_ctx))
		*enabled = (resp.flags_ext &
			    FUNC_QCAPS_RESP_FLAGS_EXT_PPP_PUSH_MODE_SUPPORTED) ?
			    true : false;
	else
		*enabled = (resp.flags & FUNC_QCAPS_RESP_FLAGS_WCB_PUSH_MODE) ?
			    true : false;
	return 0;
}

/* Net -> RoCE driver */

/* Device */
static bool bnxt_re_parse_l2_ver(struct ethtool_drvinfo *drvinfo)
{
	unsigned int len,  major, minor;
	char *verstr, *src, *ptr;
	unsigned int version = 0;

	len = strlen(drvinfo->version);
	verstr = kmalloc(len + 1, GFP_KERNEL);
	if (!verstr)
		return false;
	strlcpy(verstr, drvinfo->version, len + 1);

	src = verstr;
	ptr = strsep(&src, ".");
	if (kstrtouint(ptr, 15, &version))
		goto free;
	ptr = strsep(&src, ".");
	if (kstrtouint(ptr, 15, &major))
		goto free;
	if (kstrtouint(src, 15, &minor))
		dev_dbg(NULL, "%s: bnxt_en ver %d.%d.XX detected",
			ROCE_DRV_MODULE_NAME, version, major);
	else
		dev_dbg(NULL, "%s: bnxt_en ver %d.%d.%d detected",
			ROCE_DRV_MODULE_NAME, version, major, minor);
free:
	kfree(verstr);
	return version <= 0 ? false : true;
}

static bool bnxt_re_check_l2_compatibility(struct net_device *netdev)
{
	struct ethtool_drvinfo drvinfo;

	if (!bnxt_re_get_drvinfo(netdev, &drvinfo))
		return bnxt_re_parse_l2_ver(&drvinfo);
	return false;
}

struct bnxt_re_dev *bnxt_re_from_netdev(struct net_device *netdev)
{
	struct bnxt_re_dev *rdev;

	rcu_read_lock();
	list_for_each_entry_rcu(rdev, &bnxt_re_dev_list, list) {
		if (rdev->netdev == netdev) {
			rcu_read_unlock();
			dev_dbg(rdev_to_dev(rdev),
				"netdev (%p) found, ref_count = 0x%x",
				netdev, atomic_read(&rdev->ref_count));
			return rdev;
		}
		if (rdev->binfo) {
			if (rdev->binfo->slave1 == netdev ||
			    rdev->binfo->slave2 == netdev) {
				rcu_read_unlock();
				dev_dbg(rdev_to_dev(rdev),
					"Bond netdev (%p) found, ref_count = 0x%x",
					netdev, atomic_read(&rdev->ref_count));
				return rdev;
			}
		}
	}
	rcu_read_unlock();
	return NULL;
}

static struct bnxt_re_bond_info *bnxt_re_get_bond_info
			(struct net_device *netdev)
{
	struct bnxt_re_bond_info *binfo = NULL;
	struct bnxt_re_bond_info *tmp;

	mutex_lock(&bnxt_re_dev_lock);
	list_for_each_entry_rcu(tmp, &bnxt_re_bond_list, list) {
		if (tmp->master == netdev) {
			dev_dbg(NULL,
				"netdev (%p) found to be master", netdev);
			binfo = tmp;
			break;
		}
		if (tmp->slave1 == netdev || tmp->slave2 == netdev) {
			dev_dbg(NULL,
				"netdev (%p) found to be slave master = %p slave1 = %p slave2 = %p",
				netdev, tmp->master, tmp->slave1, tmp->slave2);
			binfo = tmp;
			break;
		}
	}
	mutex_unlock(&bnxt_re_dev_lock);
	return binfo;
}

static void bnxt_re_dev_unprobe(struct bnxt_en_dev *en_dev)
{
	if (en_dev->net)
		dev_put(en_dev->net);
}

static struct bnxt_en_dev *bnxt_re_dev_probe(struct net_device *netdev)
{
	struct bnxt *bp = netdev_priv(netdev);
	struct bnxt_en_dev *en_dev;
	struct pci_dev *pdev;

	/* Call bnxt_en's RoCE probe via indirect API */
	if (!bp->ulp_probe) {
		dev_err(NULL, "%s: probe error: bp->bnxt_ulp_probe is NULL!",
			ROCE_DRV_MODULE_NAME);
		return ERR_PTR(-EINVAL);
	}
	if (bp->ulp_version != BNXT_ULP_VERSION) {
		dev_err(NULL, "%s: probe error: bnxt_en ulp version magic %x is not compatible!\n",
			ROCE_DRV_MODULE_NAME, bp->ulp_version);
		return ERR_PTR(-EINVAL);
	}
	en_dev = bp->ulp_probe(netdev);
	if (IS_ERR(en_dev)) {
		dev_err(NULL, "%s: (0x%p) probe error!\n ",
			ROCE_DRV_MODULE_NAME, bp);
		return en_dev;
	}

	pdev = en_dev->pdev;
	if (!pdev) {
		dev_err(NULL, "%s: probe error: PCI device is NULL!",
			ROCE_DRV_MODULE_NAME);
		return ERR_PTR(-EINVAL);
	}

	if (!(en_dev->flags & BNXT_EN_FLAG_ROCE_CAP)) {
		dev_dbg(&pdev->dev,
			"%s: probe error: RoCE is not supported on this device",
			ROCE_DRV_MODULE_NAME);
		return ERR_PTR(-ENODEV);
	}
	dev_dbg(&pdev->dev, "%s: RoCE is supported on this device",
		 ROCE_DRV_MODULE_NAME);
	/* Bump net device reference count */
	dev_hold(netdev);

	return en_dev;
}

#ifdef HAVE_IB_SET_DEV_OPS
static const struct ib_device_ops bnxt_re_dev_ops = {
#ifdef HAVE_IB_OWNER_IN_DEVICE_OPS
	.owner			= THIS_MODULE,
	.driver_id 		= RDMA_DRIVER_BNXT_RE,
	.uverbs_abi_ver 	= BNXT_RE_ABI_VERSION,
#endif
	.query_device		= bnxt_re_query_device,
	.modify_device		= bnxt_re_modify_device,

	.query_port		= bnxt_re_query_port,
	.modify_port		= bnxt_re_modify_port,
#ifdef HAVE_IB_GET_PORT_IMMUTABLE
	.get_port_immutable	= bnxt_re_get_port_immutable,
#endif
#ifdef HAVE_IB_GET_DEV_FW_STR
	.get_dev_fw_str		= bnxt_re_query_fw_str,
#endif
	.query_pkey		= bnxt_re_query_pkey,
	.query_gid		= bnxt_re_query_gid,
#ifdef HAVE_IB_GET_NETDEV
	.get_netdev		= bnxt_re_get_netdev,
#endif
#ifdef HAVE_IB_ADD_DEL_GID
	.add_gid		= bnxt_re_add_gid,
	.del_gid		= bnxt_re_del_gid,
#endif
#ifdef HAVE_IB_MODIFY_GID
	.modify_gid		= bnxt_re_modify_gid,
#endif
	.get_link_layer		= bnxt_re_get_link_layer,

	.alloc_pd		= bnxt_re_alloc_pd,
	.dealloc_pd		= bnxt_re_dealloc_pd,

	.create_ah		= bnxt_re_create_ah,
#ifdef HAVE_IB_CREATE_USER_AH
	.create_user_ah		= bnxt_re_create_ah,
#endif
	.modify_ah		= bnxt_re_modify_ah,
	.query_ah		= bnxt_re_query_ah,
	.destroy_ah		= bnxt_re_destroy_ah,

	.create_srq		= bnxt_re_create_srq,
	.modify_srq		= bnxt_re_modify_srq,
	.query_srq		= bnxt_re_query_srq,
	.destroy_srq		= bnxt_re_destroy_srq,
	.post_srq_recv		= bnxt_re_post_srq_recv,

	.create_qp		= bnxt_re_create_qp,
	.modify_qp		= bnxt_re_modify_qp,
	.query_qp		= bnxt_re_query_qp,
	.destroy_qp		= bnxt_re_destroy_qp,

	.post_send		= bnxt_re_post_send,
	.post_recv		= bnxt_re_post_recv,

	.create_cq		= bnxt_re_create_cq,
	.modify_cq		= bnxt_re_modify_cq,	/* Need ? */
	.destroy_cq		= bnxt_re_destroy_cq,
	.resize_cq		= bnxt_re_resize_cq,
	.poll_cq		= bnxt_re_poll_cq,
	.req_notify_cq		= bnxt_re_req_notify_cq,

	.get_dma_mr		= bnxt_re_get_dma_mr,
#ifdef HAVE_IB_REG_PHYS_MR
	.reg_phys_mr		= bnxt_re_reg_phys_mr,
	.rereg_phys_mr		= bnxt_re_rereg_phys_mr,
#endif
#ifdef HAVE_IB_QUERY_MR
	.query_mr		= bnxt_re_query_mr,
#endif
	.dereg_mr		= bnxt_re_dereg_mr,
#ifdef HAVE_IB_SIGNATURE_HANDOVER
	.destroy_mr		= bnxt_re_destroy_mr,
	.create_mr		= bnxt_re_create_mr,
#endif
#ifdef HAVE_IB_FAST_REG_MR
	.alloc_fast_reg_mr	= bnxt_re_alloc_fast_reg_mr,
	.alloc_fast_reg_page_list	= bnxt_re_alloc_fast_reg_page_list,
	.free_fast_reg_page_list	= bnxt_re_free_fast_reg_page_list,
#endif
#ifdef HAVE_IB_ALLOC_MR
	.alloc_mr		= bnxt_re_alloc_mr,
#endif
#ifdef HAVE_IB_MAP_MR_SG
	.map_mr_sg		= bnxt_re_map_mr_sg,
#endif
	.alloc_mw		= bnxt_re_alloc_mw,
#ifdef HAVE_IB_BIND_MW
	.bind_mw		= bnxt_re_bind_mw,
#endif
	.dealloc_mw		= bnxt_re_dealloc_mw,
#ifdef USE_IB_FMR
	.alloc_fmr		= bnxt_re_alloc_fmr,
	.map_phys_fmr		= bnxt_re_map_phys_fmr,
	.unmap_fmr		= bnxt_re_unmap_fmr,
	.dealloc_fmr		= bnxt_re_dealloc_fmr,
#endif
	.reg_user_mr		= bnxt_re_reg_user_mr,
#ifdef HAVE_IB_REREG_USER_MR
	.rereg_user_mr		= bnxt_re_rereg_user_mr,
#endif
#ifdef HAVE_DISASSOCIATE_UCNTX
	/*
	 * Have to be populated for both old and new kernels
	 * to allow rmmoding driver with app running
	 */
	.disassociate_ucontext	= bnxt_re_disassociate_ucntx,
#endif
	.alloc_ucontext		= bnxt_re_alloc_ucontext,
	.dealloc_ucontext	= bnxt_re_dealloc_ucontext,
	.mmap			= bnxt_re_mmap,
#ifdef USE_PROCESS_MAD
	.process_mad		= bnxt_re_process_mad,
#endif
#ifdef HAVE_AH_ALLOC_IN_IB_CORE
	INIT_RDMA_OBJ_SIZE(ib_ah, bnxt_re_ah, ib_ah),
#endif
#ifdef HAVE_PD_ALLOC_IN_IB_CORE
        INIT_RDMA_OBJ_SIZE(ib_pd, bnxt_re_pd, ib_pd),
#endif
#ifdef HAVE_SRQ_CREATE_IN_IB_CORE
        INIT_RDMA_OBJ_SIZE(ib_srq, bnxt_re_srq, ib_srq),
#endif
#ifdef HAVE_CQ_ALLOC_IN_IB_CORE
	INIT_RDMA_OBJ_SIZE(ib_cq, bnxt_re_cq, ib_cq),
#endif
#ifdef HAVE_UCONTEXT_ALLOC_IN_IB_CORE
        INIT_RDMA_OBJ_SIZE(ib_ucontext, bnxt_re_ucontext, ib_uctx),
#endif
};
#endif /* HAVE_IB_SET_DEV_OPS */
#ifdef HAVE_RDMA_SET_DEVICE_SYSFS_GROUP
static ssize_t hw_rev_show(struct device *device, struct device_attribute *attr,
                           char *buf)
{
        struct bnxt_re_dev *rdev = to_bnxt_re_dev(device, ibdev.dev);

#ifdef HAS_SYSFS_EMIT
	return sysfs_emit(buf, "0x%x\n", rdev->en_dev->pdev->vendor);
#else
        return scnprintf(buf, PAGE_SIZE, "0x%x\n", rdev->en_dev->pdev->vendor);
#endif
}
static DEVICE_ATTR_RO(hw_rev);

static ssize_t hca_type_show(struct device *device,
                             struct device_attribute *attr, char *buf)
{
        struct bnxt_re_dev *rdev = to_bnxt_re_dev(device, ibdev.dev);

#ifdef HAS_SYSFS_EMIT
	return sysfs_emit(buf, "%s\n", rdev->ibdev.node_desc);
#else
        return scnprintf(buf, PAGE_SIZE, "%s\n", rdev->ibdev.node_desc);
#endif
}
static DEVICE_ATTR_RO(hca_type);


static struct attribute *bnxt_re_attributes[] = {
        &dev_attr_hw_rev.attr,
        &dev_attr_hca_type.attr,
        NULL
};
static const struct attribute_group bnxt_re_dev_attr_group = {
        .attrs = bnxt_re_attributes,
};
#else
static ssize_t show_rev(struct device *device, struct device_attribute *attr,
			char *buf)
{
	struct bnxt_re_dev *rdev = to_bnxt_re_dev(device, ibdev.dev);

	return scnprintf(buf, PAGE_SIZE, "0x%x\n", rdev->en_dev->pdev->vendor);
	strcpy(buf, "bnxt_re");
	return 0;
}


static ssize_t show_hca(struct device *device, struct device_attribute *attr,
			char *buf)
{
	struct bnxt_re_dev *rdev = to_bnxt_re_dev(device, ibdev.dev);

	return scnprintf(buf, PAGE_SIZE, "%s\n", rdev->ibdev.node_desc);
}

#ifndef HAVE_IB_GET_DEV_FW_STR
static ssize_t show_fw_ver(struct device *device, struct device_attribute *attr,
			   char *buf)
{
	struct bnxt_re_dev *rdev = to_bnxt_re_dev(device, ibdev.dev);

	return scnprintf(buf, PAGE_SIZE, "%d.%d.%d.%d\n",
			 rdev->dev_attr->fw_ver[0], rdev->dev_attr->fw_ver[1],
			 rdev->dev_attr->fw_ver[2], rdev->dev_attr->fw_ver[3]);
}

static DEVICE_ATTR(fw_rev, 0444, show_fw_ver, NULL);
#endif
static DEVICE_ATTR(hw_rev, 0444, show_rev, NULL);
static DEVICE_ATTR(hca_type, 0444, show_hca, NULL);

static struct device_attribute *bnxt_re_attributes[] = {
	&dev_attr_hw_rev,
#ifndef HAVE_IB_GET_DEV_FW_STR
	&dev_attr_fw_rev,
#endif
	&dev_attr_hca_type
};
#endif

static int bnxt_re_register_ib(struct bnxt_re_dev *rdev)
{
	struct ib_device *ibdev = &rdev->ibdev;
	int ret = 0;

	/* ib device init */
#ifndef HAVE_IB_OWNER_IN_DEVICE_OPS
	ibdev->owner = THIS_MODULE;
	ibdev->uverbs_abi_ver = BNXT_RE_ABI_VERSION;
#ifdef HAVE_RDMA_DRIVER_ID
	ibdev->driver_id = RDMA_DRIVER_BNXT_RE;
#endif
#endif
	ibdev->node_type = RDMA_NODE_IB_CA;
	strlcpy(ibdev->node_desc, BNXT_RE_DESC " HCA",
		strlen(BNXT_RE_DESC) + 5);
	ibdev->phys_port_cnt = 1;

	bnxt_qplib_get_guid(rdev->netdev->dev_addr, (u8 *)&ibdev->node_guid);

	/* Data path irqs is one less than the max msix vectors */
	ibdev->num_comp_vectors	= rdev->nqr.num_msix - 1;
	bnxt_re_set_dma_device(ibdev, rdev);
	ibdev->local_dma_lkey = BNXT_QPLIB_RSVD_LKEY;

#ifdef HAVE_IB_UVERBS_CMD_MASK_IN_DRIVER
	/* User space */
	ibdev->uverbs_cmd_mask =
			(1ull << IB_USER_VERBS_CMD_GET_CONTEXT)		|
			(1ull << IB_USER_VERBS_CMD_QUERY_DEVICE)	|
			(1ull << IB_USER_VERBS_CMD_QUERY_PORT)		|
			(1ull << IB_USER_VERBS_CMD_ALLOC_PD)		|
			(1ull << IB_USER_VERBS_CMD_DEALLOC_PD)		|
			(1ull << IB_USER_VERBS_CMD_REG_MR)		|
			(1ull << IB_USER_VERBS_CMD_DEREG_MR)		|
			(1ull << IB_USER_VERBS_CMD_CREATE_COMP_CHANNEL) |
			(1ull << IB_USER_VERBS_CMD_CREATE_CQ)		|
			(1ull << IB_USER_VERBS_CMD_DESTROY_CQ)		|
			(1ull << IB_USER_VERBS_CMD_CREATE_QP)		|
			(1ull << IB_USER_VERBS_CMD_MODIFY_QP)		|
			(1ull << IB_USER_VERBS_CMD_QUERY_QP)		|
			(1ull << IB_USER_VERBS_CMD_DESTROY_QP)		|
			(1ull << IB_USER_VERBS_CMD_REREG_MR)		|
			(1ull << IB_USER_VERBS_CMD_RESIZE_CQ)		|
			(1ull << IB_USER_VERBS_CMD_CREATE_SRQ)		|
			(1ull << IB_USER_VERBS_CMD_MODIFY_SRQ)		|
			(1ull << IB_USER_VERBS_CMD_QUERY_SRQ)		|
			(1ull << IB_USER_VERBS_CMD_DESTROY_SRQ)		|
			(1ull << IB_USER_VERBS_CMD_POLL_CQ)		|
			(1ull << IB_USER_VERBS_CMD_ALLOC_MW)		|
			(1ull << IB_USER_VERBS_CMD_DEALLOC_MW)		|
			(1ull << IB_USER_VERBS_CMD_CREATE_AH)		|
			(1ull << IB_USER_VERBS_CMD_MODIFY_AH)		|
			(1ull << IB_USER_VERBS_CMD_QUERY_AH)		|
			(1ull << IB_USER_VERBS_CMD_DESTROY_AH);

#ifdef HAVE_IB_USER_VERBS_EX_CMD_MODIFY_QP
	ibdev->uverbs_ex_cmd_mask = (1ull << IB_USER_VERBS_EX_CMD_MODIFY_QP);
#endif
#endif

	/* REQ_NOTIFY_CQ is directly handled in libbnxt_re.
	 * POLL_CQ is processed only as part of a RESIZE_CQ operation;
	 * the library uses this to let the kernel driver know that
	 * RESIZE_CQ is complete and memory from the previous CQ can be
	 * unmapped.
	 */
#ifdef HAVE_RDMA_SET_DEVICE_SYSFS_GROUP
	rdma_set_device_sysfs_group(ibdev, &bnxt_re_dev_attr_group);
#endif
#ifdef HAVE_IB_DEVICE_SET_NETDEV
	ret = ib_device_set_netdev(&rdev->ibdev, rdev->netdev, 1);
        if (ret)
                return ret;
#endif

#ifdef HAVE_IB_SET_DEV_OPS
	ib_set_device_ops(ibdev, &bnxt_re_dev_ops);
#else
	/* Kernel verbs */
	ibdev->query_device		= bnxt_re_query_device;
	ibdev->modify_device		= bnxt_re_modify_device;

	ibdev->query_port		= bnxt_re_query_port;
	ibdev->modify_port		= bnxt_re_modify_port;
#ifdef HAVE_IB_GET_PORT_IMMUTABLE
	ibdev->get_port_immutable	= bnxt_re_get_port_immutable;
#endif
#ifdef HAVE_IB_GET_DEV_FW_STR
	ibdev->get_dev_fw_str		= bnxt_re_query_fw_str;
#endif
	ibdev->query_pkey		= bnxt_re_query_pkey;
	ibdev->query_gid		= bnxt_re_query_gid;
#ifdef HAVE_IB_GET_NETDEV
	ibdev->get_netdev		= bnxt_re_get_netdev;
#endif
#ifdef HAVE_IB_ADD_DEL_GID
	ibdev->add_gid			= bnxt_re_add_gid;
	ibdev->del_gid			= bnxt_re_del_gid;
#endif
#ifdef HAVE_IB_MODIFY_GID
	ibdev->modify_gid		= bnxt_re_modify_gid;
#endif
	ibdev->get_link_layer		= bnxt_re_get_link_layer;

	ibdev->alloc_pd			= bnxt_re_alloc_pd;
	ibdev->dealloc_pd		= bnxt_re_dealloc_pd;

	ibdev->create_ah		= bnxt_re_create_ah;
	ibdev->modify_ah		= bnxt_re_modify_ah;
	ibdev->query_ah			= bnxt_re_query_ah;
	ibdev->destroy_ah		= bnxt_re_destroy_ah;

	ibdev->create_srq		= bnxt_re_create_srq;
	ibdev->modify_srq		= bnxt_re_modify_srq;
	ibdev->query_srq		= bnxt_re_query_srq;
	ibdev->destroy_srq		= bnxt_re_destroy_srq;
	ibdev->post_srq_recv		= bnxt_re_post_srq_recv;

	ibdev->create_qp		= bnxt_re_create_qp;
	ibdev->modify_qp		= bnxt_re_modify_qp;
	ibdev->query_qp			= bnxt_re_query_qp;
	ibdev->destroy_qp		= bnxt_re_destroy_qp;

	ibdev->post_send		= bnxt_re_post_send;
	ibdev->post_recv		= bnxt_re_post_recv;

	ibdev->create_cq		= bnxt_re_create_cq;
	ibdev->modify_cq		= bnxt_re_modify_cq;	/* Need ? */
	ibdev->destroy_cq		= bnxt_re_destroy_cq;
	ibdev->resize_cq		= bnxt_re_resize_cq;
	ibdev->poll_cq			= bnxt_re_poll_cq;
	ibdev->req_notify_cq		= bnxt_re_req_notify_cq;

	ibdev->get_dma_mr		= bnxt_re_get_dma_mr;
#ifdef HAVE_IB_REG_PHYS_MR
	ibdev->reg_phys_mr		= bnxt_re_reg_phys_mr;
	ibdev->rereg_phys_mr		= bnxt_re_rereg_phys_mr;
#endif
#ifdef HAVE_IB_QUERY_MR
	ibdev->query_mr			= bnxt_re_query_mr;
#endif
	ibdev->dereg_mr			= bnxt_re_dereg_mr;
#ifdef HAVE_IB_SIGNATURE_HANDOVER
	ibdev->destroy_mr		= bnxt_re_destroy_mr;
	ibdev->create_mr		= bnxt_re_create_mr;
#endif
#ifdef HAVE_IB_FAST_REG_MR
	ibdev->alloc_fast_reg_mr	= bnxt_re_alloc_fast_reg_mr;
	ibdev->alloc_fast_reg_page_list	= bnxt_re_alloc_fast_reg_page_list;
	ibdev->free_fast_reg_page_list	= bnxt_re_free_fast_reg_page_list;
#endif
#ifdef HAVE_IB_ALLOC_MR
	ibdev->alloc_mr			= bnxt_re_alloc_mr;
#endif
#ifdef HAVE_IB_MAP_MR_SG
	ibdev->map_mr_sg		= bnxt_re_map_mr_sg;
#endif
	ibdev->alloc_mw			= bnxt_re_alloc_mw;
#ifdef HAVE_IB_BIND_MW
	ibdev->bind_mw			= bnxt_re_bind_mw;
#endif
	ibdev->dealloc_mw		= bnxt_re_dealloc_mw;
#ifdef USE_IB_FMR
	ibdev->alloc_fmr		= bnxt_re_alloc_fmr;
	ibdev->map_phys_fmr		= bnxt_re_map_phys_fmr;
	ibdev->unmap_fmr		= bnxt_re_unmap_fmr;
	ibdev->dealloc_fmr		= bnxt_re_dealloc_fmr;
#endif

	ibdev->reg_user_mr		= bnxt_re_reg_user_mr;
#ifdef HAVE_IB_REREG_USER_MR
	ibdev->rereg_user_mr		= bnxt_re_rereg_user_mr;
#endif
#ifdef HAVE_DISASSOCIATE_UCNTX
#ifndef HAVE_RDMA_USER_MMAP_IO
	ibdev->disassociate_ucontext	= bnxt_re_disassociate_ucntx;
#endif
#endif
	ibdev->alloc_ucontext		= bnxt_re_alloc_ucontext;
	ibdev->dealloc_ucontext		= bnxt_re_dealloc_ucontext;
	ibdev->mmap			= bnxt_re_mmap;
	ibdev->process_mad		= bnxt_re_process_mad;
#endif /* HAVE_IB_SET_DEV_OPS */

#ifndef HAVE_IB_ALLOC_MR
	/* TODO: Workaround to uninitialized the kobj */
	ibdev->dev.kobj.state_initialized = 0;
#endif

	ret = ib_register_device_compat(rdev);
	return ret;
}


static void bnxt_re_dev_dealloc(struct bnxt_re_dev *rdev)
{
	int i = BNXT_RE_REF_WAIT_COUNT;

	dev_dbg(rdev_to_dev(rdev), "%s:Remove the device %p\n", __func__, rdev);
	/* Wait for rdev refcount to come down */
	while ((atomic_read(&rdev->ref_count) > 1) && i--)
		msleep(100);

	if (atomic_read(&rdev->ref_count) > 1)
		dev_err(rdev_to_dev(rdev),
			"Failed waiting for ref count to deplete %d",
			atomic_read(&rdev->ref_count));

	atomic_set(&rdev->ref_count, 0);
	dev_put(rdev->netdev);
	rdev->netdev = NULL;
	if (rdev->binfo) {
		mutex_lock(&bnxt_re_dev_lock);
		list_del_rcu(&rdev->binfo->list);
		kfree(rdev->binfo);
		mutex_unlock(&bnxt_re_dev_lock);
		rdev->binfo = NULL;
	}
	synchronize_rcu();

#ifdef RDMA_CORE_CAP_PROT_ROCE_UDP_ENCAP
	kfree(rdev->gid_map);
#endif
	ib_dealloc_device(&rdev->ibdev);
	/* rdev is gone */
}

static struct bnxt_re_dev *bnxt_re_dev_alloc(struct net_device *netdev,
					   struct bnxt_en_dev *en_dev)
{
	struct bnxt_re_dev *rdev;
#ifdef RDMA_CORE_CAP_PROT_ROCE_UDP_ENCAP
	u32 count;
#endif
	/* Allocate bnxt_re_dev instance here */
	rdev = (struct bnxt_re_dev *)compat_ib_alloc_device(sizeof(*rdev));
	if (!rdev) {
		dev_err(NULL, "%s: bnxt_re_dev allocation failure!",
			ROCE_DRV_MODULE_NAME);
		return NULL;
	}
	/* Default values */
	atomic_set(&rdev->ref_count, 0);
	rdev->netdev = netdev;
	dev_hold(rdev->netdev);
	rdev->en_dev = en_dev;
	rdev->id = rdev->en_dev->pdev->devfn;
	INIT_LIST_HEAD(&rdev->qp_list);
	mutex_init(&rdev->qp_lock);
	mutex_init(&rdev->cc_lock);
	bnxt_re_clear_rsors_stat(&rdev->stats.rsors);
	rdev->cosq[0] = rdev->cosq[1] = 0xFFFF;
	rdev->min_tx_depth = 1;
	rdev->stats.stats_query_sec = 1;
#ifdef RDMA_CORE_CAP_PROT_ROCE_UDP_ENCAP
	rdev->gid_map = kzalloc(sizeof(*(rdev->gid_map)) *
				  BNXT_RE_MAX_SGID_ENTRIES,
				  GFP_KERNEL);
	if (!rdev->gid_map) {
		ib_dealloc_device(&rdev->ibdev);
		return NULL;
	}
	for(count = 0; count < BNXT_RE_MAX_SGID_ENTRIES; count++)
		rdev->gid_map[count] = -1;
#endif /* RDMA_CORE_CAP_PROT_ROCE_UDP_ENCAP */

	return rdev;
}

static int bnxt_re_handle_unaffi_async_event(
		struct creq_func_event *unaffi_async)
{
	switch (unaffi_async->event) {
	case CREQ_FUNC_EVENT_EVENT_TX_WQE_ERROR:
		break;
	case CREQ_FUNC_EVENT_EVENT_TX_DATA_ERROR:
		break;
	case CREQ_FUNC_EVENT_EVENT_RX_WQE_ERROR:
		break;
	case CREQ_FUNC_EVENT_EVENT_RX_DATA_ERROR:
		break;
	case CREQ_FUNC_EVENT_EVENT_CQ_ERROR:
		break;
	case CREQ_FUNC_EVENT_EVENT_TQM_ERROR:
		break;
	case CREQ_FUNC_EVENT_EVENT_CFCQ_ERROR:
		break;
	case CREQ_FUNC_EVENT_EVENT_CFCS_ERROR:
		break;
	case CREQ_FUNC_EVENT_EVENT_CFCC_ERROR:
		break;
	case CREQ_FUNC_EVENT_EVENT_CFCM_ERROR:
		break;
	case CREQ_FUNC_EVENT_EVENT_TIM_ERROR:
		break;
	default:
		return -EINVAL;
	}
	return 0;
}

static int bnxt_re_handle_qp_async_event(void *qp_event, struct bnxt_re_qp *qp)
{
	struct creq_qp_error_notification *err_event;
	struct ib_event event;
	unsigned int flags;

	if (qp->qplib_qp.state == CMDQ_MODIFY_QP_NEW_STATE_ERR &&
	    !qp->qplib_qp.is_user) {
		flags = bnxt_re_lock_cqs(qp);
		bnxt_qplib_add_flush_qp(&qp->qplib_qp);
		bnxt_re_unlock_cqs(qp, flags);
	}
	memset(&event, 0, sizeof(event));
	event.device = &qp->rdev->ibdev;
	event.element.qp = &qp->ib_qp;

	err_event = qp_event;
	switch(err_event->res_err_state_reason) {
	case CFCQ_RES_ERR_STATE_REASON_RES_EXCEED_MAX:
	case CFCQ_RES_ERR_STATE_REASON_RES_PAYLOAD_LENGTH_MISMATCH:
	case CFCQ_RES_ERR_STATE_REASON_RES_OPCODE_ERROR:
	case CFCQ_RES_ERR_STATE_REASON_RES_PSN_SEQ_ERROR_RETRY_LIMIT:
	case CFCQ_RES_ERR_STATE_REASON_RES_RX_INVALID_R_KEY:
	case CFCQ_RES_ERR_STATE_REASON_RES_RX_DOMAIN_ERROR:
	case CFCQ_RES_ERR_STATE_REASON_RES_RX_NO_PERMISSION:
	case CFCQ_RES_ERR_STATE_REASON_RES_RX_RANGE_ERROR:
	case CFCQ_RES_ERR_STATE_REASON_RES_TX_INVALID_R_KEY:
	case CFCQ_RES_ERR_STATE_REASON_RES_TX_DOMAIN_ERROR:
	case CFCQ_RES_ERR_STATE_REASON_RES_TX_NO_PERMISSION:
	case CFCQ_RES_ERR_STATE_REASON_RES_TX_RANGE_ERROR:
	case CFCQ_RES_ERR_STATE_REASON_RES_IVALID_DUP_RKEY:
		event.event = IB_EVENT_QP_ACCESS_ERR;
		break;
	case CFCQ_RES_ERR_STATE_REASON_RES_EXCEEDS_WQE:
	case CFCQ_RES_ERR_STATE_REASON_RES_WQE_FORMAT_ERROR:
	case CFCQ_RES_ERR_STATE_REASON_RES_SRQ_LOAD_ERROR:
		/* FIXME:Enable this after confirmation
		 * from f/w.Not handling for now
		 * if(qp->qplib_qp.srq)
		 *	event.event = IB_EVENT_SRQ_ERR;
		 */
		break;
	case CFCQ_RES_ERR_STATE_REASON_RES_IRRQ_OFLOW:
		event.event = IB_EVENT_QP_FATAL;
		break;
	case CFCQ_RES_ERR_STATE_REASON_RES_UNSUPPORTED_OPCODE:
		event.event = IB_EVENT_QP_REQ_ERR;
		break;
	case CFCQ_RES_ERR_STATE_REASON_RES_CMP_ERROR:
	case CFCQ_RES_ERR_STATE_REASON_RES_CQ_LOAD_ERROR:
		/* FIXME:Enable this after confirmation
		* from f/w. Not reporting for now
		* event.event = IB_EVENT_CQ_ERR;
		*/
		break;
	case CFCQ_RES_ERR_STATE_REASON_RES_TX_PCI_ERROR:
	case CFCQ_RES_ERR_STATE_REASON_RES_RX_PCI_ERROR:
		/* FIXME:Enable this after confirmation
		 * from f/w. Not reporting for now
		 * event.event = IB_EVENT_DEVICE_FATAL;
		 */
		break;
	default:
		if (qp->qplib_qp.srq)
			event.event = IB_EVENT_QP_LAST_WQE_REACHED;
		break;
	}

	if (event.event && event.device && qp->ib_qp.event_handler)
		qp->ib_qp.event_handler(&event, qp->ib_qp.qp_context);

	return 0;
}

static int bnxt_re_handle_cq_async_error(void *event, struct bnxt_re_cq *cq)
{
	struct creq_cq_error_notification *cqerr;
	bool send = false;

	cqerr = event;
	switch (cqerr->cq_err_reason) {
	case CREQ_CQ_ERROR_NOTIFICATION_CQ_ERR_REASON_REQ_CQ_INVALID_ERROR:
	case CREQ_CQ_ERROR_NOTIFICATION_CQ_ERR_REASON_REQ_CQ_OVERFLOW_ERROR:
	case CREQ_CQ_ERROR_NOTIFICATION_CQ_ERR_REASON_REQ_CQ_LOAD_ERROR:
	case CREQ_CQ_ERROR_NOTIFICATION_CQ_ERR_REASON_RES_CQ_INVALID_ERROR:
	case CREQ_CQ_ERROR_NOTIFICATION_CQ_ERR_REASON_RES_CQ_OVERFLOW_ERROR:
	case CREQ_CQ_ERROR_NOTIFICATION_CQ_ERR_REASON_RES_CQ_LOAD_ERROR:
		send = true;
	default:
		break;
	}

	if (send && cq->ib_cq.event_handler) {
		struct ib_event event = {};

		event.event = IB_EVENT_CQ_ERR;
		event.element.cq = &cq->ib_cq;
		event.device = &cq->rdev->ibdev;
		event.element.port_num = 0;
		cq->ib_cq.event_handler(&event, cq->ib_cq.cq_context);
	}

	return 0;
}

static int bnxt_re_handle_affi_async_event(
		struct creq_qp_event *affi_async, void *obj)
{
	struct bnxt_qplib_qp *qplqp;
	struct bnxt_qplib_cq *qplcq;
	struct bnxt_re_qp *qp;
	struct bnxt_re_cq *cq;
	int rc = 0;
	u8 event;

	if (!obj)
		return rc; /* QP was already dead, still return success */

	event = affi_async->event;
	switch (event) {
	case CREQ_QP_EVENT_EVENT_QP_ERROR_NOTIFICATION:
		qplqp = obj;
		qp = container_of(qplqp, struct bnxt_re_qp, qplib_qp);
		/*FIXME: QP referencing */
		rc = bnxt_re_handle_qp_async_event(affi_async, qp);
		break;
	case CREQ_QP_EVENT_EVENT_CQ_ERROR_NOTIFICATION:
		qplcq = obj;
		cq = container_of(qplcq, struct bnxt_re_cq, qplib_cq);
		rc = bnxt_re_handle_cq_async_error(affi_async, cq);
		break;
	default:
		rc = -EINVAL;
	}

	return rc;
}

static int bnxt_re_aeq_handler(struct bnxt_qplib_rcfw *rcfw,
			       void *aeqe, void *obj)
{
	struct creq_func_event *unaffi_async;
	struct creq_qp_event *affi_async;
	u8 type;
	int rc;

	type = ((struct creq_base *)aeqe)->type;
	if (type == CREQ_BASE_TYPE_FUNC_EVENT) {
		unaffi_async = aeqe;
		rc = bnxt_re_handle_unaffi_async_event(unaffi_async);
	} else {
		affi_async = aeqe;
		rc = bnxt_re_handle_affi_async_event(affi_async, obj);
	}

	return rc;
}


static int bnxt_re_srqn_handler(struct bnxt_qplib_nq *nq,
				struct bnxt_qplib_srq *handle, u8 event)
{
	struct bnxt_re_srq *srq = to_bnxt_re(handle, struct bnxt_re_srq,
					     qplib_srq);
	struct ib_event ib_event;
	int rc = 0;

	if (srq == NULL) {
		dev_err(NULL, "%s: SRQ is NULL, SRQN not handled",
			ROCE_DRV_MODULE_NAME);
		rc = -EINVAL;
		goto done;
	}
	ib_event.device = &srq->rdev->ibdev;
	ib_event.element.srq = &srq->ib_srq;
	if (event == NQ_SRQ_EVENT_EVENT_SRQ_THRESHOLD_EVENT)
		ib_event.event = IB_EVENT_SRQ_LIMIT_REACHED;
	else
		ib_event.event = IB_EVENT_SRQ_ERR;

	if (srq->ib_srq.event_handler) {
		/* Lock event_handler? */
		(*srq->ib_srq.event_handler)(&ib_event,
					     srq->ib_srq.srq_context);
	}
done:
	return rc;
}

static int bnxt_re_cqn_handler(struct bnxt_qplib_nq *nq,
			       struct bnxt_qplib_cq *handle)
{
	struct bnxt_re_cq *cq = to_bnxt_re(handle, struct bnxt_re_cq,
					   qplib_cq);
	int rc = 0;

	if (cq == NULL) {
		dev_err(NULL, "%s: CQ is NULL, CQN not handled",
			ROCE_DRV_MODULE_NAME);
		rc = -EINVAL;
		goto done;
	}
	/* CQ already in destroy path. Do not handle any more events */
	if (handle->destroyed || !atomic_read(&cq->ib_cq.usecnt)) {
		if (!handle->destroyed)
			dev_err(NULL, "%s: CQ being destroyed, CQN not handled",
				ROCE_DRV_MODULE_NAME);
		return 0;
	}

	if (cq->ib_cq.comp_handler) {
		/* Lock comp_handler? */
		(*cq->ib_cq.comp_handler)(&cq->ib_cq, cq->ib_cq.cq_context);
	}
done:
	return rc;
}

struct bnxt_qplib_nq * bnxt_re_get_nq(struct bnxt_re_dev *rdev)
{
	int min, indx;

	mutex_lock(&rdev->nqr.load_lock);
	for (indx = 0, min = 0; indx < (rdev->nqr.num_msix - 1); indx++) {
		if (rdev->nqr.nq[min].load > rdev->nqr.nq[indx].load)
			min = indx;
	}
	rdev->nqr.nq[min].load++;
	mutex_unlock(&rdev->nqr.load_lock);

	return &rdev->nqr.nq[min];
}

void bnxt_re_put_nq(struct bnxt_re_dev *rdev, struct bnxt_qplib_nq *nq)
{
	mutex_lock(&rdev->nqr.load_lock);
		nq->load--;
	mutex_unlock(&rdev->nqr.load_lock);
}

static bool bnxt_re_check_min_attr(struct bnxt_re_dev *rdev)
{
	struct bnxt_qplib_dev_attr *attr;
	bool rc = true;

	attr = rdev->dev_attr;

	if (!attr->max_cq || !attr->max_qp ||/* !attr->max_dpi ||*/
	    !attr->max_sgid || !attr->max_mr) {
		dev_err(rdev_to_dev(rdev),"Insufficient RoCE resources");
		dev_dbg(rdev_to_dev(rdev),
			"max_cq = %d, max_qp = %d, max_dpi = %d, max_sgid = %d, max_mr = %d",
			attr->max_cq, attr->max_qp, attr->max_dpi,
			attr->max_sgid, attr->max_mr);
		rc = false;
	}
	return rc;
}

static void bnxt_re_dispatch_event(struct ib_device *ibdev, struct ib_qp *qp,
				   u8 port_num, enum ib_event_type event)
{
	struct ib_event ib_event;

	ib_event.device = ibdev;
	if (qp) {
		ib_event.element.qp = qp;
		ib_event.event = event;
		if (qp->event_handler)
			qp->event_handler(&ib_event, qp->qp_context);
	} else {
		ib_event.element.port_num = port_num;
		ib_event.event = event;
		ib_dispatch_event(&ib_event);
	}

	dev_dbg(rdev_to_dev(to_bnxt_re_dev(ibdev, ibdev)),
		"ibdev %p Event 0x%x port_num 0x%x", ibdev, event, port_num);
}

static bool bnxt_re_is_qp1_or_shadow_qp(struct bnxt_re_dev *rdev,
					struct bnxt_re_qp *qp)
{
	if (rdev->gsi_ctx.gsi_qp_mode == BNXT_RE_GSI_MODE_ALL)
		return (qp->ib_qp.qp_type == IB_QPT_GSI) ||
			(qp == rdev->gsi_ctx.gsi_sqp);
	else
		return (qp->ib_qp.qp_type == IB_QPT_GSI);
}

void bnxt_re_stop_all_nonqp1_nonshadow_qps(struct bnxt_re_dev *rdev)
{
	struct bnxt_qplib_qp *qpl_qp;
	bool dev_detached = false;
	struct ib_qp_attr qp_attr;
	int mask = IB_QP_STATE;
	struct bnxt_re_qp *qp;
	unsigned long flags;

	if (!rdev)
		return;

	if (test_bit(BNXT_RE_FLAG_ERR_DEVICE_DETACHED, &rdev->flags))
		dev_detached = true;

	qp_attr.qp_state = IB_QPS_ERR;
	mutex_lock(&rdev->qp_lock);
	list_for_each_entry(qp, &rdev->qp_list, list) {
		qpl_qp = &qp->qplib_qp;
		if (dev_detached || !bnxt_re_is_qp1_or_shadow_qp(rdev, qp)) {
			if (qpl_qp->state !=
			    CMDQ_MODIFY_QP_NEW_STATE_RESET &&
			    qpl_qp->state !=
			    CMDQ_MODIFY_QP_NEW_STATE_ERR) {
				bnxt_re_dispatch_event(&rdev->ibdev, &qp->ib_qp,
						       1, IB_EVENT_QP_FATAL);
				if (dev_detached) {
					/*
					 * Cant actually send the command down,
					 * marking the state for bookkeeping
					 */
					qpl_qp->state =
						CMDQ_MODIFY_QP_NEW_STATE_ERR;
					if (!qpl_qp->is_user) {
						/* Add to flush list */
						flags = bnxt_re_lock_cqs(qp);
						bnxt_qplib_add_flush_qp(qpl_qp);
						bnxt_re_unlock_cqs(qp, flags);
					}
				} else {
					bnxt_re_modify_qp(&qp->ib_qp,
							  &qp_attr, mask,
							  NULL);
				}
			}
		}
	}

	mutex_unlock(&rdev->qp_lock);
}

static int bnxt_re_update_gid(struct bnxt_re_dev *rdev)
{
	struct bnxt_qplib_sgid_tbl *sgid_tbl = &rdev->qplib_res.sgid_tbl;
	struct bnxt_qplib_gid gid;
	u16 gid_idx, index;
	int rc = 0;

	if (!test_bit(BNXT_RE_FLAG_IBDEV_REGISTERED, &rdev->flags))
		return 0;

	if (sgid_tbl == NULL) {
		dev_err(rdev_to_dev(rdev), "QPLIB: SGID table not allocated");
		return -EINVAL;
	}

	for (index = 0; index < sgid_tbl->active; index++) {
		gid_idx = sgid_tbl->hw_id[index];

		if (!memcmp(&sgid_tbl->tbl[index], &bnxt_qplib_gid_zero,
			    sizeof(bnxt_qplib_gid_zero)))
			continue;
		/* Need to modify the VLAN enable setting of non VLAN GID only
		 * as setting is done for VLAN GID while adding GID
		 *
		 * If disable_prio_vlan_tx is enable, then we'll need to remove the
		 * vlan entry from the sgid_tbl.
		 */
		if (sgid_tbl->vlan[index] == true)
			continue;

		memcpy(&gid, &sgid_tbl->tbl[index], sizeof(gid));

		rc = bnxt_qplib_update_sgid(sgid_tbl, &gid, gid_idx,
					    rdev->qplib_res.netdev->dev_addr);
	}

	return rc;
}

static void bnxt_re_get_pri_and_update_gid(struct bnxt_re_dev *rdev)
{
	u8 prio_map = 0;
	/* Get priority for roce */
	prio_map = bnxt_re_get_priority_mask(rdev,
					     (IEEE_8021QAZ_APP_SEL_ETHERTYPE |
					      IEEE_8021QAZ_APP_SEL_DGRAM));
	if (prio_map == rdev->cur_prio_map)
		return;

	rdev->cur_prio_map = prio_map;
	if ((prio_map == 0 && rdev->qplib_res.prio == true) ||
	    (prio_map != 0 && rdev->qplib_res.prio == false)) {
		if (!rdev->cc_param.disable_prio_vlan_tx) {
			rdev->qplib_res.prio = prio_map ? true : false;
			bnxt_re_update_gid(rdev);
		}
	}
}

static void bnxt_re_clear_cc(struct bnxt_re_dev *rdev)
{
	struct bnxt_qplib_cc_param *cc_param = &rdev->cc_param;

	if (!is_qport_service_type_supported(rdev))
		bnxt_re_clear_dscp(rdev);

	if (_is_chip_sr2(rdev->chip_ctx)) {
		cc_param->mask = CMDQ_MODIFY_ROCE_CC_MODIFY_MASK_TOS_DSCP;
	} else {
		cc_param->mask = (CMDQ_MODIFY_ROCE_CC_MODIFY_MASK_CC_MODE |
				  CMDQ_MODIFY_ROCE_CC_MODIFY_MASK_ENABLE_CC |
				  CMDQ_MODIFY_ROCE_CC_MODIFY_MASK_TOS_ECN);

		if (!is_qport_service_type_supported(rdev))
			cc_param->mask |=
			(CMDQ_MODIFY_ROCE_CC_MODIFY_MASK_ALT_VLAN_PCP |
			 CMDQ_MODIFY_ROCE_CC_MODIFY_MASK_ALT_TOS_DSCP |
			 CMDQ_MODIFY_ROCE_CC_MODIFY_MASK_TOS_DSCP);
	}

	cc_param->cur_mask  = cc_param->mask;

	if (bnxt_qplib_modify_cc(&rdev->qplib_res, cc_param))
		dev_err(rdev_to_dev(rdev), "Failed to modify cc\n");
}

static int bnxt_re_setup_cc(struct bnxt_re_dev *rdev)
{
	struct bnxt_qplib_cc_param *cc_param = &rdev->cc_param;
	int rc;

	if (_is_chip_sr2(rdev->chip_ctx)) {
		cc_param->enable = 0x0;
		cc_param->mask = CMDQ_MODIFY_ROCE_CC_MODIFY_MASK_TOS_DSCP;
	} else {
		cc_param->enable = 0x1;
		cc_param->mask = (CMDQ_MODIFY_ROCE_CC_MODIFY_MASK_CC_MODE |
				  CMDQ_MODIFY_ROCE_CC_MODIFY_MASK_ENABLE_CC |
				  CMDQ_MODIFY_ROCE_CC_MODIFY_MASK_TOS_ECN);

		if (!is_qport_service_type_supported(rdev))
			cc_param->mask |=
			(CMDQ_MODIFY_ROCE_CC_MODIFY_MASK_ALT_VLAN_PCP |
			 CMDQ_MODIFY_ROCE_CC_MODIFY_MASK_ALT_TOS_DSCP |
			 CMDQ_MODIFY_ROCE_CC_MODIFY_MASK_TOS_DSCP);
	}

	cc_param->cur_mask  = cc_param->mask;

	rc = bnxt_qplib_modify_cc(&rdev->qplib_res, cc_param);
	if (rc) {
		dev_err(rdev_to_dev(rdev), "Failed to modify cc\n");
		return rc;
	}
	if (!is_qport_service_type_supported(rdev)) {
		rc = bnxt_re_setup_dscp(rdev);
		if (rc)
			goto clear;
	}

	/* Reset the programming mask */
	cc_param->mask = 0;

	return 0;

clear:
	bnxt_re_clear_cc(rdev);
	return rc;
}

int bnxt_re_query_hwrm_dscp2pri(struct bnxt_re_dev *rdev,
				struct bnxt_re_dscp2pri *d2p, u16 *count,
				u16 target_id)
{
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	struct bnxt *bp = netdev_priv(en_dev->net);
	struct hwrm_queue_dscp2pri_qcfg_input req;
	struct hwrm_queue_dscp2pri_qcfg_output resp;
	struct bnxt_re_dscp2pri *dscp2pri;
	struct bnxt_fw_msg fw_msg;
	u16 data_len = 3 * 256; /*FIXME: Hard coding */
	u16 in_count = *count;
	dma_addr_t dma_handle;
	int rc = 0, i;
	u8 *kmem;

	memset(&fw_msg, 0, sizeof(fw_msg));
	memset(&req, 0, sizeof(req));
	bnxt_re_init_hwrm_hdr(rdev, (void *)&req,
			      HWRM_QUEUE_DSCP2PRI_QCFG, -1, target_id);
	req.port_id = (target_id == 0xFFFF) ? bp->pf.port_id : 1;

	kmem = dma_alloc_coherent(&bp->pdev->dev, data_len, &dma_handle,
				  GFP_KERNEL);
	if (!kmem) {
		dev_err(rdev_to_dev(rdev),
			"dma_alloc_coherent failure, length = %u\n",
			(unsigned)data_len);
		return -ENOMEM;
	}
	memset(kmem, 0, data_len);
	req.dest_data_addr = cpu_to_le64(dma_handle);
	req.dest_data_buffer_size = cpu_to_le16(data_len);
	bnxt_re_fill_fw_msg(&fw_msg, (void *)&req, sizeof(req), (void *)&resp,
			    sizeof(resp), DFLT_HWRM_CMD_TIMEOUT);
	rc = en_dev->en_ops->bnxt_send_fw_msg(en_dev, BNXT_ROCE_ULP, &fw_msg);
	if (rc)
		goto out;

	/* Upload the DSCP-MASK-PRI tuple(s) */
	dscp2pri = (struct bnxt_re_dscp2pri *)kmem;
	for (i = 0; i < le16_to_cpu(resp.entry_cnt) && i < in_count; i++) {
		d2p[i].dscp = dscp2pri->dscp;
		d2p[i].mask = dscp2pri->mask;
		d2p[i].pri = dscp2pri->pri;
		dscp2pri++;
	}
	*count = le16_to_cpu(resp.entry_cnt);
out:
	dma_free_coherent(&bp->pdev->dev, data_len, kmem, dma_handle);
	return rc;
}

int bnxt_re_prio_vlan_tx_update(struct bnxt_re_dev *rdev)
{
	/* Remove the VLAN from the GID entry */
	if (rdev->cc_param.disable_prio_vlan_tx)
		rdev->qplib_res.prio = false;
	else
		rdev->qplib_res.prio = true;

	return bnxt_re_update_gid(rdev);
}

int bnxt_re_set_hwrm_dscp2pri(struct bnxt_re_dev *rdev,
			      struct bnxt_re_dscp2pri *d2p, u16 count,
			      u16 target_id)
{
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	struct bnxt *bp = netdev_priv(en_dev->net);
	struct hwrm_queue_dscp2pri_cfg_input req;
	struct hwrm_queue_dscp2pri_cfg_output resp;
	struct bnxt_fw_msg fw_msg;
	struct bnxt_re_dscp2pri *dscp2pri;
	int i, rc, data_len = 3 * 256;
	dma_addr_t dma_handle;
	u8 *kmem;

	memset(&req, 0, sizeof(req));
	memset(&fw_msg, 0, sizeof(fw_msg));
	bnxt_re_init_hwrm_hdr(rdev, (void *)&req,
			      HWRM_QUEUE_DSCP2PRI_CFG, -1, target_id);
	req.port_id = (target_id == 0xFFFF) ? bp->pf.port_id : 1;

	kmem = dma_alloc_coherent(&bp->pdev->dev, data_len, &dma_handle,
				  GFP_KERNEL);
	if (!kmem) {
		dev_err(rdev_to_dev(rdev),
			"dma_alloc_coherent failure, length = %u\n",
			(unsigned)data_len);
		return -ENOMEM;
	}
	req.src_data_addr = cpu_to_le64(dma_handle);

	/* Download the DSCP-MASK-PRI tuple(s) */
	dscp2pri = (struct bnxt_re_dscp2pri *)kmem;
	for (i = 0; i < count; i++) {
		dscp2pri->dscp = d2p[i].dscp;
		dscp2pri->mask = d2p[i].mask;
		dscp2pri->pri = d2p[i].pri;
		dscp2pri++;
	}

	req.entry_cnt = cpu_to_le16(count);
	bnxt_re_fill_fw_msg(&fw_msg, (void *)&req, sizeof(req), (void *)&resp,
			    sizeof(resp), DFLT_HWRM_CMD_TIMEOUT);
	rc = en_dev->en_ops->bnxt_send_fw_msg(en_dev, BNXT_ROCE_ULP, &fw_msg);
	dma_free_coherent(&bp->pdev->dev, data_len, kmem, dma_handle);
	return rc;
}

int bnxt_re_query_hwrm_qportcfg(struct bnxt_re_dev *rdev,
			struct bnxt_re_tc_rec *tc_rec, u16 tid)
{
	struct bnxt *bp = netdev_priv(rdev->en_dev->net);
	u8 max_tc, tc, *qptr, *type_ptr0, *type_ptr1;
	struct hwrm_queue_qportcfg_output resp = {0};
	struct hwrm_queue_qportcfg_input req = {0};
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	struct bnxt_fw_msg fw_msg;
	bool def_init = false;
	u8 *tmp_type;
	u8 cos_id;
	int rc;

	memset(&fw_msg, 0, sizeof(fw_msg));
	bnxt_re_init_hwrm_hdr(rdev, (void *)&req, HWRM_QUEUE_QPORTCFG,
			      -1, tid);
	req.port_id = (tid == 0xFFFF) ? bp->pf.port_id : 1;
	bnxt_re_fill_fw_msg(&fw_msg, (void *)&req, sizeof(req), (void *)&resp,
			    sizeof(resp), DFLT_HWRM_CMD_TIMEOUT);
	rc = en_dev->en_ops->bnxt_send_fw_msg(en_dev, BNXT_ROCE_ULP, &fw_msg);
	if (rc)
		goto exit;

	if (!resp.max_configurable_queues) {
                rc = -EINVAL;
                goto exit;
        }

        max_tc = resp.max_configurable_queues;
	tc_rec->max_tc = max_tc;

	if (resp.queue_cfg_info & QUEUE_QPORTCFG_RESP_QUEUE_CFG_INFO_USE_PROFILE_TYPE)
		tc_rec->serv_type_enabled = true;

	qptr = &resp.queue_id0;
	type_ptr0 = &resp.queue_id0_service_profile_type;
	type_ptr1 = &resp.queue_id1_service_profile_type;
	for (tc = 0; tc < max_tc; tc++) {
		tmp_type = tc ? type_ptr1 + (tc - 1) : type_ptr0;

		cos_id = *qptr++;
		/* RoCE CoS queue is the first cos queue.
		 * For MP12 and MP17 order is 405 and 141015.
		 */
		if (is_bnxt_roce_queue(rdev, *qptr, *tmp_type)) {
			tc_rec->cos_id_roce = cos_id;
			tc_rec->tc_roce = tc;
		} else if (is_bnxt_cnp_queue(rdev, *qptr, *tmp_type)) {
			tc_rec->cos_id_cnp = cos_id;
			tc_rec->tc_cnp = tc;
		} else if (!def_init) {
			def_init = true;
			tc_rec->tc_def = tc;
			tc_rec->cos_id_def = cos_id;
		}
		qptr++;
	}
exit:
	return rc;
}

int bnxt_re_hwrm_cos2bw_qcfg(struct bnxt_re_dev *rdev, u16 target_id,
			     struct bnxt_re_cos2bw_cfg *cfg)
{
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	struct bnxt *bp = netdev_priv(en_dev->net);
	struct hwrm_queue_cos2bw_qcfg_output resp;
        struct hwrm_queue_cos2bw_qcfg_input req = {0};
	struct bnxt_fw_msg fw_msg;
	int rc, indx;
	void *data;

	memset(&fw_msg, 0, sizeof(fw_msg));
	bnxt_re_init_hwrm_hdr(rdev, (void *)&req,
			      HWRM_QUEUE_COS2BW_QCFG, -1, target_id);
	req.port_id = (target_id == 0xFFFF) ? bp->pf.port_id : 1;

	bnxt_re_fill_fw_msg(&fw_msg, (void *)&req, sizeof(req), (void *)&resp,
			    sizeof(resp), DFLT_HWRM_CMD_TIMEOUT);
	rc = en_dev->en_ops->bnxt_send_fw_msg(en_dev, BNXT_ROCE_ULP, &fw_msg);
	if (rc)
		goto exit;
	data = &resp.queue_id0 + offsetof(struct bnxt_re_cos2bw_cfg,
					  queue_id);
	for (indx = 0; indx < 8; indx++, data += (sizeof(*cfg) - 4)) {
		memcpy(&cfg->queue_id, data, sizeof(*cfg) - 4);
		if (indx == 0)
			cfg->queue_id = resp.queue_id0;
		cfg++;
	}
exit:
	return rc;
}

int bnxt_re_hwrm_cos2bw_cfg(struct bnxt_re_dev *rdev, u16 target_id,
			    struct bnxt_re_cos2bw_cfg *cfg)
{
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	struct bnxt *bp = netdev_priv(en_dev->net);
        struct hwrm_queue_cos2bw_cfg_input req = {0};
	struct hwrm_queue_cos2bw_cfg_output resp = {0};
	struct bnxt_fw_msg fw_msg;
	void *data;
	int indx;
	int rc;

	memset(&fw_msg, 0, sizeof(fw_msg));
	bnxt_re_init_hwrm_hdr(rdev, (void *)&req,
			      HWRM_QUEUE_COS2BW_CFG, -1, target_id);
	req.port_id = (target_id == 0xFFFF) ? bp->pf.port_id : 1;

	/* Chimp wants enable bit to retain previous
	 * config done by L2 driver
	 */
	for (indx = 0; indx < 8; indx++) {
		if (cfg[indx].queue_id < 40) {
			req.enables |= cpu_to_le32(
				QUEUE_COS2BW_CFG_REQ_ENABLES_COS_QUEUE_ID0_VALID <<
				indx);
		}

		data = (char *)&req.unused_0 + indx * (sizeof(*cfg) - 4);
		memcpy(data, &cfg[indx].queue_id, sizeof(*cfg) - 4);
		if (indx == 0) {
			req.queue_id0 = cfg[0].queue_id;
			req.unused_0 = 0;
		}
	}

	memset(&resp, 0, sizeof(resp));
	bnxt_re_fill_fw_msg(&fw_msg, (void *)&req, sizeof(req), (void *)&resp,
			    sizeof(resp), DFLT_HWRM_CMD_TIMEOUT);
	rc = en_dev->en_ops->bnxt_send_fw_msg(en_dev, BNXT_ROCE_ULP, &fw_msg);
	return rc;
}

static int bnxt_re_hwrm_pri2cos_qcfg(struct bnxt_re_dev *rdev,
				     struct bnxt_re_tc_rec *tc_rec,
				     u16 target_id,
				     u8 *prio_valid,
				     u8 *roce_prio, u8 *cnp_prio)
{
	struct hwrm_queue_pri2cos_qcfg_output resp = {0};
	struct hwrm_queue_pri2cos_qcfg_input req = {0};
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	struct bnxt_fw_msg fw_msg;
	u8 *pri2cos, queue_id;
	int rc, i;

	memset(&fw_msg, 0, sizeof(fw_msg));
	memset(&req, 0, sizeof(req));
	bnxt_re_init_hwrm_hdr(rdev, (void *)&req,
			      HWRM_QUEUE_PRI2COS_QCFG,
			      -1, target_id);

	req.flags = cpu_to_le32(QUEUE_PRI2COS_QCFG_REQ_FLAGS_IVLAN);
	bnxt_re_fill_fw_msg(&fw_msg, (void *)&req, sizeof(req), (void *)&resp,
			    sizeof(resp), DFLT_HWRM_CMD_TIMEOUT);
	rc = rdev->en_dev->en_ops->bnxt_send_fw_msg(en_dev, BNXT_ROCE_ULP,
						    &fw_msg);
	if (rc)
		return rc;

	pri2cos = &resp.pri0_cos_queue_id;
	for (i = 0; i < IEEE_8021QAZ_MAX_TCS; i++) {
		queue_id = pri2cos[i];
		if (queue_id == tc_rec->cos_id_cnp) {
			*cnp_prio = i;
			*prio_valid |= (1 << CNP_PRIO_VALID);
		} else if (queue_id == tc_rec->cos_id_roce) {
			*roce_prio = i;
			*prio_valid |= (1 << ROCE_PRIO_VALID);
		}
	}
	return rc;
}

int bnxt_re_hwrm_pri2cos_cfg(struct bnxt_re_dev *rdev,
			     u16 target_id, u16 port_id,
			     u8 *cos_id_map, u8 pri_map)
{
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	struct hwrm_queue_pri2cos_cfg_input req = {0};
	struct hwrm_queue_pri2cos_cfg_output resp = {0};
	struct bnxt_fw_msg fw_msg;
	u32 flags = 0;
	u8 *pri2cos;
	int rc = 0;
	int i;

	memset(&fw_msg, 0, sizeof(fw_msg));
	memset(&req, 0, sizeof(req));
	bnxt_re_init_hwrm_hdr(rdev, (void *)&req,
			      HWRM_QUEUE_PRI2COS_CFG, -1, target_id);
	flags = (QUEUE_PRI2COS_CFG_REQ_FLAGS_PATH_BIDIR |
		 QUEUE_PRI2COS_CFG_REQ_FLAGS_IVLAN);
	req.flags = cpu_to_le32(flags);
	req.port_id = port_id;

	pri2cos = &req.pri0_cos_queue_id;

	for (i = 0; i < 8; i++) {
		if (pri_map & (1 << i)) {
			req.enables |= cpu_to_le32
			(QUEUE_PRI2COS_CFG_REQ_ENABLES_PRI0_COS_QUEUE_ID << i);
			pri2cos[i] = cos_id_map[i];
		}
	}

	bnxt_re_fill_fw_msg(&fw_msg, (void *)&req, sizeof(req), (void *)&resp,
			    sizeof(resp), DFLT_HWRM_CMD_TIMEOUT);
	rc = en_dev->en_ops->bnxt_send_fw_msg(en_dev, BNXT_ROCE_ULP, &fw_msg);

	return rc;
}

int bnxt_re_cnp_pri2cos_cfg(struct bnxt_re_dev *rdev, u16 target_id, bool reset)
{
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	struct bnxt *bp = netdev_priv(en_dev->net);
	struct bnxt_re_tc_rec *tc_rec;
	u8 cos_id_map[8] = {0};
	u8 pri_map = 0;
	u16 port_id;
	u8 cnp_pri;

	cnp_pri = rdev->cc_param.alt_vlan_pcp;

	if (target_id == 0xFFFF) {
		port_id = bp->pf.port_id;
		tc_rec = &rdev->tc_rec[0];
	} else {
		port_id = 1;
		tc_rec = &rdev->tc_rec[1];
	}

	pri_map |= (1 << cnp_pri);
	if (reset)
		cos_id_map[cnp_pri] = tc_rec->cos_id_def;
	else
		cos_id_map[cnp_pri] = tc_rec->cos_id_cnp;

	return bnxt_re_hwrm_pri2cos_cfg(rdev, target_id, port_id,
					cos_id_map, pri_map);
}

int bnxt_re_host_pf_id_query(struct bnxt_re_dev *rdev,
			     struct bnxt_qplib_query_fn_info *fn_info,
			     u32 *pf_mask, u32 *first_pf)
{
	struct hwrm_func_host_pf_ids_query_output resp = {0};
	struct hwrm_func_host_pf_ids_query_input req;
	struct bnxt_en_dev *en_dev = rdev->en_dev;
	struct bnxt_fw_msg fw_msg;
	int rc;

	memset(&fw_msg, 0, sizeof(fw_msg));
	memset(&req, 0, sizeof(req));
	bnxt_re_init_hwrm_hdr(rdev, (void *)&req,
			      HWRM_FUNC_HOST_PF_IDS_QUERY, -1, -1);
	/* To query the info from the host EPs */
	switch (fn_info->host) {
		case FUNC_HOST_PF_IDS_QUERY_REQ_HOST_SOC:
		case FUNC_HOST_PF_IDS_QUERY_REQ_HOST_EP_0:
		case FUNC_HOST_PF_IDS_QUERY_REQ_HOST_EP_1:
		case FUNC_HOST_PF_IDS_QUERY_REQ_HOST_EP_2:
		case FUNC_HOST_PF_IDS_QUERY_REQ_HOST_EP_3:
			req.host = fn_info->host;
		break;
		default:
			req.host = FUNC_HOST_PF_IDS_QUERY_REQ_HOST_EP_0;
		break;
	}

	req.filter = fn_info->filter;
	if (req.filter > FUNC_HOST_PF_IDS_QUERY_REQ_FILTER_ROCE)
		req.filter = FUNC_HOST_PF_IDS_QUERY_REQ_FILTER_ALL;

	bnxt_re_fill_fw_msg(&fw_msg, (void *)&req, sizeof(req), (void *)&resp,
			    sizeof(resp), DFLT_HWRM_CMD_TIMEOUT);
	rc = en_dev->en_ops->bnxt_send_fw_msg(en_dev, BNXT_ROCE_ULP, &fw_msg);


	*first_pf = le16_to_cpu(resp.first_pf_id);
	*pf_mask = le16_to_cpu(resp.pf_ordinal_mask);

	return rc;
}

static int bnxt_re_put_stats_ctx(struct bnxt_re_dev *rdev)
{
	struct bnxt_qplib_ctx *hctx;
	struct bnxt_qplib_res *res;
	struct bnxt *slave_bp;
	u16 tid;

	res = &rdev->qplib_res;
	hctx = res->hctx;

	if (test_and_clear_bit(BNXT_RE_FLAG_STATS_CTX2_ALLOC,
			       &rdev->flags)) {
		if (rdev->binfo) {
			slave_bp =  netdev_priv(rdev->binfo->slave2);
			tid = PCI_FUNC(slave_bp->pdev->devfn) + 1;
			bnxt_re_net_stats_ctx_free(rdev, hctx->stats2.fw_id,
						   tid);
			bnxt_qplib_free_stat_mem(res, &hctx->stats2);
		}
	}

	tid = 0xffff;
	if (test_and_clear_bit(BNXT_RE_FLAG_STATS_CTX_ALLOC, &rdev->flags)) {
		bnxt_re_net_stats_ctx_free(rdev, hctx->stats.fw_id, tid);
		bnxt_qplib_free_stat_mem(res, &hctx->stats);
	}

	return 0;
}

static int bnxt_re_get_stats_ctx(struct bnxt_re_dev *rdev)
{
	struct bnxt_qplib_ctx *hctx;
	struct bnxt_qplib_res *res;
	u16 tid = 0xffff;
	int rc;

	res = &rdev->qplib_res;
	hctx = res->hctx;

	rc = bnxt_qplib_alloc_stat_mem(res, &hctx->stats);
	if (rc)
		return -ENOMEM;
	rc = bnxt_re_net_stats_ctx_alloc(rdev, tid);
	if (rc)
		goto free_stat_mem;
	set_bit(BNXT_RE_FLAG_STATS_CTX_ALLOC, &rdev->flags);

	if (rdev->binfo) {
		struct bnxt *slave_bp;

		rc = bnxt_qplib_alloc_stat_mem(res, &hctx->stats2);
		if (rc)
			goto free_net_ctx;

		slave_bp = netdev_priv(rdev->binfo->slave2);
		tid = (PCI_FUNC(slave_bp->pdev->devfn) + 1);
		rc = bnxt_re_net_stats_ctx_alloc(rdev, tid);
		if (rc)
			goto free_stat_mem2;
		dev_dbg(rdev_to_dev(rdev), " LAG second stat context %x\n",
			rdev->qplib_res.hctx->stats2.fw_id);
		set_bit(BNXT_RE_FLAG_STATS_CTX2_ALLOC, &rdev->flags);
	}

	return 0;

free_stat_mem2:
	bnxt_qplib_free_stat_mem(res, &hctx->stats2);
free_net_ctx:
	clear_bit(BNXT_RE_FLAG_STATS_CTX_ALLOC, &rdev->flags);
	bnxt_re_net_stats_ctx_free(rdev, hctx->stats.fw_id, tid);
free_stat_mem:
	bnxt_qplib_free_stat_mem(res, &hctx->stats);

	return rc;
}

static int bnxt_re_update_dev_attr(struct bnxt_re_dev *rdev)
{
	int rc;

	rc = bnxt_qplib_get_dev_attr(&rdev->rcfw, rdev->is_virtfn);
	if (rc)
		return rc;
	if (!bnxt_re_check_min_attr(rdev))
		return -EINVAL;
	return 0;
}

static int bnxt_re_set_port_cnp_ets(struct bnxt_re_dev *rdev, u16 portid, bool reset)
{
	struct bnxt_re_cos2bw_cfg ets_cfg[8];
	struct bnxt_re_cos2bw_cfg *cnp_ets_cfg;
	struct bnxt_re_tc_rec *tc_rec;
	int indx, rc;

	rc = bnxt_re_hwrm_cos2bw_qcfg(rdev, portid, ets_cfg);
	if (rc)
		goto bail;

	tc_rec = (portid == 0xFFFF) ? &rdev->tc_rec[0] : &rdev->tc_rec[1];
	indx = tc_rec->cos_id_cnp - tc_rec->cos_id_roce;
	cnp_ets_cfg = &ets_cfg[indx];
	cnp_ets_cfg->queue_id = tc_rec->cos_id_cnp;
	cnp_ets_cfg->tsa = QUEUE_COS2BW_QCFG_RESP_QUEUE_ID0_TSA_ASSIGN_SP;
	cnp_ets_cfg->pri_lvl = 7; /* Max priority for CNP. */

	/* Configure cnp ets to strict */
	rc = bnxt_re_hwrm_cos2bw_cfg(rdev, portid, ets_cfg);
	if (rc)
		goto bail;

	rc = bnxt_re_cnp_pri2cos_cfg(rdev, portid, reset);
bail:
	return rc;
}

static int bnxt_re_setup_port_cnp_cos(struct bnxt_re_dev *rdev, u16 portid, bool reset)
{
	int rc;
	struct bnxt_re_tc_rec *tc_rec;

	/* Query CNP cos id */
	tc_rec = (portid == 0xFFFF) ?
		       &rdev->tc_rec[0] : &rdev->tc_rec[1];
	rc = bnxt_re_query_hwrm_qportcfg(rdev, tc_rec, portid);
	if (rc)
		goto bail;
	/* config ETS */
	rc = bnxt_re_set_port_cnp_ets(rdev, portid, reset);
bail:
	return rc;
}

int bnxt_re_setup_cnp_cos(struct bnxt_re_dev *rdev, bool reset)
{
	u16 portid;
	int rc;

	portid = 0xFFFF;
	rc = bnxt_re_setup_port_cnp_cos(rdev, portid, reset);
	if (rc) {
		dev_err(rdev_to_dev(rdev),
			"Failed to setup cnp cos for pci function %d\n",
			rdev->id);
		goto bail;
	}

	if (rdev->binfo) {
		portid = 2;
		rc = bnxt_re_setup_port_cnp_cos(rdev, portid, reset);
		if (rc)
			dev_err(rdev_to_dev(rdev),
				"Failed to setup cnp cos for pci function %d\n",
				rdev->id);
	}

bail:
	return rc;
}

static void bnxt_re_free_tbls(struct bnxt_re_dev *rdev)
{
	bnxt_qplib_clear_tbls(&rdev->qplib_res);
	bnxt_qplib_free_tbls(&rdev->qplib_res);
}

static int bnxt_re_alloc_init_tbls(struct bnxt_re_dev *rdev)
{
	struct bnxt_qplib_chip_ctx *chip_ctx = rdev->chip_ctx;
	u8 pppp_factor = 0;
	struct bnxt *bp;
	int rc;

	 /*
	  * TODO: Need a better mechanism for spreading of the
	  * 512 extended PPP pages. For now, spreading it
	  * based on port_count
	  */
	if (_is_chip_sr2(chip_ctx) && chip_ctx->modes.db_push) {
		bp = netdev_priv(rdev->en_dev->net);
		pppp_factor = bp->port_count;
	}
	rc = bnxt_qplib_alloc_tbls(&rdev->qplib_res, pppp_factor);
	if (rc)
		return rc;
	bnxt_qplib_init_tbls(&rdev->qplib_res);
	set_bit(BNXT_RE_FLAG_TBLS_ALLOCINIT, &rdev->flags);

	return 0;
}

static void bnxt_re_clean_nqs(struct bnxt_re_dev *rdev)
{
	struct bnxt_qplib_nq *nq;
	int i;

	if (!rdev->nqr.max_init)
		return;

	for (i = (rdev->nqr.max_init - 1) ; i >= 0; i--) {
		nq = &rdev->nqr.nq[i];
		bnxt_qplib_disable_nq(nq);
		bnxt_re_net_ring_free(rdev, nq->ring_id);
		bnxt_qplib_free_nq_mem(nq);
	}
}

static int bnxt_re_setup_nqs(struct bnxt_re_dev *rdev)
{
	struct bnxt_re_ring_attr rattr = {};
	struct bnxt_qplib_ctx *hctx;
	struct bnxt_qplib_nq *nq;
	int rc, i;
	int depth;
	u32 offt;
	u16 vec;

	hctx = rdev->qplib_res.hctx;
	/*
	 * TODO: Optimize the depth based on the
	 * number of NQs.
	 */
	depth = BNXT_QPLIB_NQE_MAX_CNT;
	for (i = 0; i < rdev->nqr.num_msix - 1; i++) {
		nq = &rdev->nqr.nq[i];
		vec = rdev->nqr.msix_entries[i + 1].vector;
		offt = rdev->nqr.msix_entries[i + 1].db_offset;
		nq->hwq.max_elements = depth;
		rc = bnxt_qplib_alloc_nq_mem(&rdev->qplib_res, nq);
		if (rc) {
			dev_err(rdev_to_dev(rdev),
				"Failed to get mem for NQ %d, rc = 0x%x",
				i, rc);
			goto fail_mem;
		}

		rattr.dma_arr = nq->hwq.pbl[PBL_LVL_0].pg_map_arr;
		rattr.pages = nq->hwq.pbl[rdev->nqr.nq[i].hwq.level].pg_count;
		rattr.type = bnxt_re_get_rtype(rdev);
		rattr.mode = RING_ALLOC_REQ_INT_MODE_MSIX;
		rattr.depth = nq->hwq.max_elements - 1;
		rattr.lrid = rdev->nqr.msix_entries[i + 1].ring_idx;
		rc = bnxt_re_net_ring_alloc(rdev, &rattr, &nq->ring_id);
		if (rc) {
			nq->ring_id = 0xffff; /* Invalid ring-id */
			dev_err(rdev_to_dev(rdev),
				"Failed to get fw id for NQ %d, rc = 0x%x",
				i, rc);
			goto fail_ring;
		}

		rc = bnxt_qplib_enable_nq(nq, i, vec, offt,
					  &bnxt_re_cqn_handler,
					  &bnxt_re_srqn_handler);
		if (rc) {
			dev_err(rdev_to_dev(rdev),
				"Failed to enable NQ %d, rc = 0x%x", i, rc);
			goto fail_en;
		}
	}

	rdev->nqr.max_init = i;
	mutex_init(&rdev->nqr.load_lock);
	return 0;
fail_en:
	/* *nq was i'th nq */
	bnxt_re_net_ring_free(rdev, nq->ring_id);
fail_ring:
	bnxt_qplib_free_nq_mem(nq);
fail_mem:
	rdev->nqr.max_init = i;
	return rc;
}

void bnxt_re_sysfs_destroy_file(struct bnxt_re_dev *rdev)
{
#ifndef HAVE_RDMA_SET_DEVICE_SYSFS_GROUP
	int i;

	for (i = 0; i < ARRAY_SIZE(bnxt_re_attributes); i++)
		device_remove_file(&rdev->ibdev.dev, bnxt_re_attributes[i]);
#endif
}

int bnxt_re_sysfs_create_file(struct bnxt_re_dev *rdev)
{
#ifndef HAVE_RDMA_SET_DEVICE_SYSFS_GROUP
	int i, j, rc = 0;

	for (i = 0; i < ARRAY_SIZE(bnxt_re_attributes); i++) {
		rc = device_create_file(&rdev->ibdev.dev,
					bnxt_re_attributes[i]);
		if (rc) {
			dev_err(rdev_to_dev(rdev),
				"Failed to create IB sysfs with rc = 0x%x", rc);
			/* Must clean up all created device files */
			for (j = 0; j < i; j++)
				device_remove_file(&rdev->ibdev.dev,
						   bnxt_re_attributes[j]);
			clear_bit(BNXT_RE_FLAG_IBDEV_REGISTERED, &rdev->flags);
			ib_unregister_device(&rdev->ibdev);
			return 1;
		}
	}
#endif
	return 0;

}

/* worker thread for polling periodic events. Now used for QoS programming*/
static void bnxt_re_worker(struct work_struct *work)
{
	struct bnxt_re_dev *rdev = container_of(work, struct bnxt_re_dev,
						worker.work);
#ifndef HAVE_NETDEV_BONDING_INFO
	struct netdev_bonding_info *netdev_binfo = NULL;
#else
	u8 active_port_map = 0;
#endif
	int rc;

	/* QoS is in 30s cadence for PFs*/
	if (!rdev->is_virtfn && !rdev->worker_30s--) {
		bnxt_re_get_pri_and_update_gid(rdev);
		rdev->worker_30s = 30;
	}

	/* Create the debugfs port directory */
	if (test_bit(BNXT_RE_FLAG_IBDEV_REGISTERED, &rdev->flags)) {
		if (!test_bit(BNXT_RE_FLAG_PER_PORT_DEBUG_INFO, &rdev->flags)) {
			bnxt_re_debugfs_add_port(rdev);
			set_bit(BNXT_RE_FLAG_PER_PORT_DEBUG_INFO, &rdev->flags);
			dev_info(rdev_to_dev(rdev), "Device registered successfully");
		}
	}

	if (rdev->binfo) {
#ifndef HAVE_NETDEV_BONDING_INFO
			netdev_binfo = kzalloc(sizeof(*netdev_binfo),
					       GFP_KERNEL);
		/* TODO Revist to  avoid holding rtnl_lock */
		/* rtnl lock required to synchronize with netdev events */
		if (rtnl_trylock()) {
			if (!bnxt_re_get_bonding_info(rdev->binfo,
						      rdev->netdev,
						      netdev_binfo)) {
				rtnl_unlock();
				goto free_netdev;
			}
			if (!bnxt_re_bond_update_reqd
					(netdev_binfo, rdev->binfo,
					 rdev, rdev->netdev)) {
				rtnl_unlock();
				goto free_netdev;
			}
			dev_info(rdev_to_dev(rdev),
					"Updating lag device from worker");
			bnxt_re_update_fw_lag_info(rdev->binfo, rdev, true);
			rtnl_unlock();
		}
free_netdev:
			kfree(netdev_binfo);
#else

		active_port_map = bnxt_re_get_bond_link_status(rdev->binfo);
		if (rdev->binfo->active_port_map != active_port_map) {
			rdev->binfo->active_port_map = active_port_map;
			dev_info(rdev_to_dev(rdev),
				 "Updating lag device from worker active_port_map = 0x%x",
				 rdev->binfo->active_port_map);
			bnxt_re_update_fw_lag_info(rdev->binfo, rdev, true);
		}
#endif

	}

	if (!rdev->stats.stats_query_sec)
		goto resched;

	if (test_bit(BNXT_RE_FLAG_ISSUE_CFA_FLOW_STATS, &rdev->flags) &&
	    (rdev->is_virtfn ||
	    !_is_ext_stats_supported(rdev->dev_attr->dev_cap_flags))) {
		if (!(rdev->stats.stats_query_counter++ %
		      rdev->stats.stats_query_sec)) {
			rc = bnxt_re_get_qos_stats(rdev);
			if (rc)
				clear_bit(BNXT_RE_FLAG_ISSUE_CFA_FLOW_STATS,
					  &rdev->flags);
			}
	}

resched:
	schedule_delayed_work(&rdev->worker, msecs_to_jiffies(1000));
}

static void bnxt_re_ib_uninit(struct bnxt_re_dev *rdev)
{
	if (test_and_clear_bit(BNXT_RE_FLAG_IBDEV_REGISTERED, &rdev->flags)) {
		/*
		 * Issue IB_EVENT_PORT_ERR as IB_EVENT_PORT_ACTIVE
		 * was issued in init
		 */
		bnxt_re_dispatch_event(&rdev->ibdev, NULL, 1,
				       IB_EVENT_PORT_ERR);
		bnxt_re_sysfs_destroy_file(rdev);
		/* Cleanup ib dev */
		ib_unregister_device(&rdev->ibdev);
	}
}

static void bnxt_re_dev_uninit(struct bnxt_re_dev *rdev, u8 op_type)
{
	struct bnxt_qplib_dpi *kdpi;
	int rc;

	if (test_and_clear_bit(BNXT_RE_FLAG_DEV_LIST_INITIALIZED,
			       &rdev->flags)) {
		mutex_lock(&bnxt_re_dev_lock);
		list_del_rcu(&rdev->list);
		adapter_count--;
		mutex_unlock(&bnxt_re_dev_lock);
		dev_dbg(rdev_to_dev(rdev), "Device removed (adapter count %d)",
			adapter_count);
	}

	bnxt_re_uninit_resolve_wq(rdev);
	bnxt_re_uninit_dcb_wq(rdev);

	if (test_and_clear_bit(BNXT_RE_FLAG_WORKER_REG, &rdev->flags))
		cancel_delayed_work_sync(&rdev->worker);

	if (test_and_clear_bit(BNXT_RE_FLAG_PER_PORT_DEBUG_INFO, &rdev->flags))
		bnxt_re_debugfs_rem_port(rdev);

	/* Wait for ULPs to release references */
	while (atomic_read(&rdev->stats.rsors.cq_count))
		usleep_range(500, 1000);

	kdpi = &rdev->dpi_privileged;
	if (kdpi->umdbr) { /* kernel DPI was allocated with success */
		bnxt_qplib_dealloc_dpi(&rdev->qplib_res, kdpi);
		/*
		 * Driver just need to know no command had failed
		 * during driver load sequence and below command is
		 * required indeed. Piggybacking dpi allocation status.
		 */
		if (rdev->binfo) {
			rc = bnxt_qplib_set_link_aggr_mode(&rdev->qplib_res, 0,
							   0, 0, false, 0);
			if (rc)
				dev_err(rdev_to_dev(rdev),
					"Failed to disable link aggr mode = #%x",
					rc);
		}
	}
	bnxt_re_clean_nqs(rdev);

	if (test_and_clear_bit(BNXT_RE_FLAG_TBLS_ALLOCINIT, &rdev->flags))
		bnxt_re_free_tbls(rdev);
	if (test_and_clear_bit(BNXT_RE_FLAG_RCFW_CHANNEL_INIT, &rdev->flags)) {
		rc = bnxt_qplib_deinit_rcfw(&rdev->rcfw);
		if (rc)
			dev_warn(rdev_to_dev(rdev),
				 "Failed to deinitialize fw, rc = 0x%x", rc);
	}

	bnxt_re_put_stats_ctx(rdev);

	if (test_and_clear_bit(BNXT_RE_FLAG_ALLOC_CTX, &rdev->flags))
		bnxt_qplib_free_hwctx(&rdev->qplib_res);

	if (test_and_clear_bit(BNXT_RE_FLAG_RCFW_CHANNEL_EN, &rdev->flags))
		bnxt_qplib_disable_rcfw_channel(&rdev->rcfw);

	if (test_and_clear_bit(BNXT_RE_FLAG_NET_RING_ALLOC, &rdev->flags))
		bnxt_re_net_ring_free(rdev, rdev->rcfw.creq.ring_id);

	if (test_and_clear_bit(BNXT_RE_FLAG_ALLOC_RCFW, &rdev->flags))
		bnxt_qplib_free_rcfw_channel(&rdev->qplib_res);

	if (test_and_clear_bit(BNXT_RE_FLAG_GOT_MSIX, &rdev->flags)) {
		rc = bnxt_re_free_msix(rdev);
		if (rc)
			dev_warn(rdev_to_dev(rdev),
				 "free_msix failed with rc = 0x%x", rc);
	}

	bnxt_re_destroy_chip_ctx(rdev);

	if (op_type != BNXT_RE_PRE_RECOVERY_REMOVE) {
		if (test_and_clear_bit(BNXT_RE_FLAG_NETDEV_REGISTERED,
				       &rdev->flags)) {
			rc = bnxt_re_unregister_netdev(rdev);
			if (rc)
				dev_warn(rdev_to_dev(rdev),
					 "Unregister netdev failed with rc = 0x%x",
					 rc);
		}
	}
}

static int bnxt_re_dev_init(struct bnxt_re_dev *rdev, u8 op_type, u8 wqe_mode)
{
	struct bnxt_re_ring_attr rattr = {};
	struct bnxt_qplib_creq_ctx *creq;
	int vec, offset;
	int rc = 0;
#ifdef DISABLE_TIM_BLOCK
	int val;
#endif

	if (op_type != BNXT_RE_POST_RECOVERY_INIT) {
		/* Registered a new RoCE device instance to netdev */
		rc = bnxt_re_register_netdev(rdev);
		if (rc) {
			dev_err(rdev_to_dev(rdev),
				"Register driver failed with rc = 0x%x", rc);
			rc = -EINVAL;
			return rc;
		}
	}
	set_bit(BNXT_RE_FLAG_NETDEV_REGISTERED, &rdev->flags);

	rc = bnxt_re_setup_chip_ctx(rdev, wqe_mode);
	if (rc) {
		dev_err(rdev_to_dev(rdev), "Failed to get chip context\n");
		rc = -EINVAL;
		goto fail;
	}

	/* Check whether VF or PF */
	bnxt_re_get_sriov_func_type(rdev);

	rc = bnxt_re_request_msix(rdev);
	if (rc) {
		dev_err(rdev_to_dev(rdev),
			"Requesting MSI-X vectors failed with rc = 0x%x", rc);
		rc = -EINVAL;
		goto fail;
	}
	set_bit(BNXT_RE_FLAG_GOT_MSIX, &rdev->flags);

	/* Establish RCFW Communication Channel to initialize the context
	   memory for the function and all child VFs */
	rc = bnxt_qplib_alloc_rcfw_channel(&rdev->qplib_res);
	if (rc) {
		dev_err(rdev_to_dev(rdev),
			"Failed to alloc mem for rcfw, rc = %#x\n", rc);
		goto fail;
	}
	set_bit(BNXT_RE_FLAG_ALLOC_RCFW, &rdev->flags);

	creq = &rdev->rcfw.creq;
	rattr.dma_arr = creq->hwq.pbl[PBL_LVL_0].pg_map_arr;
	rattr.pages = creq->hwq.pbl[creq->hwq.level].pg_count;
	rattr.type = bnxt_re_get_rtype(rdev);
	rattr.mode = RING_ALLOC_REQ_INT_MODE_MSIX;
	rattr.depth = BNXT_QPLIB_CREQE_MAX_CNT - 1;
	rattr.lrid = rdev->nqr.msix_entries[BNXT_RE_AEQ_IDX].ring_idx;
	rc = bnxt_re_net_ring_alloc(rdev, &rattr, &creq->ring_id);
	if (rc) {
		creq->ring_id = 0xffff;
		dev_err(rdev_to_dev(rdev),
			"Failed to allocate CREQ fw id with rc = 0x%x", rc);
		goto fail;
	}
	set_bit(BNXT_RE_FLAG_NET_RING_ALLOC, &rdev->flags);

	vec = rdev->nqr.msix_entries[BNXT_RE_AEQ_IDX].vector;
	offset = rdev->nqr.msix_entries[BNXT_RE_AEQ_IDX].db_offset;
	rc = bnxt_qplib_enable_rcfw_channel(&rdev->rcfw, vec, offset,
				rdev->is_virtfn, &bnxt_re_aeq_handler);
	if (rc) {
		dev_err(rdev_to_dev(rdev),
			"Failed to enable RCFW channel with rc = 0x%x", rc);
		goto fail;
	}
	set_bit(BNXT_RE_FLAG_RCFW_CHANNEL_EN, &rdev->flags);

	rc = bnxt_re_update_dev_attr(rdev);
	if (rc)
		goto fail;
	bnxt_re_set_resource_limits(rdev);
	if (!rdev->is_virtfn && !_is_chip_gen_p5(rdev->chip_ctx)) {
		rc = bnxt_qplib_alloc_hwctx(&rdev->qplib_res);
		if (rc) {
			dev_err(rdev_to_dev(rdev),
				"Failed to alloc hw contexts, rc = 0x%x", rc);
			goto fail;
		}
		set_bit(BNXT_RE_FLAG_ALLOC_CTX, &rdev->flags);
	}

	rc = bnxt_re_get_stats_ctx(rdev);
	if (rc)
		goto fail;

	rc = bnxt_qplib_init_rcfw(&rdev->rcfw, rdev->is_virtfn);
	if (rc) {
		dev_err(rdev_to_dev(rdev),
			"Failed to initialize fw with rc = 0x%x", rc);
		goto fail;
	}
	set_bit(BNXT_RE_FLAG_RCFW_CHANNEL_INIT, &rdev->flags);

	/* Based resource count on the 'new' device caps */
	rc = bnxt_re_update_dev_attr(rdev);
	if (rc)
		goto fail;
	rc = bnxt_re_alloc_init_tbls(rdev);
	if (rc) {
		dev_err(rdev_to_dev(rdev), "tbls alloc-init failed rc = %#x",
			rc);
		goto fail;
	}

	rc = bnxt_re_setup_nqs(rdev);
	if (rc) {
		dev_err(rdev_to_dev(rdev), "NQs alloc-init failed rc = %#x\n",
			rc);
		goto fail;
	}

	rc = bnxt_qplib_alloc_dpi(&rdev->qplib_res, &rdev->dpi_privileged,
				  rdev, BNXT_QPLIB_DPI_TYPE_KERNEL);
	if (rc)
		goto fail;

	if (!rdev->is_virtfn) {
		struct bnxt *bp = netdev_priv(rdev->netdev);
		struct bnxt_pf_info *pf = &bp->pf;
		struct bnxt_re_tc_rec *tc_rec;

		tc_rec = &rdev->tc_rec[0];
		rc =  bnxt_re_query_hwrm_qportcfg(rdev, tc_rec, 0xFFFF);
		if (rc) {
			dev_err(rdev_to_dev(rdev),
				"Failed to query port config rc:%d", rc);
			return rc;
		}
		/* Query f/w defaults of CC params */
		rc = bnxt_qplib_query_cc_param(&rdev->qplib_res, &rdev->cc_param);
		if (rc)
			dev_warn(rdev_to_dev(rdev),
				 "Failed to query CC defaults\n");
		rdev->num_vfs = pf->active_vfs;
		if (rdev->num_vfs) {
			bnxt_re_set_resource_limits(rdev);
			bnxt_qplib_set_func_resources(&rdev->qplib_res);
		}
	}
	INIT_DELAYED_WORK(&rdev->worker, bnxt_re_worker);
	set_bit(BNXT_RE_FLAG_WORKER_REG, &rdev->flags);
	schedule_delayed_work(&rdev->worker, msecs_to_jiffies(1000));

	bnxt_re_init_dcb_wq(rdev);
	bnxt_re_init_resolve_wq(rdev);
	mutex_lock(&bnxt_re_dev_lock);
	list_add_tail_rcu(&rdev->list, &bnxt_re_dev_list);
	/* Added to the list, not in progress anymore */
	gadd_dev_inprogress--;
	set_bit(BNXT_RE_FLAG_DEV_LIST_INITIALIZED, &rdev->flags);
	adapter_count++;
	mutex_unlock(&bnxt_re_dev_lock);
	dev_dbg(rdev_to_dev(rdev), "Device added (adapter count %d)",
		adapter_count);

	return rc;
fail:
	bnxt_re_dev_uninit(rdev, BNXT_RE_COMPLETE_REMOVE);

	return rc;
}

int bnxt_re_ib_init(struct bnxt_re_dev *rdev)
{
	int rc = 0;

	/* Register ib dev */
	rc = bnxt_re_register_ib(rdev);
	if (rc) {
		dev_err(rdev_to_dev(rdev),
			"Register IB failed with rc = 0x%x", rc);
		goto fail;
	}
	if (bnxt_re_sysfs_create_file(rdev)) {
		bnxt_re_stopqps_and_ib_uninit(rdev);
		goto fail;
	}

	set_bit(BNXT_RE_FLAG_IBDEV_REGISTERED, &rdev->flags);
	set_bit(BNXT_RE_FLAG_ISSUE_ROCE_STATS, &rdev->flags);
	set_bit(BNXT_RE_FLAG_ISSUE_CFA_FLOW_STATS, &rdev->flags);
	bnxt_re_dispatch_event(&rdev->ibdev, NULL, 1, IB_EVENT_PORT_ACTIVE);
	bnxt_re_dispatch_event(&rdev->ibdev, NULL, 1, IB_EVENT_GID_CHANGE);

	dev_dbg(rdev_to_dev(rdev), "rdev->mem_usage = %llu\n",
		rdev->qplib_res.mem_usage);
	return rc;
fail:
	/* Grab rtnl lock before calling bnxt_re_dev_uninit() */
	rtnl_lock();
	bnxt_re_dev_uninit(rdev, BNXT_RE_COMPLETE_REMOVE);
	rtnl_unlock();
	return rc;
}

static void bnxt_re_dev_unreg(struct bnxt_re_dev *rdev)
{
	struct bnxt_en_dev *en_dev = rdev->en_dev;

	bnxt_re_dev_dealloc(rdev);
	bnxt_re_dev_unprobe(en_dev);
}

static int bnxt_re_check_bond_in_vf_parent(struct net_device *netdev)
{
	struct bnxt *bp = netdev_priv(netdev);
	struct bnxt *s1_bp, *s2_bp;
	struct bnxt_re_dev *rdev;
	struct pci_dev *physfn;

	physfn = pci_physfn(bp->pdev);
	rcu_read_lock();
	list_for_each_entry_rcu(rdev, &bnxt_re_dev_list, list) {
		if (rdev->binfo &&  bp->pdev->is_virtfn) {
			s1_bp = netdev_priv(rdev->binfo->slave1);
			s2_bp = netdev_priv(rdev->binfo->slave2);
			if ((s1_bp->pdev == physfn) ||
			    (s2_bp->pdev == physfn)) {
				rcu_read_unlock();
				return 1;
			}
		}
	}
	rcu_read_unlock();
	return 0;
}

static int bnxt_re_dev_reg(struct bnxt_re_dev **rdev, struct net_device *netdev)
{
	struct bnxt_re_bond_info *binfo;
	struct bnxt_en_dev *en_dev;
	struct net_device *realdev;
	int rc = 0;

	realdev = netdev;
	binfo = bnxt_re_get_bond_info(netdev);
	if (binfo)
		realdev = binfo->slave1;
	dev_dbg(NULL, "%s: realdev = %p netdev = %p\n", __func__,
		realdev, netdev);

	if (!bnxt_re_check_l2_compatibility(realdev)) {
		dev_dbg(NULL, "%s: netdev %p doesn't support bnxt_re",
			 ROCE_DRV_MODULE_NAME, realdev);
		rc = -EINVAL;
		goto exit;
	}

	en_dev = bnxt_re_dev_probe(realdev);
	if (IS_ERR(en_dev)) {
		if (en_dev != ERR_PTR(-ENODEV))
			dev_info(NULL, "%s: Device probe failed",
				 ROCE_DRV_MODULE_NAME);
		return PTR_ERR(en_dev);
	}

	if (bnxt_re_check_bond_in_vf_parent(realdev)) {
		dev_err(NULL, "RoCE disabled, LAG is configured on PFs\n");
		bnxt_re_dev_unprobe(en_dev);
		return -EINVAL;
	}

	/*
	 * Note:
	 * The first argument to bnxt_re_dev_alloc() is 'netdev' and
	 * not 'realdev', since in the case of bonding we want to
	 * register the bonded virtual netdev (master) to the ib stack.
	 * And 'en_dev' (for L2/PCI communication) is the first slave
	 * device (PF0 on the card).
	 * In the case of a regular netdev, both netdev and the en_dev
	 * correspond to the same device.
	 */
	*rdev = bnxt_re_dev_alloc(netdev, en_dev);
	if (!*rdev) {
		dev_err(NULL, "%s: netdev %p not handled",
			ROCE_DRV_MODULE_NAME, netdev);
		rc = -ENOMEM;
		bnxt_re_dev_unprobe(en_dev);
		goto exit;
	}
	bnxt_re_hold(*rdev);
exit:
	return rc;
}

static void bnxt_re_update_speed_for_bond(struct bnxt_re_dev *rdev)
{
	if (rdev->binfo->aggr_mode !=
	    CMDQ_SET_LINK_AGGR_MODE_AGGR_MODE_ACTIVE_BACKUP)
		rdev->espeed *= 2;
}

void bnxt_re_get_link_speed(struct bnxt_re_dev *rdev)
{
#ifdef HAVE_ETHTOOL_GLINKSETTINGS_25G
	struct ethtool_link_ksettings lksettings;
#else
	struct ethtool_cmd ecmd;
#endif
	/* Using physical netdev for getting speed
	 * in case of bond devices.
	 * No change for normal interface.
	 */
	struct net_device *netdev = rdev->en_dev->net;

#ifdef HAVE_ETHTOOL_GLINKSETTINGS_25G
	if (netdev->ethtool_ops && netdev->ethtool_ops->get_link_ksettings) {
		memset(&lksettings, 0, sizeof(lksettings));
		netdev->ethtool_ops->get_link_ksettings(netdev, &lksettings);
		rdev->espeed = lksettings.base.speed;
	}
#else
	if (netdev->ethtool_ops && netdev->ethtool_ops->get_settings) {
		memset(&ecmd, 0, sizeof(ecmd));
		netdev->ethtool_ops->get_settings(netdev, &ecmd);
		rdev->espeed = ecmd.speed;
	}
#endif
	/* Store link speed as sl_espeed. espeed can change for bond device */
	rdev->sl_espeed = rdev->espeed;
	if (rdev->binfo)
		bnxt_re_update_speed_for_bond(rdev);
}

static bool bnxt_re_bond_update_reqd(struct netdev_bonding_info *netdev_binfo,
				     struct bnxt_re_bond_info *binfo,
				     struct bnxt_re_dev *rdev,
				     struct net_device *sl_netdev)
{
	int rc = 0;
	struct bnxt_re_bond_info tmp_binfo;

	memcpy(&tmp_binfo, binfo, sizeof(*binfo));

	rc = bnxt_re_get_port_map(netdev_binfo, binfo, sl_netdev);
	if (rc) {
		dev_dbg(rdev_to_dev(binfo->rdev),
			"%s: Error in receiving port state rdev = %p",
			__func__, binfo->rdev);
		return false;
	}

	if (binfo->aggr_mode == tmp_binfo.aggr_mode &&
	    binfo->active_port_map == tmp_binfo.active_port_map) {
		dev_dbg(rdev_to_dev(binfo->rdev),
			"%s: No need to update rdev=%p active_port_map=%#x",
			__func__, binfo->rdev, binfo->active_port_map);
		return false;
	}

	return true;
}

static int bnxt_re_update_fw_lag_info(struct bnxt_re_bond_info *binfo,
			       struct bnxt_re_dev *rdev,
			       bool bond_mode)
{
	struct bnxt_qplib_ctx *hctx;
	int rc = 0;

	dev_dbg(rdev_to_dev(binfo->rdev), "%s: port_map for rdev=%p bond=%#x",
		__func__, binfo->rdev, binfo->active_port_map);

	/* Send LAG info to BONO */
	hctx = rdev->qplib_res.hctx;
	rc = bnxt_qplib_set_link_aggr_mode(&binfo->rdev->qplib_res,
					   binfo->aggr_mode,
					   BNXT_RE_MEMBER_PORT_MAP,
					   binfo->active_port_map,
					   bond_mode, hctx->stats2.fw_id);

	if (rc)
		dev_err(rdev_to_dev(binfo->rdev),
			"%s: setting link aggr mode rc = %d\n", __func__, rc);

	dev_info(rdev_to_dev(binfo->rdev),
		"binfo->aggr_mode = %d binfo->active_port_map = 0x%x\n",
		binfo->aggr_mode, binfo->active_port_map);

	return rc;
}

void bnxt_re_stopqps_and_ib_uninit(struct bnxt_re_dev *rdev)
{
	dev_dbg(rdev_to_dev(rdev), "%s: Stopping QPs, IB uninit on rdev: %p\n",
		__func__, rdev);
	bnxt_re_stop_all_nonqp1_nonshadow_qps(rdev);
	bnxt_re_ib_uninit(rdev);
}

void bnxt_re_remove_device(struct bnxt_re_dev *rdev, u8 op_type)
{
	struct bnxt_qplib_cmdq_ctx *cmdq;
	struct bnxt_qplib_rcfw *rcfw;

	rcfw = &rdev->rcfw;
	cmdq = &rcfw->cmdq;
	if (test_bit(FIRMWARE_TIMED_OUT, &cmdq->flags))
		set_bit(BNXT_RE_FLAG_ERR_DEVICE_DETACHED, &rdev->flags);

	dev_dbg(rdev_to_dev(rdev), "%s: Removing rdev: %p\n",
		__func__, rdev);
	if (enable_fc && !rdev->is_virtfn)
		bnxt_re_clear_dcbx_cc_param(rdev);
	bnxt_re_dev_uninit(rdev, op_type);
	bnxt_re_put_refs(rdev);
	bnxt_re_dev_unreg(rdev);
}

int bnxt_re_add_device(struct bnxt_re_dev **rdev,
		       struct net_device *netdev,
		       struct bnxt_re_bond_info *info,
		       u8 qp_mode, u8 op_type, u8 wqe_mode)
{
	int rc = 0;

	mutex_lock(&bnxt_re_dev_lock);
	/* Check if driver already in mod exit */
	if (gmod_exit) {
		mutex_unlock(&bnxt_re_dev_lock);
		return -ENODEV;
	}
	/* Add device in progress */
	gadd_dev_inprogress++;
	mutex_unlock(&bnxt_re_dev_lock);

	rc = bnxt_re_dev_reg(rdev, netdev);
	if (rc) {
		dev_dbg(NULL, "Failed to create add device for netdev %p\n",
			netdev);
		/*
		 * For BNXT_RE_POST_RECOVERY_INIT special case
		 * called from bnxt_re_start, the work is
		 * complete only after, bnxt_re_start completes
		 * bnxt_unregister_device in case of failure.
		 * So bnxt_re_start will decrement gadd_dev_inprogress
		 * in case of failure.
		 */
		if (op_type != BNXT_RE_POST_RECOVERY_INIT) {
			mutex_lock(&bnxt_re_dev_lock);
			gadd_dev_inprogress--;
			mutex_unlock(&bnxt_re_dev_lock);
		}
		return rc;
	}
	bnxt_re_get_refs(*rdev);

	/* Set Bonding info for handling bond devices */
	(*rdev)->binfo = info;
	/* Inherit gsi_qp_mode only if info is valid
	 * and qp_mode specified is invalid
	 */
	(*rdev)->gsi_ctx.gsi_qp_mode = (info && !qp_mode) ?
					info->gsi_qp_mode : qp_mode;
	wqe_mode = (info && wqe_mode == BNXT_QPLIB_WQE_MODE_INVALID) ?
		    info->wqe_mode : wqe_mode;
	rc = bnxt_re_dev_init(*rdev, op_type, wqe_mode);
	if (rc) {
		(*rdev)->binfo = NULL;
		bnxt_re_put_refs(*rdev);
		bnxt_re_dev_unreg(*rdev);
		*rdev = NULL;
		/*
		 * For BNXT_RE_POST_RECOVERY_INIT special case
		 * called from bnxt_re_start, the work is
		 * complete only after, bnxt_re_start completes
		 * bnxt_unregister_device in case of failure.
		 * So bnxt_re_start will decrement gadd_dev_inprogress
		 * in case of failure.
		 */
		if (op_type != BNXT_RE_POST_RECOVERY_INIT) {
			mutex_lock(&bnxt_re_dev_lock);
			gadd_dev_inprogress--;
			mutex_unlock(&bnxt_re_dev_lock);
		}
	}
	return rc;
}

struct bnxt_re_dev *bnxt_re_get_peer_pf(struct bnxt_re_dev *rdev)
{
	struct pci_dev *pdev_in = rdev->en_dev->pdev;
	int tmp_bus_num, bus_num = pdev_in->bus->number;
	int tmp_dev_num, dev_num = PCI_SLOT(pdev_in->devfn);
	int tmp_func_num, func_num = PCI_FUNC(pdev_in->devfn);
	struct bnxt_re_dev *tmp_rdev;

	rcu_read_lock();
	list_for_each_entry_rcu(tmp_rdev, &bnxt_re_dev_list, list) {
		tmp_bus_num = tmp_rdev->en_dev->pdev->bus->number;
		tmp_dev_num = PCI_SLOT(tmp_rdev->en_dev->pdev->devfn);
		tmp_func_num = PCI_FUNC(tmp_rdev->en_dev->pdev->devfn);

		if (bus_num == tmp_bus_num && dev_num == tmp_dev_num &&
		    func_num != tmp_func_num) {
			rcu_read_unlock();
			return tmp_rdev;
		}
	}
	rcu_read_unlock();
	return NULL;
}


static struct bnxt_re_bond_info *bnxt_re_alloc_lag(struct bnxt_re_dev *rdev,
						   u8 gsi_mode, u8 wqe_mode)
{
	struct bnxt_re_dev *rdev_peer;
	struct bnxt_re_bond_info *info = NULL;
	struct net_device *master;
	int rc = 0;

	master = netdev_master_upper_dev_get(rdev->netdev);
	if (!master)
		return NULL;

	rdev_peer = bnxt_re_get_peer_pf(rdev);
	if (!rdev_peer)
		return NULL;

	info = kzalloc(sizeof(*info), GFP_KERNEL);
	if (!info)
		return NULL;

	info->master = master;
	if (BNXT_RE_IS_PORT0(rdev)) {
		info->slave1 = rdev->netdev;
		info->slave2 = rdev_peer->netdev;
		info->pdev1 = rdev->en_dev->pdev;
		info->pdev2 = rdev_peer->en_dev->pdev;
		info->gsi_qp_mode = rdev->gsi_ctx.gsi_qp_mode;
		info->wqe_mode = rdev->chip_ctx->modes.wqe_mode;
	} else {
		info->slave2 = rdev->netdev;
		info->slave1 = rdev_peer->netdev;
		info->pdev2 = rdev->en_dev->pdev;
		info->pdev1 = rdev_peer->en_dev->pdev;
		info->gsi_qp_mode = rdev_peer->gsi_ctx.gsi_qp_mode;
		info->wqe_mode = rdev->chip_ctx->modes.wqe_mode;
	}

	bnxt_re_stopqps_and_ib_uninit(rdev);
	dev_dbg(rdev_to_dev(rdev), "Removing device 1\n");
	bnxt_re_remove_device(rdev, BNXT_RE_COMPLETE_REMOVE);
	bnxt_re_stopqps_and_ib_uninit(rdev_peer);
	dev_dbg(rdev_to_dev(rdev_peer), "Removing device 2\n");
	bnxt_re_remove_device(rdev_peer, BNXT_RE_COMPLETE_REMOVE);

	mutex_lock(&bnxt_re_dev_lock);
	list_add_tail_rcu(&info->list, &bnxt_re_bond_list);
	mutex_unlock(&bnxt_re_dev_lock);
	rc = bnxt_re_add_device(&info->rdev, info->master, info,
				gsi_mode, BNXT_RE_COMPLETE_INIT, wqe_mode);
	dev_dbg(rdev_to_dev(info->rdev), "Added bond device rdev %p\n",
		info->rdev);
	if (rc) {
		list_del_rcu(&info->list);
		return ERR_PTR(-EIO);
	}
	/* Query link speed.. in rtnl context */
	bnxt_re_get_link_speed(info->rdev);

	return info;
}

int bnxt_re_schedule_work(struct bnxt_re_dev *rdev, unsigned long event,
			  struct net_device *vlan_dev,
			  struct netdev_bonding_info *netdev_binfo,
			  struct bnxt_re_bond_info *binfo,
			  struct net_device *netdev)
{
	struct bnxt_re_work *re_work;

		/* Allocate for the deferred task */
	re_work = kzalloc(sizeof(*re_work), GFP_KERNEL);
	if (!re_work)
		return -ENOMEM;

	re_work->rdev = rdev;
	re_work->event = event;
	re_work->vlan_dev = vlan_dev;
	if (netdev_binfo) {
		memcpy(&re_work->netdev_binfo, netdev_binfo,
		       sizeof(*netdev_binfo));
		re_work->binfo = binfo;
		re_work->netdev = netdev;
	}
	INIT_WORK(&re_work->work, bnxt_re_task);
	if (rdev)
		atomic_inc(&rdev->sched_count);
	else
		re_work->netdev = netdev;
	queue_work(bnxt_re_wq, &re_work->work);

	return 0;
}

static void bnxt_re_create_base_interfaces(struct bnxt_re_bond_info *binfo)
{
	struct bnxt_re_dev *tmp_rdev1 = NULL;
	struct bnxt_re_dev *tmp_rdev2 = NULL;
	int rc = 0;

	rc = bnxt_re_add_device(&tmp_rdev1, binfo->slave1, NULL,
				binfo->gsi_qp_mode, BNXT_RE_COMPLETE_INIT,
				binfo->wqe_mode);
	if (rc) {
		dev_err(rdev_to_dev(tmp_rdev1), "Failed to add the interface 1");
		goto clean_dev;
	}
	dev_dbg(rdev_to_dev(tmp_rdev1), "Added device 1 netdev = %p", binfo->slave1);
	bnxt_re_put(tmp_rdev1);
	/* Query link speed.. Already in rtnl context */
	bnxt_re_get_link_speed(tmp_rdev1);

	rc = bnxt_re_add_device(&tmp_rdev2, binfo->slave2, NULL,
				binfo->gsi_qp_mode, BNXT_RE_COMPLETE_INIT,
				binfo->wqe_mode);
	if (rc) {
		dev_err(rdev_to_dev(tmp_rdev2), "Failed to add the interface 2");
		goto clean_dev;
	}
	dev_dbg(rdev_to_dev(tmp_rdev2), "Added device 2 netdev = %p", binfo->slave2);
	bnxt_re_put(tmp_rdev2);
	/* Query link speed.. Already in rtnl context */
	bnxt_re_get_link_speed(tmp_rdev2);
	dev_dbg(rdev_to_dev(tmp_rdev2), "scheduling for tmp_rdev1 registration");
	bnxt_re_schedule_work(tmp_rdev1, NETDEV_REGISTER, NULL, NULL, NULL,
			      NULL);
	dev_dbg(rdev_to_dev(tmp_rdev2), "scheduling for tmp_rdev2 registration");
	bnxt_re_schedule_work(tmp_rdev2, NETDEV_REGISTER, NULL, NULL, NULL,
			      NULL);

	return;

clean_dev:
	if (tmp_rdev1) {
		bnxt_re_stopqps_and_ib_uninit(tmp_rdev1);
		bnxt_re_remove_device(tmp_rdev1, BNXT_RE_COMPLETE_REMOVE);
	}
	if (tmp_rdev2) {
		bnxt_re_stopqps_and_ib_uninit(tmp_rdev2);
		bnxt_re_remove_device(tmp_rdev2, BNXT_RE_COMPLETE_REMOVE);
	}
}

int bnxt_re_get_slot_pf_count(struct bnxt_re_dev *rdev)
{
	struct pci_dev *pdev_in = rdev->en_dev->pdev;
	int tmp_bus_num, bus_num = pdev_in->bus->number;
	int tmp_dev_num, dev_num = PCI_SLOT(pdev_in->devfn);
	struct bnxt_re_dev *tmp_rdev;
	int pf_cnt = 0;

	rcu_read_lock();
	list_for_each_entry_rcu(tmp_rdev, &bnxt_re_dev_list, list) {
		tmp_bus_num = tmp_rdev->en_dev->pdev->bus->number;
		tmp_dev_num = PCI_SLOT(tmp_rdev->en_dev->pdev->devfn);

		if (bus_num == tmp_bus_num && dev_num == tmp_dev_num)
			pf_cnt++;
	}
	rcu_read_unlock();
	return pf_cnt;
}

int bnxt_re_destroy_lag(struct bnxt_re_dev **rdev)
{
	struct bnxt_re_dev *tmp_rdev = *rdev;
	struct bnxt_re_bond_info bkup_binfo;
	int rc;

	memcpy(&bkup_binfo, tmp_rdev->binfo, sizeof(*(tmp_rdev->binfo)));
	dev_dbg(rdev_to_dev(tmp_rdev), "Destroying lag device\n");
	rc = bnxt_re_update_fw_lag_info(tmp_rdev->binfo, tmp_rdev, false);
	if (rc)
		return rc;
	bnxt_re_stopqps_and_ib_uninit(tmp_rdev);
	bnxt_re_remove_device(tmp_rdev, BNXT_RE_COMPLETE_REMOVE);
	bnxt_re_create_base_interfaces(&bkup_binfo);
	*rdev = NULL;

	return 0;
}

#ifdef HAVE_NETDEV_BONDING_INFO
int bnxt_re_create_lag(ifbond *master, ifslave *slave,
		       struct netdev_bonding_info *netdev_binfo,
		       struct net_device *netdev,
		       struct bnxt_re_dev **rdev,
		       u8 gsi_mode, u8 wqe_mode)
{
	struct bnxt_re_bond_info *tmp_binfo = NULL;
	struct net_device *real_dev;
	struct bnxt_re_dev *tmp_rdev;
	unsigned long event;
	bool do_lag;

	real_dev = rdma_vlan_dev_real_dev(netdev);
	if (!real_dev)
		real_dev = netdev;
	tmp_rdev = bnxt_re_from_netdev(real_dev);
	if (!tmp_rdev)
		return -EINVAL;
	bnxt_re_hold(tmp_rdev);

	if (master && slave) {
		do_lag = bnxt_re_is_lag_allowed(master, slave, tmp_rdev);
		if (!do_lag) {
			bnxt_re_put(tmp_rdev);
			return -EINVAL;
		}
		dev_dbg(rdev_to_dev(tmp_rdev), "%s:do_lag = %d\n",
			__func__, do_lag);
	}

	tmp_binfo = bnxt_re_alloc_lag(tmp_rdev, gsi_mode, wqe_mode);
	if (!tmp_binfo) {
		bnxt_re_put(tmp_rdev);
		return -ENOMEM;
	}
	/*
	 * EIO is returned only if new device creation failed.
	 * This means older rdev is destroyed already.
	 */

	if (PTR_ERR(tmp_binfo) == -EIO) {
		*rdev = NULL;
		return -EIO;
	}

	/* Current rdev already destroyed */
	dev_dbg(rdev_to_dev(tmp_binfo->rdev), "Scheduling for BOND info\n");
	*rdev = tmp_binfo->rdev;
	/* Save netdev_bonding_info for non notifier contexts. */
	memcpy(&tmp_binfo->nbinfo, netdev_binfo, sizeof(*netdev_binfo));

	/*
	 * Schedule for registering with IB device.
	 * Handle it as a NETDEV REGISTER event
	 */

	event = NETDEV_REGISTER;
	bnxt_re_schedule_work(*rdev, event, NULL, netdev_binfo,
			tmp_binfo, real_dev);
	return 0;
}

static void bond_fill_ifbond(struct bonding *bond, struct ifbond *info)
{
	info->bond_mode = BOND_MODE(bond);
	info->miimon = bond->params.miimon;
	info->num_slaves = bond->slave_cnt;
}

static void bond_fill_ifslave(struct slave *slave, struct ifslave *info)
{
	strcpy(info->slave_name, slave->dev->name);
	info->link = slave->link;
	info->state = bond_slave_state(slave);
	info->link_failure_count = slave->link_failure_count;
}

static int __handle_bond_from_task(struct net_device *netdev)
{
	struct bnxt_re_dev *rdev = NULL;
	struct netdev_bonding_info binfo;
	struct bonding *bond;
	struct slave *slave;
	struct list_head *iter;
	bool found = false;
	int rc = -1;

	bond = netdev_priv(netdev);
	bond_for_each_slave(bond, slave, iter) {
		found = true;
		break;
	}

	if (found) {
		rtnl_lock();
		bond_fill_ifslave(slave, &binfo.slave);
		bond_fill_ifbond(slave->bond, &binfo.master);

		rc = bnxt_re_create_lag(&binfo.master, &binfo.slave,
					&binfo, slave->dev, &rdev,
					BNXT_RE_GSI_MODE_INVALID,
					BNXT_QPLIB_WQE_MODE_INVALID);
		rtnl_unlock();
	}
	return rc;
}

#else
static int bnxt_re_update_lag(struct bnxt_re_dev *rdev)
{
	struct netdev_bonding_info *netdev_binfo = NULL;
	int rc = 0;

	if (rdev && rdev->binfo) {
		netdev_binfo = kzalloc(sizeof(*netdev_binfo), GFP_KERNEL);
		if (!netdev_binfo)
			return -ENOMEM;

		if (!bnxt_re_get_bonding_info(rdev->binfo, rdev->netdev,
					      netdev_binfo)) {
			rc = -EACCES;
			goto free_netdev;
		}

		if (!bnxt_re_bond_update_reqd(netdev_binfo, rdev->binfo,
					      rdev, rdev->netdev)) {
			rc = -EEXIST;
			goto free_netdev;
		}
		dev_info(rdev_to_dev(rdev),
			 "Updating lag device from event\n");
		bnxt_re_update_fw_lag_info(rdev->binfo, rdev, true);
free_netdev:
		kfree(netdev_binfo);
	}

	return rc;
}

int bnxt_re_create_lag(struct net_device *netdev,
		       struct bnxt_re_dev **rdev,
		       u8 gsi_mode, u8 wqe_mode)
{
	struct netdev_bonding_info *netdev_binfo = NULL;
	struct bnxt_re_bond_info *tmp_binfo = NULL;
	struct bnxt_re_dev *tmp_rdev;
	struct net_device *real_dev;
	unsigned long event;
	bool do_lag;
	int rc = 0;

	real_dev = rdma_vlan_dev_real_dev(netdev);
	if (!real_dev)
		real_dev = netdev;

	netdev_binfo = kzalloc(sizeof(*netdev_binfo), GFP_KERNEL);
	if (!netdev_binfo) {
		rc = -ENOMEM;
		goto done;
	}

	do_lag = bnxt_re_is_lag_allowed(real_dev, netdev_binfo);
	dev_err(rdev_to_dev(*rdev), "%s:do_lag is %d\n",
		__func__, do_lag);
	if (!do_lag) {
		rc = EAGAIN;
		goto done;
	}

	tmp_rdev = bnxt_re_from_netdev(netdev_binfo->slave1->dev);
	if (!tmp_rdev) {
		rc = -EINVAL;
		goto done;
	}

	/* Pass gsi_mode invalid, we want to inherit from PF0 */
	tmp_binfo = bnxt_re_alloc_lag(tmp_rdev, gsi_mode, wqe_mode);
	if (!tmp_binfo) {
		rc = -ENOMEM;
		goto done;
	}
	/*
	 * EIO is returned only if new device creation failed.
	 * This means older rdev is destroyed already.
	 */

	if (PTR_ERR(tmp_binfo) == -EIO) {
		*rdev = NULL;
		goto done;
	}

	/* Current rdev already destroyed */
	*rdev = tmp_binfo->rdev;
	dev_dbg(rdev_to_dev(tmp_binfo->rdev),
		"Scheduling for BOND info rdev = %p\n", rdev);
	/*
	 * Registering the IB device for the Bond.
	 * Handle it as a NETDEV REGISTER event
	 */
	event = NETDEV_REGISTER;
	bnxt_re_schedule_work(*rdev, event, NULL, netdev_binfo,
			      tmp_binfo, real_dev);
done:
	return rc;
}

static int __handle_bond_from_task(struct net_device *netdev)
{
	struct bnxt_re_dev *rdev = NULL;
	int rc = 0;

	rtnl_lock();
	rc = bnxt_re_create_lag(netdev, &rdev, BNXT_RE_GSI_MODE_INVALID,
				BNXT_QPLIB_WQE_MODE_INVALID);
	rtnl_unlock();

	return rc;
}
#endif

static void bnxt_re_handle_bond_from_task(struct bnxt_re_work *re_work)
{
	static int resched_bond_create_cnt;

	dev_err(rdev_to_dev(re_work->rdev),
		"Handle bond event from task %s\n", __func__);
	if (__handle_bond_from_task(re_work->netdev) &&
	    resched_bond_create_cnt++ < BNXT_RE_BOND_RESCHED_CNT) {
		/* Wait for 1 sec before rescheduling the bond create task */
		msleep(1000);
		bnxt_re_schedule_work(re_work->rdev, re_work->event, NULL, NULL,
				      NULL, re_work->netdev);
	} else {
		/*
		 * Acquire rtnl_lock and reset the flag which indicates that
		 * the delayed task for bond creation is scheduled. This
		 * path is executed either retry count expires or bond device
		 * is created
		 */

		rtnl_lock();
		resched_bond_create_cnt = 0;
		delayed_bond_task_scheduled = false;
		rtnl_unlock();
	}
}

void bnxt_re_clear_dcbx_cc_param(struct bnxt_re_dev *rdev)
{
	struct bnxt_qplib_cc_param *cc_param = &rdev->cc_param;
	struct bnxt_re_tc_rec *tc_rec;

	cc_param->enable = 0;
	cc_param->tos_ecn = 0;


	bnxt_re_clear_cc(rdev);
	tc_rec = &rdev->tc_rec[0];
	bnxt_re_clear_dcb(rdev, rdev->en_dev->net, tc_rec);
	if (rdev->binfo) {
		tc_rec = &rdev->tc_rec[1];
		bnxt_re_clear_dcb(rdev, rdev->binfo->slave2, tc_rec);
	}
	cc_param->alt_tos_dscp = 0;
	cc_param->alt_vlan_pcp = 0;
	cc_param->tos_dscp = 0;
	cc_param->roce_pri = 0;
}

int bnxt_re_init_dcbx_cc_param(struct bnxt_re_dev *rdev)
{
	struct bnxt_qplib_cc_param *cc_param = &rdev->cc_param;
	struct bnxt_re_tc_rec *tc_rec;
	int rc;

	/* Disable priority vlan as the default mode is DSCP based PFC
	 * Set the  default values of dscp and pri values for RoCE and
	 * CNP. Also, set the default parameters for enabling CC
	 */
	cc_param->disable_prio_vlan_tx = 1;

	cc_param->tos_ecn = 0x1;
	cc_param->cc_mode = 0;

	cc_param->alt_tos_dscp = BNXT_RE_DEFAULT_CNP_DSCP;
	cc_param->alt_vlan_pcp = BNXT_RE_DEFAULT_CNP_PRI;
	cc_param->tos_dscp = BNXT_RE_DEFAULT_ROCE_DSCP;
	cc_param->roce_pri = BNXT_RE_DEFAULT_ROCE_PRI;

	tc_rec = &rdev->tc_rec[0];
	rc = bnxt_re_setup_dcb(rdev, rdev->en_dev->net, tc_rec, 0xFFFF);
	if (rc)
		return rc;
	if (rdev->binfo) {
		tc_rec = &rdev->tc_rec[1];
		rc = bnxt_re_setup_dcb(rdev, rdev->binfo->slave2, tc_rec, 2);
		if (rc)
			goto clear_port0;
	}

	/* CC is not enabled on non p5 adapters at 10G speed */
	if (rdev->sl_espeed == SPEED_10000 &&
	    !_is_chip_gen_p5(rdev->chip_ctx))
		return 0;

	rc = bnxt_re_setup_cc(rdev);
	if (rc)
		goto clear_port1;
	return 0;
clear_port1:
	bnxt_re_clear_dcb(rdev, rdev->en_dev->net, &rdev->tc_rec[1]);
clear_port0:
	bnxt_re_clear_dcb(rdev, rdev->en_dev->net, &rdev->tc_rec[0]);
	return rc;
}

/* Handle all deferred netevents tasks */
static void bnxt_re_task(struct work_struct *work)
{
	struct bnxt_re_pre_recovery_snapshot *recovery_item;
	struct bnxt_re_bond_info *binfo = NULL, bkup_binfo;
	struct netdev_bonding_info *netdev_binfo = NULL;
	struct net_device *netdev = NULL;
	const struct bnxt_en_ops *en_ops;
	struct bnxt_re_work *re_work;
	struct bnxt_re_dev *rdev;
	int rc = 0;

	re_work = container_of(work, struct bnxt_re_work, work);
	rdev = re_work->rdev;

	if (!rdev && re_work->netdev) {
		if (netif_is_bond_master(re_work->netdev)) {
			bnxt_re_handle_bond_from_task(re_work);
			kfree(re_work);
			return;
		}
	}
	/*
	 * If the previous rdev is deleted due to bond creation
	 * do not handle the event
	 */
	if (!bnxt_re_is_rdev_valid(rdev)) {
		kfree(re_work);
		return;
	}

	/* Other than NETDEV_REGISTER event, avoid any other event if
	 * the device is not registred with IB stack. This is to avoid
	 * handling any event while the device is added/removed.
	 */
	if (re_work->event != NETDEV_REGISTER && rdev &&
	    !test_bit(BNXT_RE_FLAG_IBDEV_REGISTERED, &rdev->flags)) {
		dev_dbg(rdev_to_dev(rdev), "%s: Ignoring netdev event 0x%lx",
			__func__, re_work->event);
		goto done;
	}

	/* Extra check to silence coverity. We shouldn't handle any event
	 * when rdev is NULL.
	 */
	if (!rdev)
		goto exit;

	dev_dbg(rdev_to_dev(rdev), "Scheduled work for event 0x%lx",
		re_work->event);
	switch (re_work->event) {
	case NETDEV_REGISTER:

			rc = bnxt_re_ib_init(rdev);
			if (!rc && re_work->binfo) {
				netdev_binfo = &re_work->netdev_binfo;
				binfo = re_work->binfo;
				rc = bnxt_re_get_port_map(netdev_binfo, binfo,
							  re_work->netdev);
				if (rc) {
					dev_dbg(rdev_to_dev(rdev),
						"%s: Error in receiving port state rdev = %p",
						__func__, re_work->binfo->rdev);
					break;
				}
				bnxt_re_update_fw_lag_info(binfo, rdev, true);
			}
			if (rc) {
				dev_err(rdev_to_dev(rdev),
					"Failed to register rc = 0x%x", rc);
					bnxt_re_put_refs(rdev);
					bnxt_re_dev_unreg(rdev);
					break;
			}
			rc = bnxt_re_get_device_stats(rdev);
			if (rc)
				dev_err(rdev_to_dev(rdev),
					"Failed initial device stat query");
			rtnl_lock();
			rdev->event_bitmap |=
			(1 << ASYNC_EVENT_CMPL_EVENT_ID_DCB_CONFIG_CHANGE);
			en_ops = rdev->en_dev->en_ops;
			if (is_qport_service_type_supported(rdev) &&
			    en_ops->bnxt_register_fw_async_events
			    (rdev->en_dev, BNXT_ROCE_ULP,
			     &rdev->event_bitmap,
			     ASYNC_EVENT_CMPL_EVENT_ID_DCB_CONFIG_CHANGE))
				dev_err(rdev_to_dev(rdev),
					"Failed to reg Async event");
			if (enable_fc && !rdev->is_virtfn) {
				if (bnxt_re_init_dcbx_cc_param(rdev))
					dev_err(rdev_to_dev(rdev),
						"Fail to initialize Flow control");
			}
			rtnl_unlock();
			break;
	case NETDEV_UP:
			if (re_work->vlan_dev) {
#ifndef STACK_MANAGES_GID
				bnxt_qplib_netdev_add_gid(
						&rdev->qplib_res.sgid_tbl,
						re_work->vlan_dev);
				bnxt_re_dispatch_event(&rdev->ibdev, NULL, 1,
						       IB_EVENT_GID_CHANGE);
#endif
			} else {
				bnxt_re_dispatch_event(&rdev->ibdev, NULL, 1,
						       IB_EVENT_PORT_ACTIVE);
			}
			break;

	case NETDEV_DOWN:
			bnxt_re_dispatch_event(&rdev->ibdev, NULL, 1,
					       IB_EVENT_PORT_ERR);
			bnxt_re_stop_all_nonqp1_nonshadow_qps(rdev);
			break;

	case NETDEV_CHANGE:
			if (bnxt_re_get_link_state(rdev) == IB_PORT_DOWN) {
				bnxt_re_dispatch_event(&rdev->ibdev, NULL, 1,
						       IB_EVENT_PORT_ERR);
				bnxt_re_stop_all_nonqp1_nonshadow_qps(rdev);
			}
			else if (bnxt_re_get_link_state(rdev) ==
				 IB_PORT_ACTIVE) {
				bnxt_re_dispatch_event(&rdev->ibdev, NULL, 1,
						       IB_EVENT_PORT_ACTIVE);
			}

			/* temporarily disable the check for SR2 */
			if (!bnxt_qplib_query_cc_param(&rdev->qplib_res,
						       &rdev->cc_param) &&
			    !_is_chip_sr2(rdev->chip_ctx)) {
				/*
				 *  Disable CC for 10G speed
				 * for non p5 devices
				 */
				if (rdev->sl_espeed == SPEED_10000 &&
				    !_is_chip_gen_p5(rdev->chip_ctx)) {
					if (rdev->cc_param.enable)
						bnxt_re_clear_cc(rdev);
				} else {
					if (!rdev->cc_param.enable &&
					    rdev->cc_param.admin_enable)
						bnxt_re_setup_cc(rdev);
				}
			}
/* Only required for older kernels */
#ifndef HAVE_NETDEV_BONDING_INFO
			/* Link state changes needs to be updated to the FW
			 * during link change event (active -active bond)
			 */
			bnxt_re_update_lag(rdev);
#endif /* HAVE_NETDEV_BONDING_INFO */
			bnxt_re_put(rdev);
			break;
	case NETDEV_UNREGISTER:
			netdev = rdev->netdev;
			/* rtnl_lock not held */
			bnxt_re_stopqps_and_ib_uninit(rdev);
			mutex_lock(&bnxt_re_dev_lock);
			list_for_each_entry(recovery_item,
					    &bnxt_re_recovery_snapshot_list,
					    list) {
				if (recovery_item->netdev == netdev) {
					recovery_item->ib_uninit_done = true;
					wake_up(&recovery_item->waitq);
					mutex_unlock(&bnxt_re_dev_lock);
					goto done;
				}
			}
			mutex_unlock(&bnxt_re_dev_lock);

			rtnl_lock();
			if (rdev->binfo) {
				binfo = rdev->binfo;
				memcpy(&bkup_binfo, rdev->binfo, sizeof(*(rdev->binfo)));
			}
			bnxt_re_remove_device(rdev, BNXT_RE_COMPLETE_REMOVE);
			if (binfo)
				bnxt_re_create_base_interfaces(&bkup_binfo);
			rtnl_unlock();

			break;
	default:
			break;
	}
done:
	if (rdev) {
		/* memory barrier to guarantee task completion
		 * before decrementing sched count
		 */
		smp_mb__before_atomic();
		atomic_dec(&rdev->sched_count);
	}
exit:
	kfree(re_work);
}


/*
    "Notifier chain callback can be invoked for the same chain from
    different CPUs at the same time".

    For cases when the netdev is already present, our call to the
    register_netdevice_notifier() will actually get the rtnl_lock()
    before sending NETDEV_REGISTER and (if up) NETDEV_UP
    events.

    But for cases when the netdev is not already present, the notifier
    chain is subjected to be invoked from different CPUs simultaneously.

    This is protected by the netdev_mutex.
*/
static int bnxt_re_netdev_event(struct notifier_block *notifier,
				unsigned long event, void *ptr)
{
	struct netdev_bonding_info *netdev_binfo = NULL;
	struct net_device *real_dev, *netdev;
	struct bnxt_re_dev *rdev = NULL;
#ifdef HAVE_NETDEV_BONDING_INFO
	struct netdev_notifier_bonding_info *notifier_info = ptr;
	bool do_lag = false;
	ifbond *master;
	ifslave *slave;
#endif
	int rc = 0;

	netdev = netdev_notifier_info_to_dev(ptr);
	real_dev = rdma_vlan_dev_real_dev(netdev);
	if (!real_dev)
		real_dev = netdev;
	/* In case of bonding,this will be bond's rdev */
	rdev = bnxt_re_from_netdev(real_dev);
	if (rdev) {
		bnxt_re_hold(rdev);
		if (rdev->binfo) {
			if (!test_bit(BNXT_RE_FLAG_IBDEV_REGISTERED,
				      &rdev->flags))
				goto done;
			if (rdev->binfo->slave1 == real_dev ||
			    rdev->binfo->slave2 == real_dev) {
				dev_dbg(rdev_to_dev(rdev),
					"Event %#lx on slave interface = %p\n",
					event, rdev);
				/*
				 * Handle NETDEV_CHANGE event,
				 * to update PFC/ECN settings on BONO
				 * without a shut/noshut
				 */
				if (event == NETDEV_CHANGE)
					goto handle_event;
#ifdef HAVE_NETDEV_BONDING_INFO
				if (event != NETDEV_BONDING_INFO) {
					dev_dbg(rdev_to_dev(rdev),
						"Ignoring event real_dev = %p"
						"master=%p slave1=%p slave2=%p\n",
						real_dev, rdev->binfo->master,
						rdev->binfo->slave1,
						rdev->binfo->slave2);
					goto done;
				} else {
					dev_dbg(rdev_to_dev(rdev),
						"Handle event real_dev = %p"
						"master=%p slave1=%p slave2=%p\n",
						real_dev, rdev->binfo->master,
						rdev->binfo->slave1,
						rdev->binfo->slave2);
					netdev_binfo = &notifier_info->bonding_info;
					goto handle_event;
				}
#endif /* HAVE_NETDEV_BONDING_INFO */
			}
		}
	} else {
		if (event != NETDEV_REGISTER) {
#ifndef HAVE_NETDEV_BONDING_INFO
			if (event == NETDEV_BONDING_FAILOVER ||
			    (netif_is_bond_master(real_dev) && event == NETDEV_UP)) {
					dev_dbg(NULL, "Received bonding failover\n");
				goto handle_event;
			}
#endif /* HAVE_NETDEV_BONDING_INFO */
			dev_dbg(NULL,
				"%s: NETDEV_%s (netdev %p) doesn't belong event = %lu",
				ROCE_DRV_MODULE_NAME, bnxt_re_netevent(event),
				real_dev, event);
			dev_dbg(NULL, "%s: to bnxt_re", ROCE_DRV_MODULE_NAME);
			goto exit;
		}
	}

	if (real_dev != netdev) {
		switch (event) {
		case NETDEV_UP:
			bnxt_re_schedule_work(rdev, event, netdev, NULL, NULL,
					      NULL);
			break;
		case NETDEV_DOWN:
#ifndef STACK_MANAGES_GID
			if (test_bit(BNXT_RE_FLAG_IBDEV_REGISTERED,
				     &rdev->flags)) {
				bnxt_qplib_netdev_del_gid(
					&rdev->qplib_res.sgid_tbl, netdev);
				bnxt_re_dispatch_event(&rdev->ibdev, NULL, 1,
						       IB_EVENT_GID_CHANGE);
			}
#endif
			break;
		default:
			break;
		}
		goto done;
	}
handle_event:
	switch (event) {
	case NETDEV_CHANGEADDR:
		/* MAC addr change event */
		if (!_is_chip_gen_p5(rdev->chip_ctx))
			bnxt_re_update_shadow_ah(rdev);
		bnxt_qplib_get_guid(rdev->netdev->dev_addr,
				    (u8 *)&rdev->ibdev.node_guid);
		break;

	case NETDEV_CHANGE:
		/*
		 * In bonding case bono requires to
		 * send this HWRM after PFC/ECN config
		 * avoding a shut/noshut workaround
		 * For other case handle as usual
		 */
		if (rdev->binfo) {
			u8 active_port_map;
			/* Update the latest link information */
			active_port_map = bnxt_re_get_bond_link_status(rdev->binfo);
			if (active_port_map) {
				rdev->binfo->active_port_map = active_port_map;
				bnxt_re_update_fw_lag_info(rdev->binfo, rdev, true);
			}
		}
		bnxt_re_get_link_speed(rdev);
		bnxt_re_schedule_work(rdev, event, NULL, NULL, NULL, NULL);
		goto exit;

	case NETDEV_CHANGEMTU:
		/*
		 * In bonding case bono requires to
		 * resend this HWRM after MTU change
		 */
		if (rdev && rdev->binfo && netif_is_bond_master(real_dev))
			bnxt_re_update_fw_lag_info(rdev->binfo, rdev, true);
		goto done;

	case NETDEV_REGISTER:
		if (rdev) {
			dev_dbg(rdev_to_dev(rdev),
				"NETDEV_%s (netdev %p), is already registered",
				 bnxt_re_netevent(event), real_dev);
			break;
		}

#ifdef HAVE_NETDEV_BONDING_INFO
		if (netif_is_bond_master(real_dev) &&
		    (!rdev || (rdev && !rdev->binfo)) && !delayed_bond_task_scheduled) {
			bnxt_re_schedule_work(NULL, event, NULL,
					      NULL, NULL, netdev);
			delayed_bond_task_scheduled = true;
			break;
		}
#endif
		/* Do not proceed further for non-brcm netdevs */
		if (!bnxt_re_is_brcm_l2(real_dev))
			goto exit;
		rc = bnxt_re_add_device(&rdev, real_dev, NULL,
					BNXT_RE_GSI_MODE_ALL,
					BNXT_RE_COMPLETE_INIT,
					BNXT_QPLIB_WQE_MODE_STATIC);
		if (rc)
			break;
		bnxt_re_get_link_speed(rdev);
		/* Schedule the bnxt_re_task to perform registration
		 * with IB stack.
		 */
		bnxt_re_schedule_work(rdev, event, NULL, NULL, NULL, NULL);
		break;

	case NETDEV_UNREGISTER:
		/* netdev notifier will call NETDEV_UNREGISTER again later since
		 * we are still holding the reference to the netdev
		 */

		/*
		 *  Workaround to avoid ib_unregister hang. Check for module
		 *  reference and dont free up the device if the reference
		 *  is non zero. Checking only for PF functions.
		 */

		if (rdev && !rdev->is_virtfn && module_refcount(THIS_MODULE) >  0) {
			dev_info(rdev_to_dev(rdev),
				 "bnxt_re:Unreg recvd when module refcnt > 0");
			dev_info(rdev_to_dev(rdev),
				 "bnxt_re:Close all apps using bnxt_re devs");
			dev_info(rdev_to_dev(rdev),
				 "bnxt_re:Refer documentation for details");
			goto done;
		}

		if (atomic_read(&rdev->sched_count) > 0)
			goto done;
		if (!rdev->unreg_sched) {
			bnxt_re_schedule_work(rdev, NETDEV_UNREGISTER,
					      NULL, NULL, NULL, NULL);
			rdev->unreg_sched = true;
			goto done;
		}

		goto exit;

#ifdef HAVE_NETDEV_BONDING_INFO
	case NETDEV_BONDING_INFO:

		netdev_binfo = &notifier_info->bonding_info;
		master = &netdev_binfo->master;
		slave = &netdev_binfo->slave;

		dev_info(NULL, "Bonding Info Received: rdev: %p\n", rdev);
		dev_info(NULL, "\tMaster: mode: %d num_slaves:%d\n",
			 master->bond_mode, master->num_slaves);
		dev_info(NULL, "\tSlave: id: %d name:%s link:%d state:%d\n",
			 slave->slave_id, slave->slave_name,
			 slave->link, slave->state);

		if (rdev->binfo) {
			/* Change in bond state */
			dev_dbg(rdev_to_dev(rdev),
				"Change in Bond state rdev = %p\n", rdev);
			do_lag = false;
			if (master->num_slaves != 2) {
				if (atomic_read(&rdev->sched_count) > 0)
					goto done;
				/* Need to decrement the rdev ref count before
				 * destroying as the bnxt_re_dev_hold incremented the
				 * same in the begining of this function
				 */
				bnxt_re_put(rdev);
				bnxt_re_destroy_lag(&rdev);
				goto done;
			} else {
				if (slave->link == BOND_LINK_UP) {
					dev_dbg(rdev_to_dev(rdev),
						"LAG: Skip link up\n");
					dev_dbg(rdev_to_dev(rdev),
						"LAG: Handle from worker\n");
					goto done;
				}
				dev_dbg(rdev_to_dev(rdev),"Updating lag device\n");
				if (bnxt_re_bond_update_reqd
				    (netdev_binfo, rdev->binfo, rdev, real_dev))
					bnxt_re_update_fw_lag_info
						(rdev->binfo, rdev, true);
				goto done;
			}
		} else {
		/* Check BOND needs to be created or not  rdev will be valid
		 * Pass gsi_mode invalid, we want to inherit from PF0
		 */
			rc = bnxt_re_create_lag(master, slave,
					netdev_binfo, real_dev, &rdev,
					BNXT_RE_GSI_MODE_INVALID,
					BNXT_QPLIB_WQE_MODE_INVALID);
			if (rc == -EINVAL)
				break;
			if (rc == -ENOMEM || rc == -EIO)
				goto done;
		}
		break;
#else /* Older kernels */
	case NETDEV_UP:
	case NETDEV_DOWN:
		/*
		 * Active - Active mode is created based on NETDEV_UP
		 * event. After adding devices to BOND, when bond
		 * device is opened, this event is received and we
		 * proceed with creating RoCE bond.
		 * NETDEV_DOWN is issued when ifconfig down is invoked.
		 * Need to update the link state if the device is in
		 * bond.
		 */
		if (!netif_is_bond_master(real_dev)) {
			if (!(rdev && rdev->binfo)) {
				bnxt_re_schedule_work(rdev, event, NULL,
						      NULL, NULL, NULL);
				break;
			}
		}
		dev_dbg(rdev_to_dev(rdev),
			"NETDEV UP or DOWN received\n");
		/* Fall through */

	case NETDEV_BONDING_FAILOVER:
		/* Update lag if needed */
		if (rdev) {
			rc = bnxt_re_update_lag(rdev);
			if (!rc || rc == -ENOMEM || rc == -EEXIST)
				goto done;
			if (rc == -EACCES)
				goto destroy_lag;
		}
		/* Create the Bond device */
		rc = bnxt_re_create_lag(netdev, &rdev,
					BNXT_RE_GSI_MODE_INVALID,
					BNXT_QPLIB_WQE_MODE_INVALID);
		if (rc == EAGAIN) {
			if (!rdev && netif_is_bond_master(real_dev)) {
				/*
				 * If driver gets loaded after L2 bond is created,
				 * lag allowed check returns false as the bnxt_re
				 * devices are getting created. Offload this check
				 * to the task and check again
				 */
				/* Dont schedule again if it is already scheduled */
				if (!delayed_bond_task_scheduled) {
					dev_info(rdev_to_dev(rdev),
						 "Bond event not handled.. Scheduling...\n");
					bnxt_re_schedule_work(rdev, event, NULL,
							      NULL, NULL, netdev);
					delayed_bond_task_scheduled = true;
				}
			}
			goto done;
		}
		if (rc)
			goto done;
		break;
	case NETDEV_RELEASE:
		if (!rdev ||  !rdev->binfo) {
			dev_dbg(rdev_to_dev(rdev),
				"FEAT_CHANGE/RELEASE recvd - rdev/binfo NULL");
			break;
		}
		dev_dbg(rdev_to_dev(rdev),
			"FEAT_CHANGE or RELEASE received rdev = %p\n", rdev);
		netdev_binfo = kzalloc(sizeof(*netdev_binfo), GFP_KERNEL);
		if (!netdev_binfo)
			break;
		/* Update the Bond */
		if (bnxt_re_get_bonding_info(rdev->binfo, real_dev,
					     netdev_binfo))
			break;
destroy_lag:
		if (atomic_read(&rdev->sched_count) > 0)
			goto done;
		rc = bnxt_re_destroy_lag(&rdev);
		break;
#endif /* HAVE_NETDEV_BONDING_INFO */

	default:
		bnxt_re_schedule_work(rdev, event, NULL, NULL, NULL, NULL);
		break;
	}
done:
	if (rdev)
		bnxt_re_put(rdev);
exit:
	netdev_dbg(netdev, "%s: Event = NETDEV_%s rdev = %p netdev = %p\n",__func__, bnxt_re_netevent(event), rdev, netdev);
#ifndef HAVE_NETDEV_BONDING_INFO
	if (netdev_binfo) {
		kfree(netdev_binfo);
		netdev_binfo = NULL;
	}
#endif /* HAVE_NETDEV_BONDING_INFO*/
	return NOTIFY_DONE;
}

static struct notifier_block bnxt_re_netdev_notifier = {
	.notifier_call = bnxt_re_netdev_event
};

#ifndef STACK_MANAGES_GID
static int bnxt_re_addr_event(struct net_device *netdev, unsigned long event,
			      union ib_gid *gid)
{
	struct bnxt_re_dev *rdev;
	struct net_device *real_dev;
	u16 vlan_id = 0xFFFF;
	u32 idx;
	int rc = 0;

	real_dev = rdma_vlan_dev_real_dev(netdev);
	if (real_dev)
		vlan_id = vlan_dev_vlan_id(netdev);
	else
		real_dev = netdev;

	rdev = bnxt_re_from_netdev(real_dev);
	if (rdev) {
		bnxt_re_hold(rdev);
	} else {
		dev_dbg(rdev_to_dev(rdev),
			"NETDEV_%s (inet %p) doesn't belong to bnxt_re",
			bnxt_re_netevent(event), real_dev);
		goto done;
	}
	if (!test_bit(BNXT_RE_FLAG_IBDEV_REGISTERED, &rdev->flags)) {
		dev_dbg(rdev_to_dev(rdev),
			"NETDEV_%s (inet %p) IBDEV is not registered",
			bnxt_re_netevent(event), real_dev);
		goto fail;
	}
	dev_err(rdev_to_dev(rdev), "NETDEV_%s (inet %p/%p) addr event recv'd",
		bnxt_re_netevent(event), netdev, real_dev);
	switch (event) {
	case NETDEV_UP:
		rc = bnxt_qplib_add_sgid(&rdev->qplib_res.sgid_tbl,
					 (struct bnxt_qplib_gid *)gid,
					 netdev->dev_addr, vlan_id, true, &idx);
		if (rc) {
			dev_dbg(rdev_to_dev(rdev),
				"NETDEV_%s (inet %p) failed to add sgid",
				bnxt_re_netevent(event), real_dev);
			goto fail;
		}
		break;
	case NETDEV_DOWN:
		/* On older distros where stack does not manage the GIDs, we
		 * get an inet6_addr event corresponding to the IPV6/link local
		 * address when the interface is brought down.
		 * HW requires that the GID 0 entry (GID corresponding to link
		 * local address) be retained as long as the QP1 QP is present.
		 */
		if (rdma_link_local_addr((struct in6_addr *)gid))
			goto fail;
		rc = bnxt_qplib_del_sgid(&rdev->qplib_res.sgid_tbl,
					 (struct bnxt_qplib_gid *)gid,
					 vlan_id, true);
		if (rc) {
			dev_dbg(rdev_to_dev(rdev),
				"NETDEV_%s (inet %p) failed to del sgid",
				bnxt_re_netevent(event), real_dev);
			goto fail;
		}
		break;
	default:
		dev_err(rdev_to_dev(rdev),
			"NETDEV_%s (inet %p) is not supported",
			bnxt_re_netevent(event), real_dev);
		break;
	}
	/* Notify about GID table change */
	bnxt_re_dispatch_event(&rdev->ibdev, NULL, 1, IB_EVENT_GID_CHANGE);
	/* TODO: Need to tell Bono about SGID change?
	   If so, need to queue to sp wq. */
	/* Upon sp wq completion, notify the update SGID consumer if requested */
fail:
	bnxt_re_put(rdev);
done:
	return NOTIFY_DONE;
}

static int bnxt_re_inetaddr_event(struct notifier_block *notifier,
				  unsigned long event, void *ptr)
{
	struct in_ifaddr *ifa = ptr;
	struct net_device *netdev = ifa->ifa_dev->dev;
	union ib_gid gid;

	ipv6_addr_set_v4mapped(ifa->ifa_address, (struct in6_addr *)&gid);

	return bnxt_re_addr_event(netdev, event, &gid);
}

static struct notifier_block bnxt_re_inetaddr_notifier = {
	.notifier_call = bnxt_re_inetaddr_event
};

#if IS_ENABLED(CONFIG_IPV6)
static int bnxt_re_inet6addr_event(struct notifier_block *notifier,
				   unsigned long event, void *ptr)
{
	struct inet6_ifaddr *ifa = ptr;
	union ib_gid *gid = (union ib_gid *)&ifa->addr;
	struct net_device *netdev = ifa->idev->dev;

	return bnxt_re_addr_event(netdev, event, gid);
}

static struct notifier_block bnxt_re_inet6addr_notifier = {
	.notifier_call = bnxt_re_inet6addr_event
};
#endif
#endif

static int __init bnxt_re_mod_init(void)
{
	int rc = 0;

	pr_info("%s: %s", ROCE_DRV_MODULE_NAME, version);

	bnxt_re_wq = create_singlethread_workqueue("bnxt_re");
	if (!bnxt_re_wq)
		return -ENOMEM;

#ifdef ENABLE_DEBUGFS
	bnxt_re_debugfs_init();
#endif
#ifdef HAVE_CONFIGFS_ENABLED
	bnxt_re_configfs_init();
#endif
	INIT_LIST_HEAD(&bnxt_re_dev_list);
	INIT_LIST_HEAD(&bnxt_re_bond_list);
	adapter_count = 0;

	rc = bnxt_re_register_netdevice_notifier(&bnxt_re_netdev_notifier);
	if (rc) {
		dev_err(NULL, "%s: Cannot register to netdevice_notifier",
			ROCE_DRV_MODULE_NAME);
		goto err_netdev;
	}
#ifndef STACK_MANAGES_GID
	/* Register to the inet/6 notifier to add/del SGID entries */
	rc = register_inetaddr_notifier(&bnxt_re_inetaddr_notifier);
	if (rc) {
		dev_err(NULL, "%s: Cannot register to inetaddr_notifier",
			ROCE_DRV_MODULE_NAME);
		goto err_inetaddr;
	}
#if IS_ENABLED(CONFIG_IPV6)
	rc = register_inet6addr_notifier(&bnxt_re_inet6addr_notifier);
	if (rc) {
		dev_err(NULL, "%s: Cannot register to inet6addr_notifier",
			ROCE_DRV_MODULE_NAME);
		goto err_inet6addr;
	}
#endif
#endif
	return 0;

#ifndef STACK_MANAGES_GID
err_inet6addr:
	unregister_inetaddr_notifier(&bnxt_re_inetaddr_notifier);
err_inetaddr:
	bnxt_re_unregister_netdevice_notifier(&bnxt_re_netdev_notifier);
#endif
err_netdev:

#ifdef HAVE_CONFIGFS_ENABLED
	bnxt_re_configfs_exit();
#endif
#ifdef ENABLE_DEBUGFS
	bnxt_re_debugfs_remove();
#endif
	destroy_workqueue(bnxt_re_wq);

	return rc;
}

static void __exit bnxt_re_mod_exit(void)
{
	struct bnxt_re_dev *rdev, *next;
	unsigned int new_devs = 0;

	/* Free all adapter allocated resources */
	do {
		mutex_lock(&bnxt_re_dev_lock);
		gmod_exit = 1;
		/* Take a note of devices being added */
		new_devs = gadd_dev_inprogress;
		mutex_unlock(&bnxt_re_dev_lock);
		/*
		 * Can use the new list without protection.
		 * Cleanup the devices in reverse order so that the VF device
		 * cleanup is done before PF cleanup
		 */
		list_for_each_entry_safe_reverse(rdev, next, &bnxt_re_dev_list, list) {
			dev_dbg(rdev_to_dev(rdev), "Unregistering Device");
			/*
			 *  Flush out any scheduled tasks before destroying the
			 *  resources
			 */
			flush_workqueue(bnxt_re_wq);
			bnxt_re_stopqps_and_ib_uninit(rdev);
			/* rtnl_lock needs to be held to call into the bnxt_en APIs
			 *  All other invocations of bnxt_re_remove_device() are in
			 *  contexts where rtnl lock is already hed
			 */
			rtnl_lock();
			if (!bnxt_re_is_rdev_valid(rdev)) {
				rtnl_unlock();
				continue;
			}
			bnxt_re_remove_device(rdev, BNXT_RE_COMPLETE_REMOVE);
			rtnl_unlock();
		}
	} while (new_devs);


#ifndef STACK_MANAGES_GID
#if IS_ENABLED(CONFIG_IPV6)
	unregister_inet6addr_notifier(&bnxt_re_inet6addr_notifier);
#endif
	unregister_inetaddr_notifier(&bnxt_re_inetaddr_notifier);
#endif
	bnxt_re_unregister_netdevice_notifier(&bnxt_re_netdev_notifier);

	if (adapter_count)
		dev_warn(NULL, "%s: Mod exit failed to free all adapters!",
			 ROCE_DRV_MODULE_NAME);
#ifdef HAVE_CONFIGFS_ENABLED
	bnxt_re_configfs_exit();
#endif
#ifdef ENABLE_DEBUGFS
	bnxt_re_debugfs_remove();
#endif
	if (bnxt_re_wq)
		destroy_workqueue(bnxt_re_wq);
}

module_init(bnxt_re_mod_init);
module_exit(bnxt_re_mod_exit);
